@@ -19,7 +19,7 @@ namespace roles {
 
 enum Role {
   /**
-   * Used when accessible hans't strong defined role.
+   * Used when the accessible has no strongly-defined role.
    */
   NOTHING = 0,
 
@@ -845,11 +845,6 @@ enum Role {
    */
   MATHML_ROOT = 141,
 
-  /**
-   * A MathML fenced element (mfenced in MathML).
-   */
-  MATHML_FENCED = 142,
-
   /**
    * A MathML enclosed element (menclose in MathML).
    */
@@ -1066,7 +1061,38 @@ enum Role {
    */
   CODE = 182,
 
-  LAST_ROLE = CODE
+  /**
+   * Represents control whose purpose is to allow a user to edit a time.
+   */
+  TIME_EDITOR = 183,
+
+  /**
+   * Represents the marker associated with a list item. In unordered lists,
+   * this is a bullet, while in ordered lists this is a number.
+   */
+  LISTITEM_MARKER = 184,
+
+  /**
+   * Essentially, this is a progress bar with a contextually defined
+   * scale, ex. the strength of a password entered in an input.
+   */
+  METER = 185,
+
+  /**
+   * Represents phrasing content that is presented with vertical alignment
+   * lower than the baseline and a smaller font size. For example, the "2" in
+   * the chemical formula H2O.
+   */
+  SUBSCRIPT = 186,
+
+  /**
+   * Represents phrasing content that is presented with vertical alignment
+   * higher than the baseline and a smaller font size. For example, the
+   * exponent in a math expression.
+   */
+  SUPERSCRIPT = 187,
+
+  LAST_ROLE = SUPERSCRIPT
 };
 
 }  // namespace roles
