@@ -11,11 +11,8 @@
 #include "ia2AccessibleTable.h"
 #include "ia2AccessibleTableCell.h"
 #include "mozilla/a11y/AccessibleWrap.h"
+#include "mozilla/a11y/Compatibility.h"
 #include "mozilla/a11y/DocAccessibleParent.h"
-#include "mozilla/dom/BrowserBridgeParent.h"
-#include "mozilla/dom/BrowserParent.h"
-#include "mozilla/mscom/Interceptor.h"
-#include "mozilla/StaticPrefs_accessibility.h"
 #include "MsaaAccessible.h"
 #include "MsaaDocAccessible.h"
 #include "MsaaRootAccessible.h"
@@ -26,6 +23,11 @@
 #include "Relation.h"
 #include "sdnAccessible.h"
 #include "sdnTextAccessible.h"
+#include "HyperTextAccessible-inl.h"
+#include "ServiceProvider.h"
+#include "Statistics.h"
+#include "ARIAMap.h"
+#include "mozilla/PresShell.h"
 
 using namespace mozilla;
 using namespace mozilla::a11y;
@@ -38,41 +40,40 @@ ITypeInfo* MsaaAccessible::gTypeInfo = nullptr;
 
 /* static */
 MsaaAccessible* MsaaAccessible::Create(Accessible* aAcc) {
-  // If the cache is enabled, this should only ever be called in the parent
-  // process.
-  MOZ_ASSERT(!StaticPrefs::accessibility_cache_enabled_AtStartup() ||
-             XRE_IsParentProcess());
+  // This should only ever be called in the parent process.
+  MOZ_ASSERT(XRE_IsParentProcess());
   // The order of some of these is important! For example, when isRoot is true,
   // IsDoc will also be true, so we must check IsRoot first. IsTable/Cell and
   // IsHyperText are a similar case.
-  if (aAcc->IsLocal() && aAcc->IsRoot()) {
+  if (aAcc->IsRoot()) {
+    MOZ_ASSERT(aAcc->IsLocal());
     return new MsaaRootAccessible(aAcc);
   }
   if (aAcc->IsDoc()) {
     return new MsaaDocAccessible(aAcc);
   }
-  if (aAcc->IsLocal()) {
-    // XXX These classes don't support RemoteAccessible yet.
-    if (aAcc->IsTable()) {
-      return new ia2AccessibleTable(aAcc);
-    }
-    if (aAcc->IsTableCell()) {
-      return new ia2AccessibleTableCell(aAcc);
-    }
-    if (aAcc->IsApplication()) {
-      return new ia2AccessibleApplication(aAcc);
-    }
-    if (aAcc->IsHyperText()) {
-      return new ia2AccessibleHypertext(aAcc);
-    }
-    if (aAcc->IsImage()) {
-      return new ia2AccessibleImage(aAcc);
-    }
-    if (aAcc->AsLocal()->GetContent() &&
-        aAcc->AsLocal()->GetContent()->IsXULElement(nsGkAtoms::menuitem)) {
+  if (aAcc->IsTable()) {
+    return new ia2AccessibleTable(aAcc);
+  }
+  if (aAcc->IsTableCell()) {
+    return new ia2AccessibleTableCell(aAcc);
+  }
+  if (aAcc->IsApplication()) {
+    MOZ_ASSERT(aAcc->IsLocal());
+    return new ia2AccessibleApplication(aAcc);
+  }
+  if (aAcc->IsImage()) {
+    return new ia2AccessibleImage(aAcc);
+  }
+  if (LocalAccessible* localAcc = aAcc->AsLocal()) {
+    if (localAcc->GetContent() &&
+        localAcc->GetContent()->IsXULElement(nsGkAtoms::menuitem)) {
       return new MsaaXULMenuitemAccessible(aAcc);
     }
   }
+  if (aAcc->IsHyperText()) {
+    return new ia2AccessibleHypertext(aAcc);
+  }
   return new MsaaAccessible(aAcc);
 }
 
@@ -94,48 +95,15 @@ void MsaaAccessible::MsaaShutdown() {
     return;
   }
 
-  if (mAcc->IsProxy()) {
-    // For RemoteAccessibleWrap, we just need to clear mAcc.
-    mAcc = nullptr;
-    return;
-  }
-
   if (mID != kNoID) {
     auto doc = MsaaDocAccessible::GetFromOwned(mAcc);
     MOZ_ASSERT(doc);
     doc->RemoveID(mID);
   }
 
-  if (XRE_IsContentProcess()) {
-    // Bug 1434822: To improve performance for cross-process COM, we disable COM
-    // garbage collection. However, this means we never receive Release calls
-    // from clients, so defunct accessibles can never be deleted. Since we
-    // know when an accessible is shutting down, we can work around this by
-    // forcing COM to disconnect this object from all of its remote clients,
-    // which will cause associated references to be released.
-    IUnknown* unk = static_cast<IAccessible*>(this);
-    mscom::Interceptor::DisconnectRemotesForTarget(unk);
-    // If an accessible was retrieved via IAccessibleHypertext::hyperlink*,
-    // it will have a different Interceptor that won't be matched by the above
-    // call, even though it's the same object. Therefore, call it again with
-    // the IAccessibleHyperlink pointer. We can remove this horrible hack once
-    // bug 1440267 is fixed.
-    unk = static_cast<IAccessibleHyperlink*>(this);
-    mscom::Interceptor::DisconnectRemotesForTarget(unk);
-    for (auto& assocUnk : mAssociatedCOMObjectsForDisconnection) {
-      mscom::Interceptor::DisconnectRemotesForTarget(assocUnk);
-    }
-    mAssociatedCOMObjectsForDisconnection.Clear();
-  }
-
   mAcc = nullptr;
 }
 
-void MsaaAccessible::SetID(uint32_t aID) {
-  MOZ_ASSERT(XRE_IsParentProcess() && mAcc && mAcc->IsProxy());
-  mID = aID;
-}
-
 int32_t MsaaAccessible::GetChildIDFor(Accessible* aAccessible) {
   // A child ID of the window is required, when we use NotifyWinEvent,
   // so that the 3rd party application can call back and get the IAccessible
@@ -145,13 +113,6 @@ int32_t MsaaAccessible::GetChildIDFor(Accessible* aAccessible) {
     return 0;
   }
 
-  // Chrome should use mID which has been generated by the content process.
-  if (aAccessible->IsProxy()) {
-    const uint32_t id = MsaaAccessible::GetFrom(aAccessible)->mID;
-    MOZ_ASSERT(id != kNoID);
-    return id;
-  }
-
   auto doc = MsaaDocAccessible::GetFromOwned(aAccessible);
   if (!doc) {
     return 0;
@@ -166,21 +127,6 @@ int32_t MsaaAccessible::GetChildIDFor(Accessible* aAccessible) {
   return *id;
 }
 
-/* static */
-uint32_t MsaaAccessible::GetContentProcessIdFor(
-    dom::ContentParentId aIPCContentId) {
-  if (StaticPrefs::accessibility_cache_enabled_AtStartup()) {
-    return 0;
-  }
-  return sIDGen.GetContentProcessIDFor(aIPCContentId);
-}
-
-/* static */
-void MsaaAccessible::ReleaseContentProcessIdFor(
-    dom::ContentParentId aIPCContentId) {
-  sIDGen.ReleaseContentProcessIDFor(aIPCContentId);
-}
-
 /* static */
 void MsaaAccessible::AssignChildIDTo(NotNull<sdnAccessible*> aSdnAcc) {
   aSdnAcc->SetUniqueID(sIDGen.GetID());
@@ -197,9 +143,8 @@ HWND MsaaAccessible::GetHWNDFor(Accessible* aAccessible) {
   }
 
   LocalAccessible* localAcc = aAccessible->AsLocal();
-  if (!localAcc || localAcc->IsProxy()) {
-    RemoteAccessible* proxy =
-        localAcc ? localAcc->Proxy() : aAccessible->AsRemote();
+  if (!localAcc) {
+    RemoteAccessible* proxy = aAccessible->AsRemote();
     if (!proxy) {
       return nullptr;
     }
@@ -242,8 +187,7 @@ HWND MsaaAccessible::GetHWNDFor(Accessible* aAccessible) {
     nsIWidget* widget = frame->GetNearestWidget();
     if (widget && widget->IsVisible()) {
       if (nsViewManager* vm = document->PresShellPtr()->GetViewManager()) {
-        nsCOMPtr<nsIWidget> rootWidget;
-        vm->GetRootWidget(getter_AddRefs(rootWidget));
+        nsCOMPtr<nsIWidget> rootWidget = vm->GetRootWidget();
         // Make sure the accessible belongs to popup. If not then use
         // document HWND (which might be different from root widget in the
         // case of window emulation).
@@ -256,69 +200,34 @@ HWND MsaaAccessible::GetHWNDFor(Accessible* aAccessible) {
   return static_cast<HWND>(document->GetNativeWindow());
 }
 
-static bool IsHandlerInvalidationNeeded(uint32_t aEvent) {
-  // We want to return true for any events that would indicate that something
-  // in the handler cache is out of date.
-  switch (aEvent) {
-    case EVENT_OBJECT_STATECHANGE:
-    case EVENT_OBJECT_LOCATIONCHANGE:
-    case EVENT_OBJECT_NAMECHANGE:
-    case EVENT_OBJECT_DESCRIPTIONCHANGE:
-    case EVENT_OBJECT_VALUECHANGE:
-    case EVENT_OBJECT_FOCUS:
-    case IA2_EVENT_ACTION_CHANGED:
-    case IA2_EVENT_DOCUMENT_LOAD_COMPLETE:
-    case IA2_EVENT_DOCUMENT_LOAD_STOPPED:
-    case IA2_EVENT_DOCUMENT_ATTRIBUTE_CHANGED:
-    case IA2_EVENT_DOCUMENT_CONTENT_CHANGED:
-    case IA2_EVENT_PAGE_CHANGED:
-    case IA2_EVENT_TEXT_ATTRIBUTE_CHANGED:
-    case IA2_EVENT_TEXT_CHANGED:
-    case IA2_EVENT_TEXT_INSERTED:
-    case IA2_EVENT_TEXT_REMOVED:
-    case IA2_EVENT_TEXT_UPDATED:
-    case IA2_EVENT_OBJECT_ATTRIBUTE_CHANGED:
-      return true;
-    default:
-      return false;
-  }
-}
-
 void MsaaAccessible::FireWinEvent(Accessible* aTarget, uint32_t aEventType) {
   MOZ_ASSERT(XRE_IsParentProcess());
   static_assert(sizeof(gWinEventMap) / sizeof(gWinEventMap[0]) ==
                     nsIAccessibleEvent::EVENT_LAST_ENTRY,
                 "MSAA event map skewed");
 
-  NS_ASSERTION(aEventType > 0 && aEventType < ArrayLength(gWinEventMap),
-               "invalid event type");
+  if (aEventType == 0 || aEventType >= ArrayLength(gWinEventMap)) {
+    MOZ_ASSERT_UNREACHABLE("invalid event type");
+    return;
+  }
 
   uint32_t winEvent = gWinEventMap[aEventType];
   if (!winEvent) return;
 
-  Accessible* target = aTarget;
-  if (!StaticPrefs::accessibility_cache_enabled_AtStartup() &&
-      target->IsRemote()) {
-    target = WrapperFor(target->AsRemote());
-  }
-  int32_t childID = MsaaAccessible::GetChildIDFor(target);
+  int32_t childID = MsaaAccessible::GetChildIDFor(aTarget);
   if (!childID) return;  // Can't fire an event without a child ID
 
-  HWND hwnd = GetHWNDFor(target);
+  HWND hwnd = GetHWNDFor(aTarget);
   if (!hwnd) {
     return;
   }
 
-  if (IsHandlerInvalidationNeeded(winEvent)) {
-    AccessibleWrap::InvalidateHandlers();
-  }
-
   // Fire MSAA event for client area window.
   ::NotifyWinEvent(winEvent, hwnd, OBJID_CLIENT, childID);
 }
 
 AccessibleWrap* MsaaAccessible::LocalAcc() {
-  if (!mAcc || mAcc->IsRemote() || mAcc->IsProxy()) {
+  if (!mAcc || mAcc->IsRemote()) {
     return nullptr;
   }
   auto acc = static_cast<AccessibleWrap*>(mAcc);
@@ -414,48 +323,14 @@ static Accessible* GetAccessibleInSubtree(DocAccessibleParent* aDoc,
   return nullptr;
 }
 
-static already_AddRefed<IDispatch> GetProxiedAccessibleInSubtree(
-    const DocAccessibleParent* aDoc, const VARIANT& aVarChild) {
-  MOZ_ASSERT(!StaticPrefs::accessibility_cache_enabled_AtStartup());
-  auto wrapper = static_cast<DocRemoteAccessibleWrap*>(WrapperFor(aDoc));
-  RefPtr<IAccessible> comProxy;
-  int32_t docWrapperChildId = MsaaAccessible::GetChildIDFor(wrapper);
-  // Only document accessible proxies at the top level of their content process
-  // are created with a pointer to their COM proxy.
-  if (aDoc->IsTopLevelInContentProcess()) {
-    wrapper->GetNativeInterface(getter_AddRefs(comProxy));
-  } else {
-    auto tab = static_cast<dom::BrowserParent*>(aDoc->Manager());
-    MOZ_ASSERT(tab);
-    DocAccessibleParent* topLevelDoc = tab->GetTopLevelDocAccessible();
-    MOZ_ASSERT(topLevelDoc && topLevelDoc->IsTopLevelInContentProcess());
-    VARIANT docId = {{{VT_I4}}};
-    docId.lVal = docWrapperChildId;
-    RefPtr<IDispatch> disp = GetProxiedAccessibleInSubtree(topLevelDoc, docId);
-    if (!disp) {
-      return nullptr;
+static bool IsInclusiveDescendantOf(DocAccessible* aAncestor,
+                                    DocAccessible* aDescendant) {
+  for (DocAccessible* doc = aDescendant; doc; doc = doc->ParentDocument()) {
+    if (doc == aAncestor) {
+      return true;
     }
-
-    DebugOnly<HRESULT> hr =
-        disp->QueryInterface(IID_IAccessible, getter_AddRefs(comProxy));
-    MOZ_ASSERT(SUCCEEDED(hr));
   }
-
-  MOZ_ASSERT(comProxy);
-  if (!comProxy) {
-    return nullptr;
-  }
-
-  if (docWrapperChildId == aVarChild.lVal) {
-    return comProxy.forget();
-  }
-
-  RefPtr<IDispatch> disp;
-  if (FAILED(comProxy->get_accChild(aVarChild, getter_AddRefs(disp)))) {
-    return nullptr;
-  }
-
-  return disp.forget();
+  return false;
 }
 
 already_AddRefed<IAccessible> MsaaAccessible::GetIAccessibleFor(
@@ -474,104 +349,39 @@ already_AddRefed<IAccessible> MsaaAccessible::GetIAccessibleFor(
     return nullptr;
   }
 
-  AccessibleWrap* localAcc = static_cast<AccessibleWrap*>(mAcc->AsLocal());
   if (varChild.lVal == CHILDID_SELF) {
-    MOZ_ASSERT(!localAcc || !localAcc->IsProxy());
     result = this;
     return result.forget();
   }
 
-  if (varChild.ulVal != GetExistingID() &&
-      ((localAcc && localAcc->IsProxy())
-           ? nsAccUtils::MustPrune(localAcc->Proxy())
-           : nsAccUtils::MustPrune(mAcc))) {
+  if (varChild.ulVal != GetExistingID() && nsAccUtils::MustPrune(mAcc)) {
     // This accessible should have no subtree in platform, return null for its
     // children.
     return nullptr;
   }
 
-  // If the MSAA ID is not a chrome id then we already know that we won't
-  // find it here and should look remotely instead. This handles the case when
-  // accessible is part of the chrome process and is part of the xul browser
-  // window and the child id points in the content documents. Thus we need to
-  // make sure that it is never called on proxies.
-  // Bug 1422674: We must only handle remote ids here (< 0), not child indices.
-  // Child indices (> 0) are handled below for both local and remote children.
-  if (XRE_IsParentProcess() && localAcc && !localAcc->IsProxy() &&
-      varChild.lVal < 0 && !sIDGen.IsChromeID(varChild.lVal)) {
-    MOZ_ASSERT(!StaticPrefs::accessibility_cache_enabled_AtStartup());
-    if (!localAcc->IsRootForHWND()) {
-      // Bug 1422201, 1424657: accChild with a remote id is only valid on the
-      // root accessible for an HWND.
-      // Otherwise, we might return remote accessibles which aren't descendants
-      // of this accessible. This would confuse clients which use accChild to
-      // check whether something is a descendant of a document.
-      return nullptr;
-    }
-    return GetRemoteIAccessibleFor(varChild);
-  }
-
   if (varChild.lVal > 0) {
     // Gecko child indices are 0-based in contrast to indices used in MSAA.
-    MOZ_ASSERT(!localAcc || !localAcc->IsProxy());
-    if (!StaticPrefs::accessibility_cache_enabled_AtStartup() &&
-        XRE_IsContentProcess() && localAcc->IsOuterDoc()) {
-      // With the cache disabled, we need to be able to return a COM proxy for
-      // an OOP iframe. The COM proxy is wrapped in a
-      // RemoteIframeDocRemoteAccessibleWrap, which can only be retrieved
-      // using the LocalAccessible hierarchy.
-      LocalAccessible* xpAcc = localAcc->LocalChildAt(varChild.lVal - 1);
-      if (!xpAcc) {
-        return nullptr;
-      }
-      xpAcc->GetNativeInterface(getter_AddRefs(result));
-      return result.forget();
-    }
     Accessible* xpAcc = mAcc->ChildAt(varChild.lVal - 1);
     if (!xpAcc) {
       return nullptr;
     }
     MOZ_ASSERT(xpAcc->IsRemote() || !xpAcc->AsLocal()->IsDefunct(),
                "Shouldn't get a defunct child");
-    if (!StaticPrefs::accessibility_cache_enabled_AtStartup() && localAcc &&
-        xpAcc->IsRemote()) {
-      // We're crossing from a local OuterDoc to a DocRemoteAccessibleWrap.
-      // We must return the COM proxy.
-      MOZ_ASSERT(localAcc->IsOuterDoc());
-      xpAcc->AsRemote()->GetCOMInterface(getter_AddRefs(result));
-      return result.forget();
-    }
     result = MsaaAccessible::GetFrom(xpAcc);
     return result.forget();
   }
 
   // If lVal negative then it is treated as child ID and we should look for
   // accessible through whole accessible subtree including subdocuments.
-  // First see about the case that both this accessible and the target one are
-  // RemoteAccessibleWraps.
-  if (localAcc && localAcc->IsProxy()) {
-    MOZ_ASSERT(!StaticPrefs::accessibility_cache_enabled_AtStartup());
-    DocAccessibleParent* proxyDoc = localAcc->Proxy()->Document();
-    RefPtr<IDispatch> disp = GetProxiedAccessibleInSubtree(proxyDoc, varChild);
-    if (!disp) {
-      return nullptr;
-    }
-
-    MOZ_ASSERT(mscom::IsProxy(disp));
-    DebugOnly<HRESULT> hr =
-        disp->QueryInterface(IID_IAccessible, getter_AddRefs(result));
-    MOZ_ASSERT(SUCCEEDED(hr));
-    return result.forget();
-  }
-
   Accessible* doc = nullptr;
   Accessible* child = nullptr;
   auto id = static_cast<uint32_t>(varChild.lVal);
-  if (localAcc) {
+  if (LocalAccessible* localAcc = mAcc->AsLocal()) {
     DocAccessible* localDoc = localAcc->Document();
     doc = localDoc;
     child = GetAccessibleInSubtree(localDoc, id);
-    if (!child && StaticPrefs::accessibility_cache_enabled_AtStartup()) {
+    if (!child) {
       // Search remote documents which are descendants of this local document.
       const auto remoteDocs = DocManager::TopLevelRemoteDocs();
       if (!remoteDocs) {
@@ -579,9 +389,8 @@ already_AddRefed<IAccessible> MsaaAccessible::GetIAccessibleFor(
       }
       for (DocAccessibleParent* remoteDoc : *remoteDocs) {
         LocalAccessible* outerDoc = remoteDoc->OuterDocOfRemoteBrowser();
-        if (!outerDoc || outerDoc->Document() != localDoc) {
-          // The OuterDoc isn't inside our document, so this isn't a
-          // descendant.
+        if (!outerDoc ||
+            !IsInclusiveDescendantOf(localDoc, outerDoc->Document())) {
           continue;
         }
         child = GetAccessibleInSubtree(remoteDoc, id);
@@ -590,7 +399,7 @@ already_AddRefed<IAccessible> MsaaAccessible::GetIAccessibleFor(
         }
       }
     }
-  } else if (StaticPrefs::accessibility_cache_enabled_AtStartup()) {
+  } else {
     DocAccessibleParent* remoteDoc = mAcc->AsRemote()->Document();
     doc = remoteDoc;
     child = GetAccessibleInSubtree(remoteDoc, id);
@@ -599,8 +408,6 @@ already_AddRefed<IAccessible> MsaaAccessible::GetIAccessibleFor(
     return nullptr;
   }
 
-  MOZ_ASSERT(child->IsLocal() ||
-             StaticPrefs::accessibility_cache_enabled_AtStartup());
   MOZ_ASSERT(child->IsRemote() || !child->AsLocal()->IsDefunct(),
              "Shouldn't get a defunct child");
   // If this method is being called on the document we searched, we can just
@@ -626,104 +433,6 @@ already_AddRefed<IAccessible> MsaaAccessible::GetIAccessibleFor(
   return nullptr;
 }
 
-/**
- * Visit DocAccessibleParent descendants of `aBrowser` that are at the top
- * level of their content process.
- * That is, IsTopLevelInContentProcess() will be true for each visited actor.
- * Each visited actor will be an embedded document in a different content
- * process to its embedder.
- * The DocAccessibleParent for `aBrowser` itself is excluded.
- * `aCallback` will be called for each DocAccessibleParent.
- * The callback should return true to continue traversal, false to cease.
- */
-template <typename Callback>
-static bool VisitDocAccessibleParentDescendantsAtTopLevelInContentProcess(
-    dom::BrowserParent* aBrowser, Callback aCallback) {
-  // We can't use BrowserBridgeParent::VisitAllDescendants because it doesn't
-  // provide a way to stop the search.
-  const auto& bridges = aBrowser->ManagedPBrowserBridgeParent();
-  return std::all_of(bridges.cbegin(), bridges.cend(), [&](const auto& key) {
-    auto* bridge = static_cast<dom::BrowserBridgeParent*>(key);
-    dom::BrowserParent* childBrowser = bridge->GetBrowserParent();
-    DocAccessibleParent* childDocAcc = childBrowser->GetTopLevelDocAccessible();
-    if (!childDocAcc || childDocAcc->IsShutdown()) {
-      return true;
-    }
-    if (!aCallback(childDocAcc)) {
-      return false;  // Stop traversal.
-    }
-    if (!VisitDocAccessibleParentDescendantsAtTopLevelInContentProcess(
-            childBrowser, aCallback)) {
-      return false;  // Stop traversal.
-    }
-    return true;  // Continue traversal.
-  });
-}
-
-already_AddRefed<IAccessible> MsaaAccessible::GetRemoteIAccessibleFor(
-    const VARIANT& aVarChild) {
-  a11y::RootAccessible* root = LocalAcc()->RootAccessible();
-  const nsTArray<DocAccessibleParent*>* remoteDocs =
-      DocManager::TopLevelRemoteDocs();
-  if (!remoteDocs) {
-    return nullptr;
-  }
-
-  RefPtr<IAccessible> result;
-
-  // We intentionally leave the call to remoteDocs->Length() inside the loop
-  // condition because it is possible for reentry to occur in the call to
-  // GetProxiedAccessibleInSubtree() such that remoteDocs->Length() is mutated.
-  for (size_t i = 0; i < remoteDocs->Length(); i++) {
-    DocAccessibleParent* topRemoteDoc = remoteDocs->ElementAt(i);
-
-    LocalAccessible* outerDoc = topRemoteDoc->OuterDocOfRemoteBrowser();
-    if (!outerDoc) {
-      continue;
-    }
-
-    if (outerDoc->RootAccessible() != root) {
-      continue;
-    }
-
-    RefPtr<IDispatch> disp;
-    auto checkDoc = [&aVarChild,
-                     &disp](DocAccessibleParent* aRemoteDoc) -> bool {
-      uint32_t remoteDocMsaaId =
-          WrapperFor(aRemoteDoc)->GetMsaa()->GetExistingID();
-      if (!sIDGen.IsSameContentProcessFor(aVarChild.lVal, remoteDocMsaaId)) {
-        return true;  // Continue the search.
-      }
-      if ((disp = GetProxiedAccessibleInSubtree(aRemoteDoc, aVarChild))) {
-        return false;  // Found it! Stop traversal!
-      }
-      return true;  // Continue the search.
-    };
-
-    // Check the top level document for this id.
-    checkDoc(topRemoteDoc);
-    if (!disp) {
-      // The top level document doesn't contain this id. Recursively check any
-      // out-of-process iframe documents it embeds.
-      VisitDocAccessibleParentDescendantsAtTopLevelInContentProcess(
-          static_cast<dom::BrowserParent*>(topRemoteDoc->Manager()), checkDoc);
-    }
-
-    if (!disp) {
-      continue;
-    }
-
-    DebugOnly<HRESULT> hr =
-        disp->QueryInterface(IID_IAccessible, getter_AddRefs(result));
-    // QI can fail on rare occasions if the LocalAccessible dies after we
-    // fetched disp but before we QI.
-    NS_WARNING_ASSERTION(SUCCEEDED(hr), "QI failed on remote IDispatch");
-    return result.forget();
-  }
-
-  return nullptr;
-}
-
 IDispatch* MsaaAccessible::NativeAccessible(Accessible* aAccessible) {
   if (!aAccessible) {
     NS_WARNING("Not passing in an aAccessible");
@@ -731,19 +440,7 @@ IDispatch* MsaaAccessible::NativeAccessible(Accessible* aAccessible) {
   }
 
   RefPtr<IDispatch> disp;
-  if (aAccessible->IsProxy()) {
-    MOZ_ASSERT(!StaticPrefs::accessibility_cache_enabled_AtStartup());
-    // This is a RemoteAccessibleWrap. We must use GetNativeInterface in this
-    // case so we return the COM proxy.
-    aAccessible->AsLocal()->GetNativeInterface(getter_AddRefs(disp));
-  } else if (!StaticPrefs::accessibility_cache_enabled_AtStartup() &&
-             aAccessible->IsRemote()) {
-    // This is a RemoteAccessible and caching isn't enabled. We must return
-    // the COM proxy.
-    aAccessible->AsRemote()->GetCOMInterface(getter_AddRefs(disp));
-  } else {
-    disp = MsaaAccessible::GetFrom(aAccessible);
-  }
+  disp = MsaaAccessible::GetFrom(aAccessible);
   IDispatch* rawDisp;
   disp.forget(&rawDisp);
   return rawDisp;
@@ -766,8 +463,11 @@ ITypeInfo* MsaaAccessible::GetTI(LCID lcid) {
 
 /* static */
 MsaaAccessible* MsaaAccessible::GetFrom(Accessible* aAcc) {
+  if (!aAcc) {
+    return nullptr;
+  }
+
   if (RemoteAccessible* remoteAcc = aAcc->AsRemote()) {
-    MOZ_ASSERT(StaticPrefs::accessibility_cache_enabled_AtStartup());
     return reinterpret_cast<MsaaAccessible*>(remoteAcc->GetWrapper());
   }
   return static_cast<AccessibleWrap*>(aAcc)->GetMsaa();
@@ -816,14 +516,14 @@ MsaaAccessible::QueryInterface(REFIID iid, void** ppv) {
     return E_NOINTERFACE;
   }
   AccessibleWrap* localAcc = LocalAcc();
-  if (IID_IEnumVARIANT == iid && localAcc) {
-    // Don't support this interface for leaf elements.
-    if (!localAcc->HasChildren() || nsAccUtils::MustPrune(localAcc)) {
+  if (IID_IEnumVARIANT == iid) {
+    // We don't support this interface for leaf elements.
+    if (!mAcc->HasChildren() || nsAccUtils::MustPrune(mAcc)) {
       return E_NOINTERFACE;
     }
     *ppv = static_cast<IEnumVARIANT*>(new ChildrenEnumVariant(this));
-  } else if (IID_ISimpleDOMNode == iid && localAcc) {
-    if (!localAcc->HasOwnContent() && !localAcc->IsDoc()) {
+  } else if (IID_ISimpleDOMNode == iid) {
+    if (mAcc->IsDoc() || (localAcc && !localAcc->HasOwnContent())) {
       return E_NOINTERFACE;
     }
 
@@ -840,24 +540,16 @@ MsaaAccessible::QueryInterface(REFIID iid, void** ppv) {
     if (SUCCEEDED(hr)) return hr;
   }
 
-  if (!*ppv && localAcc) {
+  if (!*ppv) {
     HRESULT hr = ia2AccessibleHyperlink::QueryInterface(iid, ppv);
     if (SUCCEEDED(hr)) return hr;
   }
 
-  if (!*ppv && localAcc) {
+  if (!*ppv) {
     HRESULT hr = ia2AccessibleValue::QueryInterface(iid, ppv);
     if (SUCCEEDED(hr)) return hr;
   }
 
-  if (!*ppv && iid == IID_IGeckoCustom) {
-    MOZ_ASSERT(XRE_IsContentProcess() &&
-               !StaticPrefs::accessibility_cache_enabled_AtStartup());
-    RefPtr<GeckoCustom> gkCrap = new GeckoCustom(this);
-    gkCrap.forget(ppv);
-    return S_OK;
-  }
-
   if (nullptr == *ppv) return E_NOINTERFACE;
 
   (reinterpret_cast<IUnknown*>(*ppv))->AddRef();
@@ -878,8 +570,6 @@ MsaaAccessible::get_accParent(IDispatch __RPC_FAR* __RPC_FAR* ppdispParent) {
 
   Accessible* xpParentAcc = mAcc->Parent();
   if (!xpParentAcc) return S_FALSE;
-  MOZ_ASSERT(StaticPrefs::accessibility_cache_enabled_AtStartup() ||
-             xpParentAcc->IsLocal());
 
   *ppdispParent = NativeAccessible(xpParentAcc);
   return S_OK;
@@ -893,6 +583,15 @@ MsaaAccessible::get_accChildCount(long __RPC_FAR* pcountChildren) {
 
   if (!mAcc) return CO_E_OBJNOTCONNECTED;
 
+  if (Compatibility::IsA11ySuppressedForClipboardCopy() && mAcc->IsRoot()) {
+    // Bug 1798098: Windows Suggested Actions (introduced in Windows 11 22H2)
+    // might walk the entire a11y tree using UIA whenever anything is copied to
+    // the clipboard. This causes an unacceptable hang, particularly when the
+    // cache is disabled. We prevent this tree walk by returning a 0 child count
+    // for the root window, from which Windows might walk.
+    return S_OK;
+  }
+
   if (nsAccUtils::MustPrune(mAcc)) return S_OK;
 
   *pcountChildren = mAcc->ChildCount();
@@ -944,16 +643,10 @@ MsaaAccessible::get_accName(
   if (accessible) {
     return accessible->get_accName(kVarChildIdSelf, pszName);
   }
-  if (mAcc->IsRemote()) {
-    return E_NOTIMPL;  // XXX Not supported for RemoteAccessible yet.
-  }
 
   nsAutoString name;
-  LocalAcc()->Name(name);
+  Acc()->Name(name);
 
-  // The name was not provided, e.g. no alt attribute for an image. A screen
-  // reader may choose to invent its own accessible name, e.g. from an image src
-  // attribute. Refer to eNoNameOnPurpose return value.
   if (name.IsVoid()) return S_FALSE;
 
   *pszName = ::SysAllocStringLen(name.get(), name.Length());
@@ -978,12 +671,9 @@ MsaaAccessible::get_accValue(
   if (accessible) {
     return accessible->get_accValue(kVarChildIdSelf, pszValue);
   }
-  if (mAcc->IsRemote()) {
-    return E_NOTIMPL;  // XXX Not supported for RemoteAccessible yet.
-  }
 
   nsAutoString value;
-  LocalAcc()->Value(value);
+  Acc()->Value(value);
 
   // See bug 438784: need to expose URL on doc's value attribute. For this,
   // reverting part of fix for bug 425693 to make this MSAA method behave
@@ -1011,12 +701,9 @@ MsaaAccessible::get_accDescription(VARIANT varChild,
   if (accessible) {
     return accessible->get_accDescription(kVarChildIdSelf, pszDescription);
   }
-  if (mAcc->IsRemote()) {
-    return E_NOTIMPL;  // XXX Not supported for RemoteAccessible yet.
-  }
 
   nsAutoString description;
-  LocalAcc()->Description(description);
+  Acc()->Description(description);
 
   *pszDescription =
       ::SysAllocStringLen(description.get(), description.Length());
@@ -1052,10 +739,10 @@ MsaaAccessible::get_accRole(
 
   uint32_t msaaRole = 0;
 
-#define ROLE(_geckoRole, stringRole, atkRole, macRole, macSubrole, _msaaRole, \
-             ia2Role, androidClass, nameRule)                                 \
-  case roles::_geckoRole:                                                     \
-    msaaRole = _msaaRole;                                                     \
+#define ROLE(_geckoRole, stringRole, ariaRole, atkRole, macRole, macSubrole, \
+             _msaaRole, ia2Role, androidClass, nameRule)                     \
+  case roles::_geckoRole:                                                    \
+    msaaRole = _msaaRole;                                                    \
     break;
 
   switch (geckoRole) {
@@ -1085,18 +772,36 @@ MsaaAccessible::get_accRole(
   // -- Try BSTR role
   // Could not map to known enumerated MSAA role like ROLE_BUTTON
   // Use BSTR role to expose role attribute or tag name + namespace
-  LocalAccessible* localAcc = mAcc->AsLocal();
-  if (!localAcc) {
-    return E_FAIL;
+  // XXX We should remove this hack and map to standard MSAA roles, even though
+  // they're lossy. See bug 798492.
+  if (mAcc->IsRemote()) {
+    // We don't support unknown or multiple ARIA roles for RemoteAccessible
+    // here, nor can we support namespaces. No one should be relying on this
+    // anyway, so this is fine. We just want to avoid returning a failure here.
+    nsAtom* val = nullptr;
+    const nsRoleMapEntry* roleMap = mAcc->ARIARoleMap();
+    if (roleMap && roleMap->roleAtom != nsGkAtoms::_empty) {
+      val = roleMap->roleAtom;
+    } else {
+      val = mAcc->TagName();
+    }
+    if (!val) {
+      return E_FAIL;
+    }
+    pvarRole->vt = VT_BSTR;
+    pvarRole->bstrVal = ::SysAllocString(val->GetUTF16String());
+    return S_OK;
   }
+
+  LocalAccessible* localAcc = mAcc->AsLocal();
+  MOZ_ASSERT(localAcc);
   nsIContent* content = localAcc->GetContent();
   if (!content) return E_FAIL;
 
   if (content->IsElement()) {
     nsAutoString roleString;
     // Try the role attribute.
-    content->AsElement()->GetAttr(kNameSpaceID_None, nsGkAtoms::role,
-                                  roleString);
+    nsAccUtils::GetARIAAttr(content->AsElement(), nsGkAtoms::role, roleString);
 
     if (roleString.IsEmpty()) {
       // No role attribute (or it is an empty string).
@@ -1144,9 +849,6 @@ MsaaAccessible::get_accState(
   if (accessible) {
     return accessible->get_accState(kVarChildIdSelf, pvarState);
   }
-  if (mAcc->IsRemote()) {
-    return E_NOTIMPL;  // XXX Not supported for RemoteAccessible yet.
-  }
 
   // MSAA only has 31 states and the lowest 31 bits of our state bit mask
   // are the same states as MSAA.
@@ -1156,7 +858,7 @@ MsaaAccessible::get_accState(
   //   INVALID -> ALERT_HIGH
   //   CHECKABLE -> MARQUEED
 
-  uint64_t state = LocalAcc()->State();
+  uint64_t state = Acc()->State();
 
   uint32_t msaaState = 0;
   nsAccUtils::To32States(state, &msaaState, nullptr);
@@ -1204,12 +906,12 @@ MsaaAccessible::get_accKeyboardShortcut(
                                                pszKeyboardShortcut);
   }
 
-  LocalAccessible* localAcc = LocalAcc();
-  if (!localAcc) {
-    return E_NOTIMPL;  // XXX Not supported for RemoteAccessible yet.
+  KeyBinding keyBinding = mAcc->AccessKey();
+  if (keyBinding.IsEmpty()) {
+    if (LocalAccessible* localAcc = mAcc->AsLocal()) {
+      keyBinding = localAcc->KeyboardShortcut();
+    }
   }
-  KeyBinding keyBinding = localAcc->AccessKey();
-  if (keyBinding.IsEmpty()) keyBinding = localAcc->KeyboardShortcut();
 
   nsAutoString shortcut;
   keyBinding.ToString(shortcut);
@@ -1236,15 +938,9 @@ MsaaAccessible::get_accFocus(
   if (!mAcc) {
     return CO_E_OBJNOTCONNECTED;
   }
-  LocalAccessible* localAcc = LocalAcc();
-  if (!localAcc) {
-    return E_NOTIMPL;  // XXX Not supported for RemoteAccessible yet.
-  }
-
   // Return the current IAccessible child that has focus
-  LocalAccessible* focusedAccessible = localAcc->FocusedChild();
-
-  if (focusedAccessible == localAcc) {
+  Accessible* focusedAccessible = mAcc->FocusedChild();
+  if (focusedAccessible == mAcc) {
     pvarChild->vt = VT_I4;
     pvarChild->lVal = CHILDID_SELF;
   } else if (focusedAccessible) {
@@ -1263,7 +959,7 @@ MsaaAccessible::get_accFocus(
  */
 class AccessibleEnumerator final : public IEnumVARIANT {
  public:
-  explicit AccessibleEnumerator(const nsTArray<LocalAccessible*>& aArray)
+  explicit AccessibleEnumerator(const nsTArray<Accessible*>& aArray)
       : mArray(aArray.Clone()), mCurIndex(0) {}
   AccessibleEnumerator(const AccessibleEnumerator& toCopy)
       : mArray(toCopy.mArray.Clone()), mCurIndex(toCopy.mCurIndex) {}
@@ -1283,7 +979,7 @@ class AccessibleEnumerator final : public IEnumVARIANT {
   STDMETHODIMP Clone(IEnumVARIANT FAR* FAR* ppenum);
 
  private:
-  nsTArray<LocalAccessible*> mArray;
+  nsTArray<Accessible*> mArray;
   uint32_t mCurIndex;
 };
 
@@ -1373,17 +1069,14 @@ MsaaAccessible::get_accSelection(VARIANT __RPC_FAR* pvarChildren) {
   if (!mAcc) {
     return CO_E_OBJNOTCONNECTED;
   }
-  LocalAccessible* localAcc = LocalAcc();
-  if (!localAcc) {
-    return E_NOTIMPL;  // XXX Not supported for RemoteAccessible yet.
-  }
+  Accessible* acc = Acc();
 
-  if (!localAcc->IsSelect()) {
+  if (!acc->IsSelect()) {
     return S_OK;
   }
 
-  AutoTArray<LocalAccessible*, 10> selectedItems;
-  localAcc->SelectedItems(&selectedItems);
+  AutoTArray<Accessible*, 10> selectedItems;
+  acc->SelectedItems(&selectedItems);
   uint32_t count = selectedItems.Length();
   if (count == 1) {
     pvarChildren->vt = VT_DISPATCH;
@@ -1391,7 +1084,6 @@ MsaaAccessible::get_accSelection(VARIANT __RPC_FAR* pvarChildren) {
   } else if (count > 1) {
     RefPtr<AccessibleEnumerator> pEnum =
         new AccessibleEnumerator(selectedItems);
-    AssociateCOMObjectForDisconnection(pEnum);
     pvarChildren->vt =
         VT_UNKNOWN;  // this must be VT_UNKNOWN for an IEnumVARIANT
     NS_ADDREF(pvarChildren->punkVal = pEnum);
@@ -1418,12 +1110,9 @@ MsaaAccessible::get_accDefaultAction(
   if (accessible) {
     return accessible->get_accDefaultAction(kVarChildIdSelf, pszDefaultAction);
   }
-  if (mAcc->IsRemote()) {
-    return E_NOTIMPL;  // XXX Not supported for RemoteAccessible yet.
-  }
 
   nsAutoString defaultAction;
-  LocalAcc()->ActionNameAt(0, defaultAction);
+  mAcc->ActionNameAt(0, defaultAction);
 
   *pszDefaultAction =
       ::SysAllocStringLen(defaultAction.get(), defaultAction.Length());
@@ -1443,38 +1132,24 @@ MsaaAccessible::accSelect(
   if (accessible) {
     return accessible->accSelect(flagsSelect, kVarChildIdSelf);
   }
-  if (mAcc->IsRemote()) {
-    return E_NOTIMPL;  // XXX Not supported for RemoteAccessible yet.
-  }
 
-  LocalAccessible* localAcc = LocalAcc();
   if (flagsSelect & SELFLAG_TAKEFOCUS) {
-    if (XRE_IsContentProcess()) {
-      // In this case we might have been invoked while the IPC MessageChannel is
-      // waiting on a sync reply. We cannot dispatch additional IPC while that
-      // is happening, so we dispatch TakeFocus from the main thread to
-      // guarantee that we are outside any IPC.
-      nsCOMPtr<nsIRunnable> runnable = mozilla::NewRunnableMethod(
-          "LocalAccessible::TakeFocus", localAcc, &LocalAccessible::TakeFocus);
-      NS_DispatchToMainThread(runnable, NS_DISPATCH_NORMAL);
-      return S_OK;
-    }
-    localAcc->TakeFocus();
+    mAcc->TakeFocus();
     return S_OK;
   }
 
   if (flagsSelect & SELFLAG_TAKESELECTION) {
-    localAcc->TakeSelection();
+    mAcc->TakeSelection();
     return S_OK;
   }
 
   if (flagsSelect & SELFLAG_ADDSELECTION) {
-    localAcc->SetSelected(true);
+    mAcc->SetSelected(true);
     return S_OK;
   }
 
   if (flagsSelect & SELFLAG_REMOVESELECTION) {
-    localAcc->SetSelected(false);
+    mAcc->SetSelected(false);
     return S_OK;
   }
 
@@ -1506,13 +1181,7 @@ MsaaAccessible::accLocation(
                                    kVarChildIdSelf);
   }
 
-  LocalAccessible* localAcc = LocalAcc();
-  if (!localAcc) {
-    return E_NOTIMPL;  // XXX Not supported for RemoteAccessible yet.
-  }
-
-  nsIntRect rect = localAcc->Bounds();
-
+  LayoutDeviceIntRect rect = Acc()->Bounds();
   *pxLeft = rect.X();
   *pyTop = rect.Y();
   *pcxWidth = rect.Width();
@@ -1538,7 +1207,6 @@ MsaaAccessible::accNavigate(
   if (accessible) {
     return accessible->accNavigate(navDir, kVarChildIdSelf, pvarEndUpAt);
   }
-  MOZ_ASSERT(!mAcc->IsProxy());
 
   Accessible* navAccessible = nullptr;
   Maybe<RelationType> xpRelation;
@@ -1550,26 +1218,12 @@ MsaaAccessible::accNavigate(
 
   switch (navDir) {
     case NAVDIR_FIRSTCHILD:
-      if (!StaticPrefs::accessibility_cache_enabled_AtStartup() &&
-          XRE_IsContentProcess() && mAcc->IsOuterDoc()) {
-        // With the cache disabled, we need to be able to return a COM proxy for
-        // an OOP iframe. The COM proxy is wrapped in a
-        // RemoteIframeDocRemoteAccessibleWrap, which can only be retrieved
-        // using the LocalAccessible hierarchy.
-        navAccessible = LocalAcc()->LocalFirstChild();
-      } else if (!nsAccUtils::MustPrune(mAcc)) {
+      if (!nsAccUtils::MustPrune(mAcc)) {
         navAccessible = mAcc->FirstChild();
       }
       break;
     case NAVDIR_LASTCHILD:
-      if (!StaticPrefs::accessibility_cache_enabled_AtStartup() &&
-          XRE_IsContentProcess() && mAcc->IsOuterDoc()) {
-        // With the cache disabled, we need to be able to return a COM proxy for
-        // an OOP iframe. The COM proxy is wrapped in a
-        // RemoteIframeDocRemoteAccessibleWrap, which can only be retrieved
-        // using the LocalAccessible hierarchy.
-        navAccessible = LocalAcc()->LocalLastChild();
-      } else if (!nsAccUtils::MustPrune(mAcc)) {
+      if (!nsAccUtils::MustPrune(mAcc)) {
         navAccessible = mAcc->LastChild();
       }
       break;
@@ -1597,10 +1251,7 @@ MsaaAccessible::accNavigate(
   pvarEndUpAt->vt = VT_EMPTY;
 
   if (xpRelation) {
-    if (mAcc->IsRemote()) {
-      return E_NOTIMPL;  // XXX Not supported for RemoteAccessible yet.
-    }
-    Relation rel = mAcc->AsLocal()->RelationByType(*xpRelation);
+    Relation rel = mAcc->RelationByType(*xpRelation);
     navAccessible = rel.Next();
   }
 
@@ -1623,21 +1274,21 @@ MsaaAccessible::accHitTest(
   if (!mAcc) {
     return CO_E_OBJNOTCONNECTED;
   }
-  LocalAccessible* localAcc = LocalAcc();
-  if (!localAcc) {
-    return E_NOTIMPL;  // XXX Not supported for RemoteAccessible yet.
-  }
 
-  LocalAccessible* accessible = localAcc->LocalChildAtPoint(
-      xLeft, yTop, Accessible::EWhichChildAtPoint::DirectChild);
+  // The MSAA documentation says accHitTest should return a child. However,
+  // clients call AccessibleObjectFromPoint, which ends up walking the
+  // descendants calling accHitTest on each one. Since clients want the
+  // deepest descendant anyway, it's faster and probably more accurate to
+  // just do this ourselves.
+  Accessible* accessible = mAcc->ChildAtPoint(
+      xLeft, yTop, Accessible::EWhichChildAtPoint::DeepestChild);
 
   // if we got a child
   if (accessible) {
-    // if the child is us
-    if (accessible == localAcc) {
+    if (accessible == mAcc) {
       pvarChild->vt = VT_I4;
       pvarChild->lVal = CHILDID_SELF;
-    } else {  // its not create a LocalAccessible for it.
+    } else {
       pvarChild->vt = VT_DISPATCH;
       pvarChild->pdispVal = NativeAccessible(accessible);
     }
@@ -1661,11 +1312,8 @@ MsaaAccessible::accDoDefaultAction(
   if (accessible) {
     return accessible->accDoDefaultAction(kVarChildIdSelf);
   }
-  if (mAcc->IsRemote()) {
-    return E_NOTIMPL;  // XXX Not supported for RemoteAccessible yet.
-  }
 
-  return LocalAcc()->DoAction(0) ? S_OK : E_INVALIDARG;
+  return mAcc->DoAction(0) ? S_OK : E_INVALIDARG;
 }
 
 STDMETHODIMP
@@ -1688,11 +1336,8 @@ MsaaAccessible::put_accValue(
   if (accessible) {
     return accessible->put_accValue(kVarChildIdSelf, szValue);
   }
-  if (mAcc->IsRemote()) {
-    return E_NOTIMPL;  // XXX Not supported for RemoteAccessible yet.
-  }
 
-  HyperTextAccessible* ht = LocalAcc()->AsHyperText();
+  HyperTextAccessibleBase* ht = mAcc->AsHyperTextBase();
   if (!ht) {
     return E_NOTIMPL;
   }
