@@ -3,8 +3,8 @@
 
 "use strict";
 
-const { FxAccountsClient } = ChromeUtils.import(
-  "resource://gre/modules/FxAccountsClient.jsm"
+const { FxAccountsClient } = ChromeUtils.importESModule(
+  "resource://gre/modules/FxAccountsClient.sys.mjs"
 );
 
 const FAKE_SESSION_TOKEN =
@@ -47,7 +47,7 @@ add_task(async function test_authenticated_get_request() {
   let method = "GET";
 
   let server = httpd_setup({
-    "/foo": function(request, response) {
+    "/foo": function (request, response) {
       Assert.ok(request.hasHeader("Authorization"));
 
       response.setStatusLine(request.httpVersion, 200, "OK");
@@ -72,7 +72,7 @@ add_task(async function test_authenticated_post_request() {
   let method = "POST";
 
   let server = httpd_setup({
-    "/foo": function(request, response) {
+    "/foo": function (request, response) {
       Assert.ok(request.hasHeader("Authorization"));
 
       response.setStatusLine(request.httpVersion, 200, "OK");
@@ -99,7 +99,7 @@ add_task(async function test_500_error() {
   let method = "GET";
 
   let server = httpd_setup({
-    "/foo": function(request, response) {
+    "/foo": function (request, response) {
       response.setStatusLine(request.httpVersion, 500, "Internal Server Error");
       response.bodyOutputStream.write(message, message.length);
     },
@@ -121,7 +121,7 @@ add_task(async function test_500_error() {
 add_task(async function test_backoffError() {
   let method = "GET";
   let server = httpd_setup({
-    "/retryDelay": function(request, response) {
+    "/retryDelay": function (request, response) {
       response.setHeader("Retry-After", "30");
       response.setStatusLine(
         request.httpVersion,
@@ -131,7 +131,7 @@ add_task(async function test_backoffError() {
       let message = "<h1>Ooops!</h1>";
       response.bodyOutputStream.write(message, message.length);
     },
-    "/duringDelayIShouldNotBeCalled": function(request, response) {
+    "/duringDelayIShouldNotBeCalled": function (request, response) {
       response.setStatusLine(request.httpVersion, 200, "OK");
       let jsonMessage = '{"working": "yes"}';
       response.bodyOutputStream.write(jsonMessage, jsonMessage.length);
@@ -190,7 +190,7 @@ add_task(async function test_signUp() {
   let unicodeUsername = "andr\xe9@example.org"; // 'andré@example.org'
   let unicodePassword = "p\xe4ssw\xf6rd"; // 'pässwörd'
   let server = httpd_setup({
-    "/account/create": function(request, response) {
+    "/account/create": function (request, response) {
       let body = CommonUtils.readBytesFromInputStream(request.bodyInputStream);
       body = CommonUtils.decodeUTF8(body);
       let jsonBody = JSON.parse(body);
@@ -295,7 +295,7 @@ add_task(async function test_signIn() {
   // Note this strings must be unicode and not already utf-8 encoded.
   let unicodeUsername = "m\xe9@example.com"; // 'mé@example.com'
   let server = httpd_setup({
-    "/account/login": function(request, response) {
+    "/account/login": function (request, response) {
       let body = CommonUtils.readBytesFromInputStream(request.bodyInputStream);
       body = CommonUtils.decodeUTF8(body);
       let jsonBody = JSON.parse(body);
@@ -389,7 +389,7 @@ add_task(async function test_signOut() {
   let signedOut = false;
 
   let server = httpd_setup({
-    "/session/destroy": function(request, response) {
+    "/session/destroy": function (request, response) {
       if (!signedOut) {
         signedOut = true;
         Assert.ok(request.hasHeader("Authorization"));
@@ -429,7 +429,7 @@ add_task(async function test_recoveryEmailStatus() {
   let tries = 0;
 
   let server = httpd_setup({
-    "/recovery_email/status": function(request, response) {
+    "/recovery_email/status": function (request, response) {
       Assert.ok(request.hasHeader("Authorization"));
       Assert.equal("", request._queryString);
 
@@ -464,7 +464,7 @@ add_task(async function test_recoveryEmailStatus() {
 add_task(async function test_recoveryEmailStatusWithReason() {
   let emailStatus = JSON.stringify({ verified: true });
   let server = httpd_setup({
-    "/recovery_email/status": function(request, response) {
+    "/recovery_email/status": function (request, response) {
       Assert.ok(request.hasHeader("Authorization"));
       // if there is a query string then it will have a reason
       Assert.equal("reason=push", request._queryString);
@@ -492,7 +492,7 @@ add_task(async function test_resendVerificationEmail() {
   let tries = 0;
 
   let server = httpd_setup({
-    "/recovery_email/resend_code": function(request, response) {
+    "/recovery_email/resend_code": function (request, response) {
       Assert.ok(request.hasHeader("Authorization"));
       if (tries === 0) {
         tries += 1;
@@ -537,7 +537,7 @@ add_task(async function test_accountKeys() {
   let attempt = 0;
 
   let server = httpd_setup({
-    "/account/keys": function(request, response) {
+    "/account/keys": function (request, response) {
       Assert.ok(request.hasHeader("Authorization"));
       attempt += 1;
 
@@ -613,68 +613,9 @@ add_task(async function test_accountKeys() {
   await promiseStopServer(server);
 });
 
-add_task(async function test_signCertificate() {
-  let certSignMessage = JSON.stringify({ cert: { bar: "baz" } });
-  let errorMessage = JSON.stringify({
-    code: 400,
-    errno: 102,
-    error: "doesn't exist",
-  });
-  let tries = 0;
-
-  let server = httpd_setup({
-    "/certificate/sign": function(request, response) {
-      Assert.ok(request.hasHeader("Authorization"));
-      Assert.ok(request.queryString.startsWith("service="));
-
-      if (tries === 0) {
-        tries += 1;
-        let body = CommonUtils.readBytesFromInputStream(
-          request.bodyInputStream
-        );
-        let jsonBody = JSON.parse(body);
-        Assert.equal(JSON.parse(jsonBody.publicKey).foo, "bar");
-        Assert.equal(jsonBody.duration, 600);
-        response.setStatusLine(request.httpVersion, 200, "OK");
-        response.bodyOutputStream.write(
-          certSignMessage,
-          certSignMessage.length
-        );
-        return;
-      }
-
-      // Second attempt, trigger error
-      response.setStatusLine(request.httpVersion, 400, "Bad request");
-      response.bodyOutputStream.write(errorMessage, errorMessage.length);
-    },
-  });
-
-  let client = new FxAccountsClient(server.baseURI);
-  let result = await client.signCertificate(
-    FAKE_SESSION_TOKEN,
-    JSON.stringify({ foo: "bar" }),
-    600
-  );
-  Assert.equal("baz", result.bar);
-
-  // Account doesn't exist
-  try {
-    result = await client.signCertificate(
-      "bogus",
-      JSON.stringify({ foo: "bar" }),
-      600
-    );
-    do_throw("Expected to catch an exception");
-  } catch (expectedError) {
-    Assert.equal(102, expectedError.errno);
-  }
-
-  await promiseStopServer(server);
-});
-
 add_task(async function test_accessTokenWithSessionToken() {
   let server = httpd_setup({
-    "/oauth/token": function(request, response) {
+    "/oauth/token": function (request, response) {
       const responseMessage = JSON.stringify({
         access_token:
           "43793fdfffec22eb39fc3c44ed09193a6fde4c24e5d6a73f73178597b268af69",
@@ -720,7 +661,7 @@ add_task(async function test_accountExists() {
   let emptyMessage = "{}";
 
   let server = httpd_setup({
-    "/account/login": function(request, response) {
+    "/account/login": function (request, response) {
       let body = CommonUtils.readBytesFromInputStream(request.bodyInputStream);
       let jsonBody = JSON.parse(body);
 
@@ -783,7 +724,7 @@ add_task(async function test_registerDevice() {
   const ERROR_NAME = "test that the client promise rejects";
 
   const server = httpd_setup({
-    "/account/device": function(request, response) {
+    "/account/device": function (request, response) {
       const body = JSON.parse(
         CommonUtils.readBytesFromInputStream(request.bodyInputStream)
       );
@@ -846,7 +787,7 @@ add_task(async function test_updateDevice() {
   const ERROR_ID = "test that the client promise rejects";
 
   const server = httpd_setup({
-    "/account/device": function(request, response) {
+    "/account/device": function (request, response) {
       const body = JSON.parse(
         CommonUtils.readBytesFromInputStream(request.bodyInputStream)
       );
@@ -901,7 +842,7 @@ add_task(async function test_getDeviceList() {
   let canReturnDevices;
 
   const server = httpd_setup({
-    "/account/devices": function(request, response) {
+    "/account/devices": function (request, response) {
       if (canReturnDevices) {
         response.setStatusLine(request.httpVersion, 200, "OK");
         response.bodyOutputStream.write("[]", 2);
@@ -939,7 +880,7 @@ add_task(async function test_client_metrics() {
   }
 
   let server = httpd_setup({
-    "/session/destroy": function(request, response) {
+    "/session/destroy": function (request, response) {
       response.setHeader("Content-Type", "application/json; charset=utf-8");
       response.setStatusLine(request.httpVersion, 401, "Unauthorized");
       writeResp(response, {
@@ -956,7 +897,7 @@ add_task(async function test_client_metrics() {
     client.signOut(FAKE_SESSION_TOKEN, {
       service: "sync",
     }),
-    function(err) {
+    function (err) {
       return err.errno == 111;
     }
   );
@@ -977,7 +918,7 @@ add_task(async function test_email_case() {
   }
 
   let server = httpd_setup({
-    "/account/login": function(request, response) {
+    "/account/login": function (request, response) {
       response.setHeader("Content-Type", "application/json; charset=utf-8");
       attempts += 1;
       if (attempts > 2) {
