@@ -3,18 +3,15 @@
 
 "use strict";
 
-const {
-  ON_PROFILE_CHANGE_NOTIFICATION,
-  WEBCHANNEL_ID,
-  log,
-} = ChromeUtils.import("resource://gre/modules/FxAccountsCommon.js");
-const { CryptoUtils } = ChromeUtils.import(
-  "resource://services-crypto/utils.js"
+const { ON_PROFILE_CHANGE_NOTIFICATION, WEBCHANNEL_ID, log } =
+  ChromeUtils.import("resource://gre/modules/FxAccountsCommon.js");
+const { CryptoUtils } = ChromeUtils.importESModule(
+  "resource://services-crypto/utils.sys.mjs"
 );
-const {
-  FxAccountsWebChannel,
-  FxAccountsWebChannelHelpers,
-} = ChromeUtils.import("resource://gre/modules/FxAccountsWebChannel.jsm", null);
+const { FxAccountsWebChannel, FxAccountsWebChannelHelpers } =
+  ChromeUtils.importESModule(
+    "resource://gre/modules/FxAccountsWebChannel.sys.mjs"
+  );
 
 const URL_STRING = "https://example.com";
 
@@ -24,7 +21,7 @@ const mockSendingContext = {
   eventTarget: {},
 };
 
-add_test(function() {
+add_test(function () {
   validationHelper(undefined, "Error: Missing configuration options");
 
   validationHelper(
@@ -53,6 +50,11 @@ add_test(function() {
 });
 
 add_task(async function test_rejection_reporting() {
+  Services.prefs.setBoolPref(
+    "browser.tabs.remote.separatePrivilegedMozillaWebContentProcess",
+    false
+  );
+
   let mockMessage = {
     command: "fxaccounts:login",
     messageId: "1234",
@@ -221,7 +223,7 @@ add_test(function test_profile_image_change_message() {
     data: { uid: "foo" },
   };
 
-  makeObserver(ON_PROFILE_CHANGE_NOTIFICATION, function(subject, topic, data) {
+  makeObserver(ON_PROFILE_CHANGE_NOTIFICATION, function (subject, topic, data) {
     Assert.equal(data, "foo");
     run_next_test();
   });
@@ -831,7 +833,7 @@ add_test(function test_helpers_open_sync_preferences() {
   let mockBrowser = {
     loadURI(uri) {
       Assert.equal(
-        uri,
+        uri.spec,
         "about:preferences?entrypoint=fxa%3Averification_complete#sync"
       );
       run_next_test();
@@ -1269,6 +1271,7 @@ add_task(async function test_helpers_change_password() {
             Assert.ok(credentials.hasOwnProperty("unwrapBKey"));
             Assert.ok(credentials.hasOwnProperty("device"));
             Assert.equal(null, credentials.device);
+            Assert.equal(null, credentials.encryptedSendTabKeys);
             // "foo" isn't a field known by storage, so should be dropped.
             Assert.ok(!credentials.hasOwnProperty("foo"));
             wasCalled.updateUserAccountData = true;
@@ -1325,7 +1328,7 @@ add_task(async function test_helpers_change_password_with_error() {
 });
 
 function makeObserver(aObserveTopic, aObserveFunc) {
-  let callback = function(aSubject, aTopic, aData) {
+  let callback = function (aSubject, aTopic, aData) {
     log.debug("observed " + aTopic + " " + aData);
     if (aTopic == aObserveTopic) {
       removeMe();
