@@ -10,17 +10,16 @@ extern crate bitflags;
 #[macro_use]
 extern crate cssparser;
 #[macro_use]
+extern crate debug_unreachable;
+#[macro_use]
 extern crate derive_more;
 extern crate fxhash;
 #[macro_use]
 extern crate log;
-#[macro_use]
-extern crate matches;
 extern crate phf;
 extern crate precomputed_hash;
 extern crate servo_arc;
 extern crate smallvec;
-extern crate thin_slice;
 extern crate to_shmem;
 #[macro_use]
 extern crate to_shmem_derive;
