@@ -17,16 +17,18 @@ def main(filename):
         or 'data-dfn-for="<counter-style>"' in line
     ]
     with open(filename, "wb") as f:
-        f.write("""\
+        f.write(
+            """\
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
 
 predefined! {
-""")
+"""
+        )
         for name in names:
             f.write('    "%s",\n' % name)
-        f.write('}\n')
+        f.write("}\n")
 
 
 if __name__ == "__main__":
