@@ -5,16 +5,17 @@
 //! Code for invalidations due to state or attribute changes.
 
 use crate::context::QuirksMode;
-use crate::element_state::{DocumentState, ElementState};
-use crate::selector_map::{MaybeCaseInsensitiveHashMap, PrecomputedHashMap, SelectorMap, SelectorMapEntry};
+use crate::selector_map::{
+    MaybeCaseInsensitiveHashMap, PrecomputedHashMap, SelectorMap, SelectorMapEntry,
+};
 use crate::selector_parser::SelectorImpl;
-use crate::{Atom, LocalName, Namespace};
-use fallible::FallibleVec;
-use hashglobe::FailedAllocationError;
+use crate::AllocErr;
+use crate::{Atom, LocalName, Namespace, ShrinkIfNeeded};
+use dom::{DocumentState, ElementState};
 use selectors::attr::NamespaceConstraint;
 use selectors::parser::{Combinator, Component};
 use selectors::parser::{Selector, SelectorIter};
-use selectors::visitor::SelectorVisitor;
+use selectors::visitor::{SelectorListKind, SelectorVisitor};
 use smallvec::SmallVec;
 
 /// Mapping between (partial) CompoundSelectors (and the combinator to their
@@ -33,7 +34,7 @@ use smallvec::SmallVec;
 /// We generate a Dependency for both |a _ b:X _| and |a _ b:X _ c _ d:Y _|,
 /// even though those selectors may not appear on their own in any stylesheet.
 /// This allows us to quickly scan through the dependency sites of all style
-/// rules and determine the maximum effect that a given state or attributef
+/// rules and determine the maximum effect that a given state or attribute
 /// change may have on the style of elements in the document.
 #[derive(Clone, Debug, MallocSizeOf)]
 pub struct Dependency {
@@ -183,7 +184,7 @@ pub struct DocumentStateDependency {
 /// In particular, we want to lookup as few things as possible to get the fewer
 /// selectors the better, so this looks up by id, class, or looks at the list of
 /// state/other attribute affecting selectors.
-#[derive(Debug, MallocSizeOf)]
+#[derive(Clone, Debug, MallocSizeOf)]
 pub struct InvalidationMap {
     /// A map from a given class name to all the selectors with that class
     /// selector.
@@ -196,7 +197,8 @@ pub struct InvalidationMap {
     /// A list of document state dependencies in the rules we represent.
     pub document_state_selectors: Vec<DocumentStateDependency>,
     /// A map of other attribute affecting selectors.
-    pub other_attribute_affecting_selectors: PrecomputedHashMap<Atom, SmallVec<[Dependency; 1]>>,
+    pub other_attribute_affecting_selectors:
+        PrecomputedHashMap<LocalName, SmallVec<[Dependency; 1]>>,
 }
 
 impl InvalidationMap {
@@ -235,13 +237,21 @@ impl InvalidationMap {
         self.other_attribute_affecting_selectors.clear();
     }
 
+    /// Shrink the capacity of hash maps if needed.
+    pub fn shrink_if_needed(&mut self) {
+        self.class_to_selector.shrink_if_needed();
+        self.id_to_selector.shrink_if_needed();
+        self.state_affecting_selectors.shrink_if_needed();
+        self.other_attribute_affecting_selectors.shrink_if_needed();
+    }
+
     /// Adds a selector to this `InvalidationMap`.  Returns Err(..) to
     /// signify OOM.
     pub fn note_selector(
         &mut self,
         selector: &Selector<SelectorImpl>,
         quirks_mode: QuirksMode,
-    ) -> Result<(), FailedAllocationError> {
+    ) -> Result<(), AllocErr> {
         debug!("InvalidationMap::note_selector({:?})", selector);
 
         let mut document_state = DocumentState::empty();
@@ -271,7 +281,8 @@ impl InvalidationMap {
                 state: document_state,
                 dependency: Dependency::for_full_selector_invalidation(selector.clone()),
             };
-            self.document_state_selectors.try_push(dep)?;
+            self.document_state_selectors.try_reserve(1)?;
+            self.document_state_selectors.push(dep);
         }
 
         Ok(())
@@ -322,7 +333,7 @@ struct SelectorDependencyCollector<'a> {
     compound_state: PerCompoundState,
 
     /// The allocation error, if we OOM.
-    alloc_error: &'a mut Option<FailedAllocationError>,
+    alloc_error: &'a mut Option<AllocErr>,
 }
 
 impl<'a> SelectorDependencyCollector<'a> {
@@ -331,7 +342,11 @@ impl<'a> SelectorDependencyCollector<'a> {
         self.visit_whole_selector_from(iter, 0)
     }
 
-    fn visit_whole_selector_from(&mut self, mut iter: SelectorIter<SelectorImpl>, mut index: usize) -> bool {
+    fn visit_whole_selector_from(
+        &mut self,
+        mut iter: SelectorIter<SelectorImpl>,
+        mut index: usize,
+    ) -> bool {
         loop {
             // Reset the compound state.
             self.compound_state = PerCompoundState::new(index);
@@ -354,7 +369,7 @@ impl<'a> SelectorDependencyCollector<'a> {
                     self.quirks_mode,
                 );
                 if let Err(alloc_error) = result {
-                    *self.alloc_error = Some(alloc_error);
+                    *self.alloc_error = Some(alloc_error.into());
                     return false;
                 }
             }
@@ -371,21 +386,17 @@ impl<'a> SelectorDependencyCollector<'a> {
         let dependency = self.dependency();
 
         let map = &mut self.map.other_attribute_affecting_selectors;
-        let entry = match map.try_entry(name) {
-            Ok(entry) => entry,
-            Err(err) => {
-                *self.alloc_error = Some(err);
-                return false;
-            },
-        };
-
-        match entry.or_insert_with(SmallVec::new).try_push(dependency) {
-            Ok(..) => true,
-            Err(err) => {
-                *self.alloc_error = Some(err);
-                return false;
-            }
+        if let Err(err) = map.try_reserve(1) {
+            *self.alloc_error = Some(err.into());
+            return false;
         }
+        let vec = map.entry(name).or_default();
+        if let Err(err) = vec.try_reserve(1) {
+            *self.alloc_error = Some(err.into());
+            return false;
+        }
+        vec.push(dependency);
+        true
     }
 
     fn dependency(&self) -> Dependency {
@@ -395,8 +406,7 @@ impl<'a> SelectorDependencyCollector<'a> {
         // cache them or something.
         for &(ref selector, ref selector_offset) in self.parent_selectors.iter() {
             debug_assert_ne!(
-                self.compound_state.offset,
-                0,
+                self.compound_state.offset, 0,
                 "Shouldn't bother creating nested dependencies for the rightmost compound",
             );
             let new_parent = Dependency {
@@ -418,7 +428,11 @@ impl<'a> SelectorDependencyCollector<'a> {
 impl<'a> SelectorVisitor for SelectorDependencyCollector<'a> {
     type Impl = SelectorImpl;
 
-    fn visit_selector_list(&mut self, list: &[Selector<SelectorImpl>]) -> bool {
+    fn visit_selector_list(
+        &mut self,
+        _list_kind: SelectorListKind,
+        list: &[Selector<SelectorImpl>],
+    ) -> bool {
         for selector in list {
             // Here we cheat a bit: We can visit the rightmost compound with
             // the "outer" visitor, and it'd be fine. This reduces the amount of
@@ -442,7 +456,8 @@ impl<'a> SelectorVisitor for SelectorDependencyCollector<'a> {
 
             index += 1; // account for the combinator.
 
-            self.parent_selectors.push((self.selector.clone(), self.compound_state.offset));
+            self.parent_selectors
+                .push((self.selector.clone(), self.compound_state.offset));
             let mut nested = SelectorDependencyCollector {
                 map: &mut *self.map,
                 document_state: &mut *self.document_state,
@@ -461,7 +476,6 @@ impl<'a> SelectorVisitor for SelectorDependencyCollector<'a> {
     }
 
     fn visit_simple_selector(&mut self, s: &Component<SelectorImpl>) -> bool {
-        #[cfg(feature = "gecko")]
         use crate::selector_parser::NonTSPseudoClass;
 
         match *s {
@@ -472,27 +486,23 @@ impl<'a> SelectorVisitor for SelectorDependencyCollector<'a> {
                     Component::Class(..) => &mut self.map.class_to_selector,
                     _ => unreachable!(),
                 };
-                let entry = match map.try_entry(atom.clone(), self.quirks_mode) {
+                let entry = match map.try_entry(atom.0.clone(), self.quirks_mode) {
                     Ok(entry) => entry,
                     Err(err) => {
-                        *self.alloc_error = Some(err);
+                        *self.alloc_error = Some(err.into());
                         return false;
                     },
                 };
-                match entry.or_insert_with(SmallVec::new).try_push(dependency) {
-                    Ok(..) => true,
-                    Err(err) => {
-                        *self.alloc_error = Some(err);
-                        return false;
-                    }
+                let vec = entry.or_insert_with(SmallVec::new);
+                if let Err(err) = vec.try_reserve(1) {
+                    *self.alloc_error = Some(err.into());
+                    return false;
                 }
+                vec.push(dependency);
+                true
             },
             Component::NonTSPseudoClass(ref pc) => {
-                self.compound_state.element_state |= match *pc {
-                    #[cfg(feature = "gecko")]
-                    NonTSPseudoClass::Dir(ref dir) => dir.element_state(),
-                    _ => pc.state_flag(),
-                };
+                self.compound_state.element_state |= pc.state_flag();
                 *self.document_state |= pc.document_state_flag();
 
                 let attr_name = match *pc {
@@ -500,6 +510,12 @@ impl<'a> SelectorVisitor for SelectorDependencyCollector<'a> {
                     NonTSPseudoClass::MozTableBorderNonzero => local_name!("border"),
                     #[cfg(feature = "gecko")]
                     NonTSPseudoClass::MozBrowserFrame => local_name!("mozbrowser"),
+                    #[cfg(feature = "gecko")]
+                    NonTSPseudoClass::MozSelectListBox => {
+                        // This depends on two attributes.
+                        return self.add_attr_dependency(local_name!("multiple")) &&
+                            self.add_attr_dependency(local_name!("size"));
+                    },
                     NonTSPseudoClass::Lang(..) => local_name!("lang"),
                     _ => return true,
                 };
