@@ -15,19 +15,47 @@ PHYSICAL_AXES = ["x", "y"]
 LOGICAL_AXES = ["inline", "block"]
 
 # bool is True when logical
-ALL_SIDES = [(side, False) for side in PHYSICAL_SIDES] + [(side, True) for side in LOGICAL_SIDES]
-ALL_SIZES = [(size, False) for size in PHYSICAL_SIZES] + [(size, True) for size in LOGICAL_SIZES]
-ALL_CORNERS = [(corner, False) for corner in PHYSICAL_CORNERS] + [(corner, True) for corner in LOGICAL_CORNERS]
-ALL_AXES = [(axis, False) for axis in PHYSICAL_AXES] + [(axis, True) for axis in LOGICAL_AXES]
+ALL_SIDES = [(side, False) for side in PHYSICAL_SIDES] + [
+    (side, True) for side in LOGICAL_SIDES
+]
+ALL_SIZES = [(size, False) for size in PHYSICAL_SIZES] + [
+    (size, True) for size in LOGICAL_SIZES
+]
+ALL_CORNERS = [(corner, False) for corner in PHYSICAL_CORNERS] + [
+    (corner, True) for corner in LOGICAL_CORNERS
+]
+ALL_AXES = [(axis, False) for axis in PHYSICAL_AXES] + [
+    (axis, True) for axis in LOGICAL_AXES
+]
 
 SYSTEM_FONT_LONGHANDS = """font_family font_size font_style
-                           font_variant_caps font_stretch font_kerning
-                           font_variant_position font_weight
-                           font_size_adjust font_variant_alternates
-                           font_variant_ligatures font_variant_east_asian
-                           font_variant_numeric font_language_override
-                           font_feature_settings font_variation_settings
-                           font_optical_sizing""".split()
+                           font_stretch font_weight""".split()
+
+# Bitfield values for all rule types which can have property declarations.
+STYLE_RULE = 1 << 0
+PAGE_RULE = 1 << 1
+KEYFRAME_RULE = 1 << 2
+
+ALL_RULES = STYLE_RULE | PAGE_RULE | KEYFRAME_RULE
+DEFAULT_RULES = STYLE_RULE | KEYFRAME_RULE
+DEFAULT_RULES_AND_PAGE = DEFAULT_RULES | PAGE_RULE
+DEFAULT_RULES_EXCEPT_KEYFRAME = STYLE_RULE
+
+# Rule name to value dict
+RULE_VALUES = {
+    "Style": STYLE_RULE,
+    "Page": PAGE_RULE,
+    "Keyframe": KEYFRAME_RULE,
+}
+
+
+def rule_values_from_arg(that):
+    if isinstance(that, int):
+        return that
+    mask = 0
+    for rule in that.split():
+        mask |= RULE_VALUES[rule]
+    return mask
 
 
 def maybe_moz_logical_alias(engine, side, prop):
@@ -50,7 +78,9 @@ def to_snake_case(ident):
 
 
 def to_camel_case(ident):
-    return re.sub("(^|_|-)([a-z0-9])", lambda m: m.group(2).upper(), ident.strip("_").strip("-"))
+    return re.sub(
+        "(^|_|-)([a-z0-9])", lambda m: m.group(2).upper(), ident.strip("_").strip("-")
+    )
 
 
 def to_camel_case_lower(ident):
@@ -72,23 +102,32 @@ def parse_aliases(value):
 
 
 class Keyword(object):
-    def __init__(self, name, values, gecko_constant_prefix=None,
-                 gecko_enum_prefix=None, custom_consts=None,
-                 extra_gecko_values=None,
-                 extra_servo_2013_values=None,
-                 extra_servo_2020_values=None,
-                 gecko_aliases=None,
-                 servo_2013_aliases=None,
-                 servo_2020_aliases=None,
-                 gecko_strip_moz_prefix=None,
-                 gecko_inexhaustive=None):
+    def __init__(
+        self,
+        name,
+        values,
+        gecko_constant_prefix=None,
+        gecko_enum_prefix=None,
+        custom_consts=None,
+        extra_gecko_values=None,
+        extra_servo_2013_values=None,
+        extra_servo_2020_values=None,
+        gecko_aliases=None,
+        servo_2013_aliases=None,
+        servo_2020_aliases=None,
+        gecko_strip_moz_prefix=None,
+        gecko_inexhaustive=None,
+    ):
         self.name = name
         self.values = values.split()
         if gecko_constant_prefix and gecko_enum_prefix:
-            raise TypeError("Only one of gecko_constant_prefix and gecko_enum_prefix "
-                            "can be specified")
-        self.gecko_constant_prefix = gecko_constant_prefix or \
-            "NS_STYLE_" + self.name.upper().replace("-", "_")
+            raise TypeError(
+                "Only one of gecko_constant_prefix and gecko_enum_prefix "
+                "can be specified"
+            )
+        self.gecko_constant_prefix = (
+            gecko_constant_prefix or "NS_STYLE_" + self.name.upper().replace("-", "_")
+        )
         self.gecko_enum_prefix = gecko_enum_prefix
         self.extra_gecko_values = (extra_gecko_values or "").split()
         self.extra_servo_2013_values = (extra_servo_2013_values or "").split()
@@ -97,8 +136,9 @@ class Keyword(object):
         self.servo_2013_aliases = parse_aliases(servo_2013_aliases or "")
         self.servo_2020_aliases = parse_aliases(servo_2020_aliases or "")
         self.consts_map = {} if custom_consts is None else custom_consts
-        self.gecko_strip_moz_prefix = True \
-            if gecko_strip_moz_prefix is None else gecko_strip_moz_prefix
+        self.gecko_strip_moz_prefix = (
+            True if gecko_strip_moz_prefix is None else gecko_strip_moz_prefix
+        )
         self.gecko_inexhaustive = gecko_inexhaustive or (gecko_enum_prefix is None)
 
     def values_for(self, engine):
@@ -122,11 +162,14 @@ class Keyword(object):
             raise Exception("Bad engine: " + engine)
 
     def gecko_constant(self, value):
-        moz_stripped = (value.replace("-moz-", '')
-                        if self.gecko_strip_moz_prefix else value.replace("-moz-", 'moz-'))
+        moz_stripped = (
+            value.replace("-moz-", "")
+            if self.gecko_strip_moz_prefix
+            else value.replace("-moz-", "moz-")
+        )
         mapped = self.consts_map.get(value)
         if self.gecko_enum_prefix:
-            parts = moz_stripped.replace('-', '_').split('_')
+            parts = moz_stripped.replace("-", "_").split("_")
             parts = mapped if mapped else [p.title() for p in parts]
             return self.gecko_enum_prefix + "::" + "".join(parts)
         else:
@@ -146,13 +189,19 @@ class Keyword(object):
         if self.gecko_enum_prefix is None:
             return cast_type.upper() + "_" + self.gecko_constant(value)
         else:
-            return cast_type.upper() + "_" + self.gecko_constant(value).upper().replace("::", "_")
+            return (
+                cast_type.upper()
+                + "_"
+                + self.gecko_constant(value).upper().replace("::", "_")
+            )
 
 
 def arg_to_bool(arg):
     if isinstance(arg, bool):
         return arg
-    assert arg in ["True", "False"], "Unexpected value for boolean arguement: " + repr(arg)
+    assert arg in ["True", "False"], "Unexpected value for boolean arguement: " + repr(
+        arg
+    )
     return arg == "True"
 
 
@@ -169,41 +218,30 @@ def to_phys(name, logical, physical):
     return name.replace(logical, physical).replace("inset-", "")
 
 
-class Longhand(object):
-    def __init__(self, style_struct, name, spec=None, animation_value_type=None, keyword=None,
-                 predefined_type=None,
-                 servo_2013_pref=None,
-                 servo_2020_pref=None,
-                 gecko_pref=None,
-                 enabled_in="content", need_index=False,
-                 gecko_ffi_name=None,
-                 has_effect_on_gecko_scrollbars=None,
-                 allowed_in_keyframe_block=True, cast_type='u8',
-                 logical=False, logical_group=None, alias=None, extra_prefixes=None, boxed=False,
-                 flags=None, allowed_in_page_rule=False, allow_quirks="No",
-                 ignored_when_colors_disabled=False,
-                 simple_vector_bindings=False,
-                 vector=False, servo_restyle_damage="repaint"):
+class Property(object):
+    def __init__(
+        self,
+        name,
+        spec,
+        servo_2013_pref,
+        servo_2020_pref,
+        gecko_pref,
+        enabled_in,
+        rule_types_allowed,
+        aliases,
+        extra_prefixes,
+        flags,
+    ):
         self.name = name
         if not spec:
-            raise TypeError("Spec should be specified for %s" % name)
+            raise TypeError("Spec should be specified for " + name)
         self.spec = spec
-        self.keyword = keyword
-        self.predefined_type = predefined_type
         self.ident = to_rust_ident(name)
         self.camel_case = to_camel_case(self.ident)
-        self.style_struct = style_struct
         self.servo_2013_pref = servo_2013_pref
         self.servo_2020_pref = servo_2020_pref
         self.gecko_pref = gecko_pref
-        self.has_effect_on_gecko_scrollbars = has_effect_on_gecko_scrollbars
-        assert (
-            has_effect_on_gecko_scrollbars in [None, False, True] and
-            not style_struct.inherited or
-            (gecko_pref is None) == (has_effect_on_gecko_scrollbars is None)), (
-            "Property " + name + ": has_effect_on_gecko_scrollbars must be " +
-            "specified, and must have a value of True or False, iff a " +
-            "property is inherited and is behind a Gecko pref")
+        self.rule_types_allowed = rule_values_from_arg(rule_types_allowed)
         # For enabled_in, the setup is as follows:
         # It needs to be one of the four values: ["", "ua", "chrome", "content"]
         #  * "chrome" implies "ua", and implies that they're explicitly
@@ -211,8 +249,100 @@ class Longhand(object):
         #  * "" implies the property will never be parsed.
         #  * "content" implies the property is accessible unconditionally,
         #    modulo a pref, set via servo_pref / gecko_pref.
-        assert enabled_in in ["", "ua", "chrome", "content"]
+        assert enabled_in in ("", "ua", "chrome", "content")
         self.enabled_in = enabled_in
+        self.aliases = parse_property_aliases(aliases)
+        self.extra_prefixes = parse_property_aliases(extra_prefixes)
+        self.flags = flags.split() if flags else []
+
+    def rule_types_allowed_names(self):
+        for name in RULE_VALUES:
+            if self.rule_types_allowed & RULE_VALUES[name] != 0:
+                yield name
+
+    def experimental(self, engine):
+        if engine == "gecko":
+            return bool(self.gecko_pref)
+        elif engine == "servo-2013":
+            return bool(self.servo_2013_pref)
+        elif engine == "servo-2020":
+            return bool(self.servo_2020_pref)
+        else:
+            raise Exception("Bad engine: " + engine)
+
+    def explicitly_enabled_in_ua_sheets(self):
+        return self.enabled_in in ("ua", "chrome")
+
+    def explicitly_enabled_in_chrome(self):
+        return self.enabled_in == "chrome"
+
+    def enabled_in_content(self):
+        return self.enabled_in == "content"
+
+    def nscsspropertyid(self):
+        return "nsCSSPropertyID::eCSSProperty_" + self.ident
+
+
+class Longhand(Property):
+    def __init__(
+        self,
+        style_struct,
+        name,
+        spec=None,
+        animation_value_type=None,
+        keyword=None,
+        predefined_type=None,
+        servo_2013_pref=None,
+        servo_2020_pref=None,
+        gecko_pref=None,
+        enabled_in="content",
+        need_index=False,
+        gecko_ffi_name=None,
+        has_effect_on_gecko_scrollbars=None,
+        rule_types_allowed=DEFAULT_RULES,
+        cast_type="u8",
+        logical=False,
+        logical_group=None,
+        aliases=None,
+        extra_prefixes=None,
+        boxed=False,
+        flags=None,
+        allow_quirks="No",
+        ignored_when_colors_disabled=False,
+        simple_vector_bindings=False,
+        vector=False,
+        servo_restyle_damage="repaint",
+    ):
+        Property.__init__(
+            self,
+            name=name,
+            spec=spec,
+            servo_2013_pref=servo_2013_pref,
+            servo_2020_pref=servo_2020_pref,
+            gecko_pref=gecko_pref,
+            enabled_in=enabled_in,
+            rule_types_allowed=rule_types_allowed,
+            aliases=aliases,
+            extra_prefixes=extra_prefixes,
+            flags=flags,
+        )
+
+        self.keyword = keyword
+        self.predefined_type = predefined_type
+        self.style_struct = style_struct
+        self.has_effect_on_gecko_scrollbars = has_effect_on_gecko_scrollbars
+        assert (
+            has_effect_on_gecko_scrollbars in [None, False, True]
+            and not style_struct.inherited
+            or (gecko_pref is None and enabled_in != "")
+            == (has_effect_on_gecko_scrollbars is None)
+        ), (
+            "Property "
+            + name
+            + ": has_effect_on_gecko_scrollbars must be "
+            + "specified, and must have a value of True or False, iff a "
+            + "property is inherited and is behind a Gecko pref or internal"
+        )
         self.need_index = need_index
         self.gecko_ffi_name = gecko_ffi_name or "m" + self.camel_case
         self.cast_type = cast_type
@@ -221,34 +351,28 @@ class Longhand(object):
         if self.logical:
             assert logical_group, "Property " + name + " must have a logical group"
 
-        self.alias = parse_property_aliases(alias)
-        self.extra_prefixes = parse_property_aliases(extra_prefixes)
         self.boxed = arg_to_bool(boxed)
-        self.flags = flags.split() if flags else []
-        self.allowed_in_page_rule = arg_to_bool(allowed_in_page_rule)
         self.allow_quirks = allow_quirks
         self.ignored_when_colors_disabled = ignored_when_colors_disabled
         self.is_vector = vector
         self.simple_vector_bindings = simple_vector_bindings
 
-        # https://drafts.csswg.org/css-animations/#keyframes
-        # > The <declaration-list> inside of <keyframe-block> accepts any CSS property
-        # > except those defined in this specification,
-        # > but does accept the `animation-play-state` property and interprets it specially.
-        self.allowed_in_keyframe_block = allowed_in_keyframe_block \
-            and allowed_in_keyframe_block != "False"
-
         # This is done like this since just a plain bool argument seemed like
         # really random.
         if animation_value_type is None:
-            raise TypeError("animation_value_type should be specified for (" + name + ")")
+            raise TypeError(
+                "animation_value_type should be specified for (" + name + ")"
+            )
         self.animation_value_type = animation_value_type
 
         self.animatable = animation_value_type != "none"
-        self.transitionable = animation_value_type != "none" \
-            and animation_value_type != "discrete"
-        self.is_animatable_with_computed_value = animation_value_type == "ComputedValue" \
+        self.transitionable = (
+            animation_value_type != "none" and animation_value_type != "discrete"
+        )
+        self.is_animatable_with_computed_value = (
+            animation_value_type == "ComputedValue"
             or animation_value_type == "discrete"
+        )
 
         # See compute_damage for the various values this can take
         self.servo_restyle_damage = servo_restyle_damage
@@ -263,45 +387,39 @@ class Longhand(object):
         if not self.logical:
             return []
 
-        candidates = [s for s in LOGICAL_SIDES + LOGICAL_SIZES + LOGICAL_CORNERS
-                      if s in self.name] + [s for s in LOGICAL_AXES if self.name.endswith(s)]
-        assert(len(candidates) == 1)
+        candidates = [
+            s for s in LOGICAL_SIDES + LOGICAL_SIZES + LOGICAL_CORNERS if s in self.name
+        ] + [s for s in LOGICAL_AXES if self.name.endswith(s)]
+        assert len(candidates) == 1
         logical_side = candidates[0]
 
-        physical = PHYSICAL_SIDES if logical_side in LOGICAL_SIDES \
-            else PHYSICAL_SIZES if logical_side in LOGICAL_SIZES \
-            else PHYSICAL_AXES if logical_side in LOGICAL_AXES \
+        physical = (
+            PHYSICAL_SIDES
+            if logical_side in LOGICAL_SIDES
+            else PHYSICAL_SIZES
+            if logical_side in LOGICAL_SIZES
+            else PHYSICAL_AXES
+            if logical_side in LOGICAL_AXES
             else LOGICAL_CORNERS
-        return [data.longhands_by_name[to_phys(self.name, logical_side, physical_side)]
-                for physical_side in physical]
-
-    def experimental(self, engine):
-        if engine == "gecko":
-            return bool(self.gecko_pref)
-        elif engine == "servo-2013":
-            return bool(self.servo_2013_pref)
-        elif engine == "servo-2020":
-            return bool(self.servo_2020_pref)
-        else:
-            raise Exception("Bad engine: " + engine)
-
-    # FIXME(emilio): Shorthand and Longhand should really share a base class.
-    def explicitly_enabled_in_ua_sheets(self):
-        return self.enabled_in in ["ua", "chrome"]
-
-    def explicitly_enabled_in_chrome(self):
-        return self.enabled_in == "chrome"
-
-    def enabled_in_content(self):
-        return self.enabled_in == "content"
+        )
+        return [
+            data.longhands_by_name[to_phys(self.name, logical_side, physical_side)]
+            for physical_side in physical
+        ]
 
     def may_be_disabled_in(self, shorthand, engine):
         if engine == "gecko":
             return self.gecko_pref and self.gecko_pref != shorthand.gecko_pref
         elif engine == "servo-2013":
-            return self.servo_2013_pref and self.servo_2013_pref != shorthand.servo_2013_pref
+            return (
+                self.servo_2013_pref
+                and self.servo_2013_pref != shorthand.servo_2013_pref
+            )
         elif engine == "servo-2020":
-            return self.servo_2020_pref and self.servo_2020_pref != shorthand.servo_2020_pref
+            return (
+                self.servo_2020_pref
+                and self.servo_2020_pref != shorthand.servo_2020_pref
+            )
         else:
             raise Exception("Bad engine: " + engine)
 
@@ -329,21 +447,25 @@ class Longhand(object):
                 "AlignSelf",
                 "Appearance",
                 "AspectRatio",
+                "BaselineSource",
                 "BreakBetween",
                 "BreakWithin",
                 "BackgroundRepeat",
                 "BorderImageRepeat",
                 "BorderStyle",
+                "table::CaptionSide",
                 "Clear",
                 "ColumnCount",
                 "Contain",
+                "ContentVisibility",
+                "ContainerType",
                 "Display",
                 "FillRule",
                 "Float",
+                "FontLanguageOverride",
                 "FontSizeAdjust",
                 "FontStretch",
                 "FontStyle",
-                "FontStyleAdjust",
                 "FontSynthesis",
                 "FontVariantEastAsian",
                 "FontVariantLigatures",
@@ -351,16 +473,18 @@ class Longhand(object):
                 "FontWeight",
                 "GreaterThanOrEqualToOneNumber",
                 "GridAutoFlow",
+                "ImageRendering",
                 "InitialLetter",
                 "Integer",
                 "JustifyContent",
                 "JustifyItems",
                 "JustifySelf",
                 "LineBreak",
+                "LineClamp",
                 "MasonryAutoFlow",
-                "MozForceBrokenImageIcon",
-                "MozListReversed",
-                "MozScriptLevel",
+                "BoolInteger",
+                "text::MozControlCharacterVisibility",
+                "MathDepth",
                 "MozScriptMinSize",
                 "MozScriptSizeMultiplier",
                 "TextDecorationSkipInk",
@@ -373,19 +497,25 @@ class Longhand(object):
                 "OverflowClipBox",
                 "OverflowWrap",
                 "OverscrollBehavior",
+                "PageOrientation",
                 "Percentage",
-                "PositiveIntegerOrNone",
+                "PrintColorAdjust",
+                "ForcedColorAdjust",
                 "Resize",
+                "RubyPosition",
                 "SVGOpacity",
                 "SVGPaintOrder",
+                "ScrollbarGutter",
                 "ScrollSnapAlign",
                 "ScrollSnapAxis",
+                "ScrollSnapStop",
                 "ScrollSnapStrictness",
                 "ScrollSnapType",
                 "TextAlign",
                 "TextAlignLast",
                 "TextDecorationLine",
                 "TextEmphasisPosition",
+                "TextJustify",
                 "TextTransform",
                 "TextUnderlinePosition",
                 "TouchAction",
@@ -393,7 +523,7 @@ class Longhand(object):
                 "UserSelect",
                 "WordBreak",
                 "XSpan",
-                "XTextZoom",
+                "XTextScale",
                 "ZIndex",
             }
         if self.name == "overflow-y":
@@ -407,41 +537,36 @@ class Longhand(object):
             return computed
         return "<{} as ToAnimatedValue>::AnimatedValue".format(computed)
 
-    def nscsspropertyid(self):
-        return "nsCSSPropertyID::eCSSProperty_%s" % self.ident
 
-
-class Shorthand(object):
-    def __init__(self, name, sub_properties, spec=None,
-                 servo_2013_pref=None,
-                 servo_2020_pref=None,
-                 gecko_pref=None,
-                 enabled_in="content",
-                 allowed_in_keyframe_block=True, alias=None, extra_prefixes=None,
-                 allowed_in_page_rule=False, flags=None):
-        self.name = name
-        if not spec:
-            raise TypeError("Spec should be specified for %s" % name)
-        self.spec = spec
-        self.ident = to_rust_ident(name)
-        self.camel_case = to_camel_case(self.ident)
-        self.servo_2013_pref = servo_2013_pref
-        self.servo_2020_pref = servo_2020_pref
-        self.gecko_pref = gecko_pref
+class Shorthand(Property):
+    def __init__(
+        self,
+        name,
+        sub_properties,
+        spec=None,
+        servo_2013_pref=None,
+        servo_2020_pref=None,
+        gecko_pref=None,
+        enabled_in="content",
+        rule_types_allowed=DEFAULT_RULES,
+        aliases=None,
+        extra_prefixes=None,
+        flags=None,
+    ):
+        Property.__init__(
+            self,
+            name=name,
+            spec=spec,
+            servo_2013_pref=servo_2013_pref,
+            servo_2020_pref=servo_2020_pref,
+            gecko_pref=gecko_pref,
+            enabled_in=enabled_in,
+            rule_types_allowed=rule_types_allowed,
+            aliases=aliases,
+            extra_prefixes=extra_prefixes,
+            flags=flags,
+        )
         self.sub_properties = sub_properties
-        assert enabled_in in ["", "ua", "chrome", "content"]
-        self.enabled_in = enabled_in
-        self.alias = parse_property_aliases(alias)
-        self.extra_prefixes = parse_property_aliases(extra_prefixes)
-        self.allowed_in_page_rule = arg_to_bool(allowed_in_page_rule)
-        self.flags = flags.split() if flags else []
-
-        # https://drafts.csswg.org/css-animations/#keyframes
-        # > The <declaration-list> inside of <keyframe-block> accepts any CSS property
-        # > except those defined in this specification,
-        # > but does accept the `animation-play-state` property and interprets it specially.
-        self.allowed_in_keyframe_block = allowed_in_keyframe_block \
-            and allowed_in_keyframe_block != "False"
 
     def get_animatable(self):
         for sub in self.sub_properties:
@@ -464,29 +589,6 @@ class Shorthand(object):
     def type():
         return "shorthand"
 
-    def experimental(self, engine):
-        if engine == "gecko":
-            return bool(self.gecko_pref)
-        elif engine == "servo-2013":
-            return bool(self.servo_2013_pref)
-        elif engine == "servo-2020":
-            return bool(self.servo_2020_pref)
-        else:
-            raise Exception("Bad engine: " + engine)
-
-    # FIXME(emilio): Shorthand and Longhand should really share a base class.
-    def explicitly_enabled_in_ua_sheets(self):
-        return self.enabled_in in ["ua", "chrome"]
-
-    def explicitly_enabled_in_chrome(self):
-        return self.enabled_in == "chrome"
-
-    def enabled_in_content(self):
-        return self.enabled_in == "content"
-
-    def nscsspropertyid(self):
-        return "nsCSSPropertyID::eCSSProperty_%s" % self.ident
-
 
 class Alias(object):
     def __init__(self, name, original, gecko_pref):
@@ -500,13 +602,18 @@ class Alias(object):
         self.servo_2020_pref = original.servo_2020_pref
         self.gecko_pref = gecko_pref
         self.transitionable = original.transitionable
-        self.allowed_in_page_rule = original.allowed_in_page_rule
-        self.allowed_in_keyframe_block = original.allowed_in_keyframe_block
+        self.rule_types_allowed = original.rule_types_allowed
+        self.flags = original.flags
 
     @staticmethod
     def type():
         return "alias"
 
+    def rule_types_allowed_names(self):
+        for name in RULE_VALUES:
+            if self.rule_types_allowed & RULE_VALUES[name] != 0:
+                yield name
+
     def experimental(self, engine):
         if engine == "gecko":
             return bool(self.gecko_pref)
@@ -580,7 +687,9 @@ class PropertiesData(object):
         self.shorthands = []
         self.shorthands_by_name = {}
         self.shorthand_aliases = []
-        self.counted_unknown_properties = [CountedUnknownProperty(p) for p in COUNTED_UNKNOWN_PROPERTIES]
+        self.counted_unknown_properties = [
+            CountedUnknownProperty(p) for p in COUNTED_UNKNOWN_PROPERTIES
+        ]
 
     def new_style_struct(self, *args, **kwargs):
         style_struct = StyleStruct(*args, **kwargs)
@@ -595,7 +704,7 @@ class PropertiesData(object):
         #       See servo/servo#14941.
         if self.engine == "gecko":
             for (prefix, pref) in property.extra_prefixes:
-                property.alias.append(('-%s-%s' % (prefix, property.name), pref))
+                property.aliases.append(("-%s-%s" % (prefix, property.name), pref))
 
     def declare_longhand(self, name, engines=None, **kwargs):
         engines = engines.split()
@@ -604,13 +713,15 @@ class PropertiesData(object):
 
         longhand = Longhand(self.current_style_struct, name, **kwargs)
         self.add_prefixed_aliases(longhand)
-        longhand.alias = [Alias(xp[0], longhand, xp[1]) for xp in longhand.alias]
-        self.longhand_aliases += longhand.alias
+        longhand.aliases = [Alias(xp[0], longhand, xp[1]) for xp in longhand.aliases]
+        self.longhand_aliases += longhand.aliases
         self.current_style_struct.longhands.append(longhand)
         self.longhands.append(longhand)
         self.longhands_by_name[name] = longhand
         if longhand.logical_group:
-            self.longhands_by_logical_group.setdefault(longhand.logical_group, []).append(longhand)
+            self.longhands_by_logical_group.setdefault(
+                longhand.logical_group, []
+            ).append(longhand)
 
         return longhand
 
@@ -622,8 +733,8 @@ class PropertiesData(object):
         sub_properties = [self.longhands_by_name[s] for s in sub_properties]
         shorthand = Shorthand(name, sub_properties, *args, **kwargs)
         self.add_prefixed_aliases(shorthand)
-        shorthand.alias = [Alias(xp[0], shorthand, xp[1]) for xp in shorthand.alias]
-        self.shorthand_aliases += shorthand.alias
+        shorthand.aliases = [Alias(xp[0], shorthand, xp[1]) for xp in shorthand.aliases]
+        self.shorthand_aliases += shorthand.aliases
         self.shorthands.append(shorthand)
         self.shorthands_by_name[name] = shorthand
         return shorthand
@@ -652,7 +763,7 @@ def _add_logical_props(data, props):
 # These are probably Gecko bugs and should be supported per spec.
 def _remove_common_first_line_and_first_letter_properties(props, engine):
     if engine == "gecko":
-        props.remove("-moz-tab-size")
+        props.remove("tab-size")
         props.remove("hyphens")
         props.remove("line-break")
         props.remove("text-align-last")
@@ -686,31 +797,33 @@ class PropertyRestrictions:
     # https://drafts.csswg.org/css-pseudo/#first-letter-styling
     @staticmethod
     def first_letter(data):
-        props = set([
-            "color",
-            "opacity",
-            "float",
-            "initial-letter",
-
-            # Kinda like css-fonts?
-            "-moz-osx-font-smoothing",
-
-            # Kinda like css-text?
-            "-webkit-text-stroke-width",
-            "-webkit-text-fill-color",
-            "-webkit-text-stroke-color",
-            "vertical-align",
-            "line-height",
-
-            # Kinda like css-backgrounds?
-            "background-blend-mode",
-        ] + PropertyRestrictions.shorthand(data, "padding")
-          + PropertyRestrictions.shorthand(data, "margin")
-          + PropertyRestrictions.spec(data, "css-fonts")
-          + PropertyRestrictions.spec(data, "css-backgrounds")
-          + PropertyRestrictions.spec(data, "css-text")
-          + PropertyRestrictions.spec(data, "css-shapes")
-          + PropertyRestrictions.spec(data, "css-text-decor"))
+        props = set(
+            [
+                "color",
+                "opacity",
+                "float",
+                "initial-letter",
+                # Kinda like css-fonts?
+                "-moz-osx-font-smoothing",
+                # Kinda like css-text?
+                "-webkit-text-stroke-width",
+                "-webkit-text-fill-color",
+                "-webkit-text-stroke-color",
+                "vertical-align",
+                # Will become shorthand of vertical-align (Bug 1830771)
+                "baseline-source",
+                "line-height",
+                # Kinda like css-backgrounds?
+                "background-blend-mode",
+            ]
+            + PropertyRestrictions.shorthand(data, "padding")
+            + PropertyRestrictions.shorthand(data, "margin")
+            + PropertyRestrictions.spec(data, "css-fonts")
+            + PropertyRestrictions.spec(data, "css-backgrounds")
+            + PropertyRestrictions.spec(data, "css-text")
+            + PropertyRestrictions.spec(data, "css-shapes")
+            + PropertyRestrictions.spec(data, "css-text-decor")
+        )
 
         _add_logical_props(data, props)
 
@@ -720,27 +833,29 @@ class PropertyRestrictions:
     # https://drafts.csswg.org/css-pseudo/#first-line-styling
     @staticmethod
     def first_line(data):
-        props = set([
-            # Per spec.
-            "color",
-            "opacity",
-
-            # Kinda like css-fonts?
-            "-moz-osx-font-smoothing",
-
-            # Kinda like css-text?
-            "-webkit-text-stroke-width",
-            "-webkit-text-fill-color",
-            "-webkit-text-stroke-color",
-            "vertical-align",
-            "line-height",
-
-            # Kinda like css-backgrounds?
-            "background-blend-mode",
-        ] + PropertyRestrictions.spec(data, "css-fonts")
-          + PropertyRestrictions.spec(data, "css-backgrounds")
-          + PropertyRestrictions.spec(data, "css-text")
-          + PropertyRestrictions.spec(data, "css-text-decor"))
+        props = set(
+            [
+                # Per spec.
+                "color",
+                "opacity",
+                # Kinda like css-fonts?
+                "-moz-osx-font-smoothing",
+                # Kinda like css-text?
+                "-webkit-text-stroke-width",
+                "-webkit-text-fill-color",
+                "-webkit-text-stroke-color",
+                "vertical-align",
+                # Will become shorthand of vertical-align (Bug 1830771)
+                "baseline-source",
+                "line-height",
+                # Kinda like css-backgrounds?
+                "background-blend-mode",
+            ]
+            + PropertyRestrictions.spec(data, "css-fonts")
+            + PropertyRestrictions.spec(data, "css-backgrounds")
+            + PropertyRestrictions.spec(data, "css-text")
+            + PropertyRestrictions.spec(data, "css-text-decor")
+        )
 
         # These are probably Gecko bugs and should be supported per spec.
         for prop in PropertyRestrictions.shorthand(data, "border"):
@@ -770,39 +885,48 @@ class PropertyRestrictions:
     # https://drafts.csswg.org/css-pseudo/#marker-pseudo
     @staticmethod
     def marker(data):
-        return set([
-            "color",
-            "text-combine-upright",
-            "unicode-bidi",
-            "direction",
-            "content",
-            "-moz-osx-font-smoothing",
-        ] + PropertyRestrictions.spec(data, "css-fonts"))
+        return set(
+            [
+                "white-space",
+                "color",
+                "text-combine-upright",
+                "text-transform",
+                "unicode-bidi",
+                "direction",
+                "content",
+                "line-height",
+                "-moz-osx-font-smoothing",
+            ]
+            + PropertyRestrictions.spec(data, "css-fonts")
+            + PropertyRestrictions.spec(data, "css-animations")
+            + PropertyRestrictions.spec(data, "css-transitions")
+        )
 
     # https://www.w3.org/TR/webvtt1/#the-cue-pseudo-element
     @staticmethod
     def cue(data):
-        return set([
-            "color",
-            "opacity",
-            "visibility",
-            "text-shadow",
-            "white-space",
-            "text-combine-upright",
-            "ruby-position",
-
-            # XXX Should these really apply to cue?
-            "font-synthesis",
-            "-moz-osx-font-smoothing",
-
-            # FIXME(emilio): background-blend-mode should be part of the
-            # background shorthand, and get reset, per
-            # https://drafts.fxtf.org/compositing/#background-blend-mode
-            "background-blend-mode",
-        ] + PropertyRestrictions.shorthand(data, "text-decoration")
-          + PropertyRestrictions.shorthand(data, "background")
-          + PropertyRestrictions.shorthand(data, "outline")
-          + PropertyRestrictions.shorthand(data, "font"))
+        return set(
+            [
+                "color",
+                "opacity",
+                "visibility",
+                "text-shadow",
+                "white-space",
+                "text-combine-upright",
+                "ruby-position",
+                # XXX Should these really apply to cue?
+                "-moz-osx-font-smoothing",
+                # FIXME(emilio): background-blend-mode should be part of the
+                # background shorthand, and get reset, per
+                # https://drafts.fxtf.org/compositing/#background-blend-mode
+                "background-blend-mode",
+            ]
+            + PropertyRestrictions.shorthand(data, "text-decoration")
+            + PropertyRestrictions.shorthand(data, "background")
+            + PropertyRestrictions.shorthand(data, "outline")
+            + PropertyRestrictions.shorthand(data, "font")
+            + PropertyRestrictions.shorthand(data, "font-synthesis")
+        )
 
 
 class CountedUnknownProperty:
