@@ -3,32 +3,29 @@
  * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
 
 <%namespace name="helpers" file="/helpers.mako.rs" />
-<% from data import Keyword, Method, to_rust_ident, to_camel_case%>
+<% from data import ALL_AXES, Keyword, Method, to_rust_ident, to_camel_case%>
 
 <% data.new_style_struct("Box",
                          inherited=False,
                          gecko_name="Display") %>
 
-// We allow "display" to apply to placeholders because we need to make the
-// placeholder pseudo-element an inline-block in the UA stylesheet in Gecko.
 ${helpers.predefined_type(
     "display",
     "Display",
     "computed::Display::inline()",
+    engines="gecko servo-2013 servo-2020",
     initial_specified_value="specified::Display::inline()",
     animation_value_type="discrete",
-    flags="APPLIES_TO_PLACEHOLDER",
     spec="https://drafts.csswg.org/css-display/#propdef-display",
     servo_restyle_damage="rebuild_and_reflow",
-    needs_context=product == "gecko"
 )}
 
 ${helpers.single_keyword(
     "-moz-top-layer",
     "none top",
-    gecko_constant_prefix="NS_STYLE_TOP_LAYER",
+    engines="gecko",
+    gecko_enum_prefix="StyleTopLayer",
     gecko_ffi_name="mTopLayer",
-    products="gecko",
     animation_value_type="none",
     enabled_in="ua",
     spec="Internal (not web-exposed)",
@@ -39,30 +36,40 @@ ${helpers.single_keyword(
 ${helpers.single_keyword(
     "-servo-top-layer",
     "none top",
-    products="servo",
+    engines="servo-2013 servo-2020",
     animation_value_type="none",
     enabled_in="ua",
     spec="Internal (not web-exposed)",
 )}
 
-${helpers.single_keyword(
-    "position",
-    "static absolute relative fixed sticky",
-    animation_value_type="discrete",
-    flags="CREATES_STACKING_CONTEXT ABSPOS_CB",
-    spec="https://drafts.csswg.org/css-position/#position-property",
-    servo_restyle_damage="rebuild_and_reflow",
-)}
+<%helpers:single_keyword
+    name="position"
+    values="static absolute relative fixed ${'sticky' if engine in ['gecko', 'servo-2013'] else ''}"
+    engines="gecko servo-2013 servo-2020"
+    animation_value_type="discrete"
+    gecko_enum_prefix="StylePositionProperty"
+    spec="https://drafts.csswg.org/css-position/#position-property"
+    servo_restyle_damage="rebuild_and_reflow"
+>
+impl computed_value::T {
+    pub fn is_absolutely_positioned(self) -> bool {
+        matches!(self, Self::Absolute | Self::Fixed)
+    }
+    pub fn is_relative(self) -> bool {
+        self == Self::Relative
+    }
+}
+</%helpers:single_keyword>
 
 ${helpers.predefined_type(
     "float",
     "Float",
     "computed::Float::None",
+    engines="gecko servo-2013 servo-2020",
+    servo_2020_pref="layout.2020.unimplemented",
     initial_specified_value="specified::Float::None",
     spec="https://drafts.csswg.org/css-box/#propdef-float",
     animation_value_type="discrete",
-    needs_context=False,
-    flags="APPLIES_TO_FIRST_LETTER",
     servo_restyle_damage="rebuild_and_reflow",
     gecko_ffi_name="mFloat",
 )}
@@ -71,10 +78,9 @@ ${helpers.predefined_type(
     "clear",
     "Clear",
     "computed::Clear::None",
+    engines="gecko servo-2013",
     animation_value_type="discrete",
-    needs_context=False,
-    gecko_ffi_name="mBreakType",
-    spec="https://drafts.csswg.org/css-box/#propdef-clear",
+    spec="https://drafts.csswg.org/css2/#propdef-clear",
     servo_restyle_damage="rebuild_and_reflow",
 )}
 
@@ -82,28 +88,41 @@ ${helpers.predefined_type(
     "vertical-align",
     "VerticalAlign",
     "computed::VerticalAlign::baseline()",
+    engines="gecko servo-2013",
     animation_value_type="ComputedValue",
-    flags="APPLIES_TO_FIRST_LETTER APPLIES_TO_FIRST_LINE APPLIES_TO_PLACEHOLDER",
     spec="https://www.w3.org/TR/CSS2/visudet.html#propdef-vertical-align",
     servo_restyle_damage = "reflow",
 )}
 
+${helpers.predefined_type(
+    "baseline-source",
+    "BaselineSource",
+    "computed::BaselineSource::Auto",
+    engines="gecko servo-2013",
+    animation_value_type="discrete",
+    spec="https://drafts.csswg.org/css-inline-3/#baseline-source",
+    servo_restyle_damage = "reflow",
+)}
+
 // CSS 2.1, Section 11 - Visual effects
 
-${helpers.single_keyword("-servo-overflow-clip-box", "padding-box content-box",
-    products="servo", animation_value_type="none", enabled_in="ua",
+${helpers.single_keyword(
+    "-servo-overflow-clip-box",
+    "padding-box content-box",
+    engines="servo-2013",
+    animation_value_type="none",
+    enabled_in="ua",
     spec="Internal, not web-exposed, \
-          may be standardized in the future (https://developer.mozilla.org/en-US/docs/Web/CSS/overflow-clip-box)")}
+          may be standardized in the future (https://developer.mozilla.org/en-US/docs/Web/CSS/overflow-clip-box)",
+)}
 
 % for direction in ["inline", "block"]:
     ${helpers.predefined_type(
         "overflow-clip-box-" + direction,
         "OverflowClipBox",
         "computed::OverflowClipBox::PaddingBox",
-        products="gecko",
+        engines="gecko",
         enabled_in="ua",
-        needs_context=False,
-        flags="APPLIES_TO_PLACEHOLDER",
         gecko_pref="layout.css.overflow-clip-box.enabled",
         animation_value_type="discrete",
         spec="Internal, may be standardized in the future: \
@@ -111,249 +130,43 @@ ${helpers.single_keyword("-servo-overflow-clip-box", "padding-box content-box",
     )}
 % endfor
 
-// FIXME(pcwalton, #2742): Implement scrolling for `scroll` and `auto`.
-//
-// We allow it to apply to placeholders for UA sheets, which set it !important.
-${helpers.predefined_type(
-    "overflow-x",
-    "Overflow",
-    "computed::Overflow::Visible",
-    animation_value_type="discrete",
-    flags="APPLIES_TO_PLACEHOLDER",
-    spec="https://drafts.csswg.org/css-overflow/#propdef-overflow-x",
-    needs_context=False,
-    servo_restyle_damage = "reflow",
-)}
-
-${helpers.predefined_type(
-    "overflow-y",
-    "Overflow",
-    "computed::Overflow::Visible",
-    animation_value_type="discrete",
-    flags="APPLIES_TO_PLACEHOLDER",
-    spec="https://drafts.csswg.org/css-overflow/#propdef-overflow-y",
-    needs_context=False,
-    servo_restyle_damage = "reflow",
-)}
+% for (axis, logical) in ALL_AXES:
+    <% full_name = "overflow-{}".format(axis) %>
+    ${helpers.predefined_type(
+        full_name,
+        "Overflow",
+        "computed::Overflow::Visible",
+        engines="gecko servo-2013 servo-2020",
+        logical_group="overflow",
+        logical=logical,
+        animation_value_type="discrete",
+        spec="https://drafts.csswg.org/css-overflow-3/#propdef-{}".format(full_name),
+        servo_restyle_damage = "reflow",
+        gecko_pref="layout.css.overflow-logical.enabled" if logical else None,
+    )}
+% endfor
 
 ${helpers.predefined_type(
     "overflow-anchor",
     "OverflowAnchor",
     "computed::OverflowAnchor::Auto",
+    engines="gecko",
     initial_specified_value="specified::OverflowAnchor::Auto",
-    products="gecko",
-    needs_context=False,
     gecko_pref="layout.css.scroll-anchoring.enabled",
     spec="https://drafts.csswg.org/css-scroll-anchoring/#exclusion-api",
     animation_value_type="discrete",
 )}
 
-<% transition_extra_prefixes = "moz:layout.css.prefixes.transitions webkit" %>
-
-${helpers.predefined_type(
-    "transition-duration",
-    "Time",
-    "computed::Time::zero()",
-    initial_specified_value="specified::Time::zero()",
-    parse_method="parse_non_negative",
-    vector=True,
-    need_index=True,
-    animation_value_type="none",
-    extra_prefixes=transition_extra_prefixes,
-    spec="https://drafts.csswg.org/css-transitions/#propdef-transition-duration",
-)}
-
-${helpers.predefined_type(
-    "transition-timing-function",
-    "TimingFunction",
-    "computed::TimingFunction::ease()",
-    initial_specified_value="specified::TimingFunction::ease()",
-    vector=True,
-    need_index=True,
-    animation_value_type="none",
-    extra_prefixes=transition_extra_prefixes,
-    spec="https://drafts.csswg.org/css-transitions/#propdef-transition-timing-function",
-)}
-
-${helpers.predefined_type(
-    "transition-property",
-    "TransitionProperty",
-    "computed::TransitionProperty::all()",
-    initial_specified_value="specified::TransitionProperty::all()",
-    vector=True,
-    allow_empty="NotInitial",
-    need_index=True,
-    animation_value_type="none",
-    extra_prefixes=transition_extra_prefixes,
-    spec="https://drafts.csswg.org/css-transitions/#propdef-transition-property",
-)}
-
-${helpers.predefined_type(
-    "transition-delay",
-    "Time",
-    "computed::Time::zero()",
-    initial_specified_value="specified::Time::zero()",
-    vector=True,
-    need_index=True,
-    animation_value_type="none",
-    extra_prefixes=transition_extra_prefixes,
-    spec="https://drafts.csswg.org/css-transitions/#propdef-transition-delay",
-)}
-
-<% animation_extra_prefixes = "moz:layout.css.prefixes.animations webkit" %>
-
-${helpers.predefined_type(
-    "animation-name",
-    "AnimationName",
-    "computed::AnimationName::none()",
-    initial_specified_value="specified::AnimationName::none()",
-    vector=True,
-    need_index=True,
-    animation_value_type="none",
-    extra_prefixes=animation_extra_prefixes,
-    allowed_in_keyframe_block=False,
-    spec="https://drafts.csswg.org/css-animations/#propdef-animation-name",
-)}
-
-${helpers.predefined_type(
-    "animation-duration",
-    "Time",
-    "computed::Time::zero()",
-    initial_specified_value="specified::Time::zero()",
-    parse_method="parse_non_negative",
-    vector=True,
-    need_index=True,
-    animation_value_type="none",
-    extra_prefixes=animation_extra_prefixes,
-    spec="https://drafts.csswg.org/css-transitions/#propdef-transition-duration",
-)}
-
-// animation-timing-function is the exception to the rule for allowed_in_keyframe_block:
-// https://drafts.csswg.org/css-animations/#keyframes
-${helpers.predefined_type(
-    "animation-timing-function",
-    "TimingFunction",
-    "computed::TimingFunction::ease()",
-    initial_specified_value="specified::TimingFunction::ease()",
-    vector=True,
-    need_index=True,
-    animation_value_type="none",
-    extra_prefixes=animation_extra_prefixes,
-    allowed_in_keyframe_block=True,
-    spec="https://drafts.csswg.org/css-transitions/#propdef-animation-timing-function",
-)}
-
-${helpers.predefined_type(
-    "animation-iteration-count",
-    "AnimationIterationCount",
-    "computed::AnimationIterationCount::one()",
-    initial_specified_value="specified::AnimationIterationCount::one()",
-    vector=True,
-    need_index=True,
-    animation_value_type="none",
-    extra_prefixes=animation_extra_prefixes,
-    allowed_in_keyframe_block=False,
-    spec="https://drafts.csswg.org/css-animations/#propdef-animation-iteration-count",
-)}
-
-<% animation_direction_custom_consts = { "alternate-reverse": "Alternate_reverse" } %>
-${helpers.single_keyword(
-    "animation-direction",
-    "normal reverse alternate alternate-reverse",
-    need_index=True,
-    animation_value_type="none",
-    vector=True,
-    gecko_enum_prefix="PlaybackDirection",
-    custom_consts=animation_direction_custom_consts,
-    extra_prefixes=animation_extra_prefixes,
-    gecko_inexhaustive=True,
-    spec="https://drafts.csswg.org/css-animations/#propdef-animation-direction",
-    allowed_in_keyframe_block=False,
-)}
-
-${helpers.single_keyword(
-    "animation-play-state",
-    "running paused",
-    need_index=True,
-    animation_value_type="none",
-    vector=True,
-    extra_prefixes=animation_extra_prefixes,
-    gecko_enum_prefix="StyleAnimationPlayState",
-    spec="https://drafts.csswg.org/css-animations/#propdef-animation-play-state",
-    allowed_in_keyframe_block=False,
-)}
-
-${helpers.single_keyword(
-    "animation-fill-mode",
-    "none forwards backwards both",
-    need_index=True,
-    animation_value_type="none",
-    vector=True,
-    gecko_enum_prefix="FillMode",
-    extra_prefixes=animation_extra_prefixes,
-    gecko_inexhaustive=True,
-    spec="https://drafts.csswg.org/css-animations/#propdef-animation-fill-mode",
-    allowed_in_keyframe_block=False,
-)}
-
-${helpers.predefined_type(
-    "animation-delay",
-    "Time",
-    "computed::Time::zero()",
-    initial_specified_value="specified::Time::zero()",
-    vector=True,
-    need_index=True,
-    animation_value_type="none",
-    extra_prefixes=animation_extra_prefixes,
-    spec="https://drafts.csswg.org/css-animations/#propdef-animation-delay",
-    allowed_in_keyframe_block=False,
-)}
-
-% for axis in ["x", "y"]:
-    ${helpers.predefined_type(
-        "scroll-snap-points-" + axis,
-        "ScrollSnapPoint",
-        "computed::ScrollSnapPoint::none()",
-        animation_value_type="discrete",
-        gecko_pref="layout.css.scroll-snap.enabled",
-        products="gecko",
-        spec="Nonstandard (https://www.w3.org/TR/2015/WD-css-snappoints-1-20150326/#scroll-snap-points)",
-    )}
-% endfor
-
-${helpers.predefined_type(
-    "scroll-snap-destination",
-    "Position",
-    "computed::Position::zero()",
-    products="gecko",
-    gecko_pref="layout.css.scroll-snap.enabled",
-    boxed=True,
-    spec="Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/scroll-snap-destination)",
-    animation_value_type="discrete",
-)}
-
-${helpers.predefined_type(
-    "scroll-snap-coordinate",
-    "Position",
-    "computed::Position::zero()",
-    vector=True,
-    allow_empty=True,
-    products="gecko",
-    gecko_pref="layout.css.scroll-snap.enabled",
-    spec="Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/scroll-snap-destination)",
-    animation_value_type="discrete",
-)}
-
 <% transform_extra_prefixes = "moz:layout.css.prefixes.transforms webkit" %>
 
 ${helpers.predefined_type(
     "transform",
     "Transform",
     "generics::transform::Transform::none()",
+    engines="gecko servo-2013 servo-2020",
     extra_prefixes=transform_extra_prefixes,
     animation_value_type="ComputedValue",
-    flags="CREATES_STACKING_CONTEXT FIXPOS_CB \
-           GETCS_NEEDS_LAYOUT_FLUSH CAN_ANIMATE_ON_COMPOSITOR",
+    flags="CAN_ANIMATE_ON_COMPOSITOR",
     spec="https://drafts.csswg.org/css-transforms/#propdef-transform",
     servo_restyle_damage="reflow_out_of_flow",
 )}
@@ -362,9 +175,10 @@ ${helpers.predefined_type(
     "rotate",
     "Rotate",
     "generics::transform::Rotate::None",
+    engines="gecko servo-2013",
     animation_value_type="ComputedValue",
     boxed=True,
-    flags="CREATES_STACKING_CONTEXT FIXPOS_CB CAN_ANIMATE_ON_COMPOSITOR",
+    flags="CAN_ANIMATE_ON_COMPOSITOR",
     gecko_pref="layout.css.individual-transform.enabled",
     spec="https://drafts.csswg.org/css-transforms-2/#individual-transforms",
     servo_restyle_damage = "reflow_out_of_flow",
@@ -374,9 +188,10 @@ ${helpers.predefined_type(
     "scale",
     "Scale",
     "generics::transform::Scale::None",
+    engines="gecko servo-2013",
     animation_value_type="ComputedValue",
     boxed=True,
-    flags="CREATES_STACKING_CONTEXT FIXPOS_CB CAN_ANIMATE_ON_COMPOSITOR",
+    flags="CAN_ANIMATE_ON_COMPOSITOR",
     gecko_pref="layout.css.individual-transform.enabled",
     spec="https://drafts.csswg.org/css-transforms-2/#individual-transforms",
     servo_restyle_damage = "reflow_out_of_flow",
@@ -386,9 +201,10 @@ ${helpers.predefined_type(
     "translate",
     "Translate",
     "generics::transform::Translate::None",
+    engines="gecko servo-2013",
     animation_value_type="ComputedValue",
     boxed=True,
-    flags="CREATES_STACKING_CONTEXT FIXPOS_CB CAN_ANIMATE_ON_COMPOSITOR",
+    flags="CAN_ANIMATE_ON_COMPOSITOR",
     gecko_pref="layout.css.individual-transform.enabled",
     spec="https://drafts.csswg.org/css-transforms-2/#individual-transforms",
     servo_restyle_damage="reflow_out_of_flow",
@@ -399,11 +215,65 @@ ${helpers.predefined_type(
     "offset-path",
     "OffsetPath",
     "computed::OffsetPath::none()",
-    products="gecko",
+    engines="gecko",
     animation_value_type="ComputedValue",
     gecko_pref="layout.css.motion-path.enabled",
-    flags="CREATES_STACKING_CONTEXT FIXPOS_CB",
+    flags="CAN_ANIMATE_ON_COMPOSITOR",
     spec="https://drafts.fxtf.org/motion-1/#offset-path-property",
+    servo_restyle_damage="reflow_out_of_flow"
+)}
+
+// Motion Path Module Level 1
+${helpers.predefined_type(
+    "offset-distance",
+    "LengthPercentage",
+    "computed::LengthPercentage::zero()",
+    engines="gecko",
+    animation_value_type="ComputedValue",
+    gecko_pref="layout.css.motion-path.enabled",
+    flags="CAN_ANIMATE_ON_COMPOSITOR",
+    spec="https://drafts.fxtf.org/motion-1/#offset-distance-property",
+    servo_restyle_damage="reflow_out_of_flow"
+)}
+
+// Motion Path Module Level 1
+${helpers.predefined_type(
+    "offset-rotate",
+    "OffsetRotate",
+    "computed::OffsetRotate::auto()",
+    engines="gecko",
+    animation_value_type="ComputedValue",
+    gecko_pref="layout.css.motion-path.enabled",
+    flags="CAN_ANIMATE_ON_COMPOSITOR",
+    spec="https://drafts.fxtf.org/motion-1/#offset-rotate-property",
+    servo_restyle_damage="reflow_out_of_flow"
+)}
+
+// Motion Path Module Level 1
+${helpers.predefined_type(
+    "offset-anchor",
+    "PositionOrAuto",
+    "computed::PositionOrAuto::auto()",
+    engines="gecko",
+    animation_value_type="ComputedValue",
+    gecko_pref="layout.css.motion-path.enabled",
+    flags="CAN_ANIMATE_ON_COMPOSITOR",
+    spec="https://drafts.fxtf.org/motion-1/#offset-anchor-property",
+    servo_restyle_damage="reflow_out_of_flow",
+    boxed=True
+)}
+
+// Motion Path Module Level 1
+${helpers.predefined_type(
+    "offset-position",
+    "OffsetPosition",
+    "computed::OffsetPosition::auto()",
+    engines="gecko",
+    animation_value_type="ComputedValue",
+    gecko_pref="layout.css.motion-path-offset-position.enabled",
+    spec="https://drafts.fxtf.org/motion-1/#offset-position-property",
+    servo_restyle_damage="reflow_out_of_flow",
+    boxed=True
 )}
 
 // CSSOM View Module
@@ -411,17 +281,17 @@ ${helpers.predefined_type(
 ${helpers.single_keyword(
     "scroll-behavior",
     "auto smooth",
-    products="gecko",
+    engines="gecko",
     spec="https://drafts.csswg.org/cssom-view/#propdef-scroll-behavior",
     animation_value_type="discrete",
+    gecko_enum_prefix="StyleScrollBehavior",
 )}
 
 ${helpers.predefined_type(
     "scroll-snap-align",
     "ScrollSnapAlign",
     "computed::ScrollSnapAlign::none()",
-    products="gecko",
-    gecko_pref="layout.css.scroll-snap-v1.enabled",
+    engines="gecko",
     spec="https://drafts.csswg.org/css-scroll-snap-1/#scroll-snap-align",
     animation_value_type="discrete",
 )}
@@ -430,18 +300,28 @@ ${helpers.predefined_type(
     "scroll-snap-type",
     "ScrollSnapType",
     "computed::ScrollSnapType::none()",
-    products="gecko",
+    engines="gecko",
     spec="https://drafts.csswg.org/css-scroll-snap-1/#scroll-snap-type",
     animation_value_type="discrete",
 )}
 
-% for axis in ["x", "y"]:
+${helpers.predefined_type(
+    "scroll-snap-stop",
+    "ScrollSnapStop",
+    "computed::ScrollSnapStop::Normal",
+    engines="gecko",
+    spec="https://drafts.csswg.org/css-scroll-snap-1/#scroll-snap-stop",
+    animation_value_type="discrete",
+)}
+
+% for (axis, logical) in ALL_AXES:
     ${helpers.predefined_type(
         "overscroll-behavior-" + axis,
         "OverscrollBehavior",
         "computed::OverscrollBehavior::Auto",
-        products="gecko",
-        needs_context=False,
+        engines="gecko",
+        logical_group="overscroll-behavior",
+        logical=logical,
         gecko_pref="layout.css.overscroll-behavior.enabled",
         spec="https://wicg.github.io/overscroll-behavior/#overscroll-behavior-properties",
         animation_value_type="discrete",
@@ -453,9 +333,9 @@ ${helpers.predefined_type(
 ${helpers.single_keyword(
     "isolation",
     "auto isolate",
-    products="gecko",
+    engines="gecko",
     spec="https://drafts.fxtf.org/compositing/#isolation",
-    flags="CREATES_STACKING_CONTEXT",
+    gecko_enum_prefix="StyleIsolation",
     animation_value_type="discrete",
 )}
 
@@ -463,8 +343,7 @@ ${helpers.predefined_type(
     "break-after",
     "BreakBetween",
     "computed::BreakBetween::Auto",
-    needs_context=False,
-    products="gecko",
+    engines="gecko",
     spec="https://drafts.csswg.org/css-break/#propdef-break-after",
     animation_value_type="discrete",
 )}
@@ -473,8 +352,7 @@ ${helpers.predefined_type(
     "break-before",
     "BreakBetween",
     "computed::BreakBetween::Auto",
-    needs_context=False,
-    products="gecko",
+    engines="gecko",
     spec="https://drafts.csswg.org/css-break/#propdef-break-before",
     animation_value_type="discrete",
 )}
@@ -483,28 +361,20 @@ ${helpers.predefined_type(
     "break-inside",
     "BreakWithin",
     "computed::BreakWithin::Auto",
-    needs_context=False,
-    products="gecko",
-    alias="page-break-inside",
+    engines="gecko",
     spec="https://drafts.csswg.org/css-break/#propdef-break-inside",
     animation_value_type="discrete",
 )}
 
 // CSS Basic User Interface Module Level 3
 // http://dev.w3.org/csswg/css-ui
-//
-// This is APPLIES_TO_PLACEHOLDER so we can override, in the UA sheet, the
-// 'resize' property we'd inherit from textarea otherwise.  Basically, just
-// makes the UA rules easier to write.
 ${helpers.predefined_type(
     "resize",
     "Resize",
     "computed::Resize::None",
-    products="gecko",
+    engines="gecko",
     animation_value_type="discrete",
-    needs_context=False,
     gecko_ffi_name="mResize",
-    flags="APPLIES_TO_PLACEHOLDER",
     spec="https://drafts.csswg.org/css-ui/#propdef-resize",
 )}
 
@@ -512,10 +382,10 @@ ${helpers.predefined_type(
     "perspective",
     "Perspective",
     "computed::Perspective::none()",
+    engines="gecko servo-2013 servo-2020",
     gecko_ffi_name="mChildPerspective",
     spec="https://drafts.csswg.org/css-transforms/#perspective",
     extra_prefixes=transform_extra_prefixes,
-    flags="CREATES_STACKING_CONTEXT FIXPOS_CB",
     animation_value_type="AnimatedPerspective",
     servo_restyle_damage = "reflow_out_of_flow",
 )}
@@ -524,10 +394,10 @@ ${helpers.predefined_type(
     "perspective-origin",
     "Position",
     "computed::position::Position::center()",
+    engines="gecko servo-2013 servo-2020",
     boxed=True,
     extra_prefixes=transform_extra_prefixes,
     spec="https://drafts.csswg.org/css-transforms-2/#perspective-origin-property",
-    flags="GETCS_NEEDS_LAYOUT_FLUSH",
     animation_value_type="ComputedValue",
     servo_restyle_damage="reflow_out_of_flow"
 )}
@@ -535,6 +405,8 @@ ${helpers.predefined_type(
 ${helpers.single_keyword(
     "backface-visibility",
     "visible hidden",
+    engines="gecko servo-2013 servo-2020",
+    gecko_enum_prefix="StyleBackfaceVisibility",
     spec="https://drafts.csswg.org/css-transforms/#backface-visibility-property",
     extra_prefixes=transform_extra_prefixes,
     animation_value_type="discrete",
@@ -543,9 +415,8 @@ ${helpers.single_keyword(
 ${helpers.single_keyword(
     "transform-box",
     "border-box fill-box view-box",
+    engines="gecko",
     gecko_enum_prefix="StyleGeometryBox",
-    products="gecko",
-    gecko_pref="svg.transform-box.enabled",
     spec="https://drafts.csswg.org/css-transforms/#transform-box",
     gecko_inexhaustive="True",
     animation_value_type="discrete",
@@ -554,11 +425,10 @@ ${helpers.single_keyword(
 ${helpers.predefined_type(
     "transform-style",
     "TransformStyle",
-    "computed::TransformStyle::" + ("Auto" if product == "servo" else "Flat"),
+    "computed::TransformStyle::Flat",
+    engines="gecko servo-2013 servo-2020",
     spec="https://drafts.csswg.org/css-transforms-2/#transform-style-property",
-    needs_context=False,
     extra_prefixes=transform_extra_prefixes,
-    flags="CREATES_STACKING_CONTEXT FIXPOS_CB",
     animation_value_type="discrete",
     servo_restyle_damage = "reflow_out_of_flow",
 )}
@@ -567,11 +437,11 @@ ${helpers.predefined_type(
     "transform-origin",
     "TransformOrigin",
     "computed::TransformOrigin::initial_value()",
+    engines="gecko servo-2013 servo-2020",
     animation_value_type="ComputedValue",
     extra_prefixes=transform_extra_prefixes,
     gecko_ffi_name="mTransformOrigin",
     boxed=True,
-    flags="GETCS_NEEDS_LAYOUT_FLUSH",
     spec="https://drafts.csswg.org/css-transforms/#transform-origin-property",
     servo_restyle_damage="reflow_out_of_flow",
 )}
@@ -580,42 +450,71 @@ ${helpers.predefined_type(
     "contain",
     "Contain",
     "specified::Contain::empty()",
+    engines="gecko",
     animation_value_type="none",
-    products="gecko",
-    flags="CREATES_STACKING_CONTEXT FIXPOS_CB",
-    gecko_pref="layout.css.contain.enabled",
     spec="https://drafts.csswg.org/css-contain/#contain-property",
-    enabled_in="chrome",
 )}
 
-// Non-standard
 ${helpers.predefined_type(
-    "-moz-appearance",
+    "content-visibility",
+    "ContentVisibility",
+    "computed::ContentVisibility::Visible",
+    engines="gecko",
+    spec="https://drafts.csswg.org/css-contain/#content-visibility",
+    gecko_pref="layout.css.content-visibility.enabled",
+    animation_value_type="none",
+)}
+
+${helpers.predefined_type(
+    "container-type",
+    "ContainerType",
+    "computed::ContainerType::Normal",
+    engines="gecko",
+    animation_value_type="none",
+    enabled_in="ua",
+    gecko_pref="layout.css.container-queries.enabled",
+    spec="https://drafts.csswg.org/css-contain-3/#container-type",
+)}
+
+${helpers.predefined_type(
+    "container-name",
+    "ContainerName",
+    "computed::ContainerName::none()",
+    engines="gecko",
+    animation_value_type="none",
+    enabled_in="ua",
+    gecko_pref="layout.css.container-queries.enabled",
+    spec="https://drafts.csswg.org/css-contain-3/#container-name",
+)}
+
+${helpers.predefined_type(
+    "appearance",
     "Appearance",
     "computed::Appearance::None",
-    products="gecko",
-    alias="-webkit-appearance:layout.css.webkit-appearance.enabled",
-    spec="Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/-moz-appearance)",
+    engines="gecko",
+    aliases="-moz-appearance -webkit-appearance",
+    spec="https://drafts.csswg.org/css-ui-4/#propdef-appearance",
     animation_value_type="discrete",
     gecko_ffi_name="mAppearance",
 )}
 
+// The inherent widget type of an element, selected by specifying
+// `appearance: auto`.
 ${helpers.predefined_type(
-    "-moz-binding",
-    "url::UrlOrNone",
-    "computed::url::UrlOrNone::none()",
-    products="gecko",
+    "-moz-default-appearance",
+    "Appearance",
+    "computed::Appearance::None",
+    engines="gecko",
     animation_value_type="none",
-    gecko_ffi_name="mBinding",
-    gecko_pref="layout.css.moz-binding.content.enabled",
+    spec="Internal (not web-exposed)",
     enabled_in="chrome",
-    spec="Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/-moz-binding)",
+    gecko_ffi_name="mDefaultAppearance",
 )}
 
 ${helpers.single_keyword(
     "-moz-orient",
     "inline block horizontal vertical",
-    products="gecko",
+    engines="gecko",
     gecko_ffi_name="mOrient",
     gecko_enum_prefix="StyleOrient",
     spec="Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/-moz-orient)",
@@ -626,16 +525,18 @@ ${helpers.predefined_type(
     "will-change",
     "WillChange",
     "computed::WillChange::auto()",
-    products="gecko",
+    engines="gecko",
     animation_value_type="none",
     spec="https://drafts.csswg.org/css-will-change/#will-change",
 )}
 
+// The spec issue for the parse_method: https://github.com/w3c/csswg-drafts/issues/4102.
 ${helpers.predefined_type(
-    "shape-image-threshold", "Opacity", "0.0",
-    products="gecko",
+    "shape-image-threshold",
+    "Opacity",
+    "0.0",
+    engines="gecko",
     animation_value_type="ComputedValue",
-    flags="APPLIES_TO_FIRST_LETTER",
     spec="https://drafts.csswg.org/css-shapes/#shape-image-threshold-property",
 )}
 
@@ -643,20 +544,17 @@ ${helpers.predefined_type(
     "shape-margin",
     "NonNegativeLengthPercentage",
     "computed::NonNegativeLengthPercentage::zero()",
-    products="gecko",
+    engines="gecko",
     animation_value_type="NonNegativeLengthPercentage",
-    flags="APPLIES_TO_FIRST_LETTER",
     spec="https://drafts.csswg.org/css-shapes/#shape-margin-property",
 )}
 
 ${helpers.predefined_type(
     "shape-outside",
-    "basic_shape::FloatAreaShape",
-    "generics::basic_shape::ShapeSource::None",
-    products="gecko",
-    boxed=True,
-    animation_value_type="basic_shape::FloatAreaShape",
-    flags="APPLIES_TO_FIRST_LETTER",
+    "basic_shape::ShapeOutside",
+    "generics::basic_shape::ShapeOutside::None",
+    engines="gecko",
+    animation_value_type="basic_shape::ShapeOutside",
     spec="https://drafts.csswg.org/css-shapes/#shape-outside-property",
 )}
 
@@ -664,21 +562,26 @@ ${helpers.predefined_type(
     "touch-action",
     "TouchAction",
     "computed::TouchAction::auto()",
-    products="gecko",
-    gecko_pref="layout.css.touch_action.enabled",
+    engines="gecko",
     animation_value_type="discrete",
     spec="https://compat.spec.whatwg.org/#touch-action",
 )}
 
-// Note that we only implement -webkit-line-clamp as a single, longhand
-// property for now, but the spec defines line-clamp as a shorthand for separate
-// max-lines, block-ellipsis, and continue properties.
 ${helpers.predefined_type(
     "-webkit-line-clamp",
-    "PositiveIntegerOrNone",
-    "Either::Second(None_)",
-    gecko_pref="layout.css.webkit-line-clamp.enabled",
-    animation_value_type="Integer",
-    products="gecko",
+    "LineClamp",
+    "computed::LineClamp::none()",
+    engines="gecko",
+    animation_value_type="ComputedValue",
     spec="https://drafts.csswg.org/css-overflow-3/#line-clamp",
 )}
+
+${helpers.predefined_type(
+    "scrollbar-gutter",
+    "ScrollbarGutter",
+    "computed::ScrollbarGutter::AUTO",
+    engines="gecko",
+    gecko_pref="layout.css.scrollbar-gutter.enabled",
+    animation_value_type="discrete",
+    spec="https://drafts.csswg.org/css-overflow-3/#scrollbar-gutter-property",
+)}
