@@ -10,6 +10,7 @@ ${helpers.predefined_type(
     "content",
     "Content",
     "computed::Content::normal()",
+    engines="gecko servo-2013 servo-2020",
     initial_specified_value="specified::Content::normal()",
     animation_value_type="discrete",
     spec="https://drafts.csswg.org/css-content/#propdef-content",
@@ -19,6 +20,7 @@ ${helpers.predefined_type(
 ${helpers.predefined_type(
     "counter-increment",
     "CounterIncrement",
+    engines="gecko servo-2013",
     initial_value="Default::default()",
     animation_value_type="discrete",
     spec="https://drafts.csswg.org/css-lists/#propdef-counter-increment",
@@ -27,7 +29,8 @@ ${helpers.predefined_type(
 
 ${helpers.predefined_type(
     "counter-reset",
-    "CounterSetOrReset",
+    "CounterReset",
+    engines="gecko servo-2013",
     initial_value="Default::default()",
     animation_value_type="discrete",
     spec="https://drafts.csswg.org/css-lists-3/#propdef-counter-reset",
@@ -36,10 +39,10 @@ ${helpers.predefined_type(
 
 ${helpers.predefined_type(
     "counter-set",
-    "CounterSetOrReset",
+    "CounterSet",
+    engines="gecko",
     initial_value="Default::default()",
     animation_value_type="discrete",
     spec="https://drafts.csswg.org/css-lists-3/#propdef-counter-set",
     servo_restyle_damage="rebuild_and_reflow",
-    products="gecko",
 )}
