@@ -4,9 +4,12 @@
 
 <%namespace name="helpers" file="/helpers.mako.rs" />
 
-<%helpers:shorthand name="marker" products="gecko"
+<%helpers:shorthand
+    name="marker"
+    engines="gecko"
     sub_properties="marker-start marker-end marker-mid"
-    spec="https://www.w3.org/TR/SVG2/painting.html#MarkerShorthand">
+    spec="https://www.w3.org/TR/SVG2/painting.html#MarkerShorthand"
+>
     use crate::values::specified::url::UrlOrNone;
 
     pub fn parse_value<'i, 't>(
