@@ -55,18 +55,18 @@
             % endfor
             loop {
                 if image.is_none() {
-                    if let Ok(value) = input.try(|input| mask_image::single_value
+                    if let Ok(value) = input.try_parse(|input| mask_image::single_value
                                                                    ::parse(context, input)) {
                         image = Some(value);
                         continue
                     }
                 }
                 if position.is_none() {
-                    if let Ok(value) = input.try(|input| Position::parse(context, input)) {
+                    if let Ok(value) = input.try_parse(|input| Position::parse(context, input)) {
                         position = Some(value);
 
                         // Parse mask size, if applicable.
-                        size = input.try(|input| {
+                        size = input.try_parse(|input| {
                             input.expect_delim('/')?;
                             mask_size::single_value::parse(context, input)
                         }).ok();
@@ -76,7 +76,7 @@
                 }
                 % for name in "repeat origin clip composite mode".split():
                     if ${name}.is_none() {
-                        if let Ok(value) = input.try(|input| mask_${name}::single_value
+                        if let Ok(value) = input.try_parse(|input| mask_${name}::single_value
                                                                                ::parse(context, input)) {
                             ${name} = Some(value);
                             continue
@@ -150,7 +150,7 @@
                 image.to_css(dest)?;
 
                 if *mode != mask_mode::single_value::get_initial_specified_value() {
-                    dest.write_str(" ")?;
+                    dest.write_char(' ')?;
                     mode.to_css(dest)?;
                 }
 
@@ -158,7 +158,7 @@
                     *position_y != PositionComponent::zero() ||
                     *size != mask_size::single_value::get_initial_specified_value()
                 {
-                    dest.write_str(" ")?;
+                    dest.write_char(' ')?;
                     Position {
                         horizontal: position_x.clone(),
                         vertical: position_y.clone()
@@ -171,21 +171,21 @@
                 }
 
                 if *repeat != mask_repeat::single_value::get_initial_specified_value() {
-                    dest.write_str(" ")?;
+                    dest.write_char(' ')?;
                     repeat.to_css(dest)?;
                 }
 
                 if *origin != Origin::BorderBox || *clip != Clip::BorderBox {
-                    dest.write_str(" ")?;
+                    dest.write_char(' ')?;
                     origin.to_css(dest)?;
                     if *clip != From::from(*origin) {
-                        dest.write_str(" ")?;
+                        dest.write_char(' ')?;
                         clip.to_css(dest)?;
                     }
                 }
 
                 if *composite != mask_composite::single_value::get_initial_specified_value() {
-                    dest.write_str(" ")?;
+                    dest.write_char(' ')?;
                     composite.to_css(dest)?;
                 }
             }
