@@ -8,18 +8,18 @@
 use crate::applicable_declarations::ApplicableDeclarationList;
 use crate::context::QuirksMode;
 use crate::dom::TElement;
-use crate::hash::map as hash_map;
-use crate::hash::{HashMap, HashSet};
-use crate::rule_tree::{CascadeLevel, ShadowCascadeOrder};
+use crate::rule_tree::CascadeLevel;
 use crate::selector_parser::SelectorImpl;
-use crate::stylist::Rule;
-use crate::{Atom, LocalName, Namespace, WeakAtom};
-use fallible::FallibleVec;
-use hashglobe::FailedAllocationError;
+use crate::stylist::{CascadeData, ContainerConditionId, Rule, Stylist};
+use crate::AllocErr;
+use crate::{Atom, LocalName, Namespace, ShrinkIfNeeded, WeakAtom};
+use dom::ElementState;
 use precomputed_hash::PrecomputedHash;
-use selectors::matching::{matches_selector, ElementSelectorFlags, MatchingContext};
+use selectors::matching::{matches_selector, MatchingContext};
 use selectors::parser::{Combinator, Component, SelectorIter};
 use smallvec::SmallVec;
+use std::collections::hash_map;
+use std::collections::{HashMap, HashSet};
 use std::hash::{BuildHasherDefault, Hash, Hasher};
 
 /// A hasher implementation that doesn't hash anything, because it expects its
@@ -34,6 +34,22 @@ impl Default for PrecomputedHasher {
     }
 }
 
+/// This is a set of pseudo-classes that are both relatively-rare (they don't
+/// affect most elements by default) and likely or known to have global rules
+/// (in e.g., the UA sheets).
+///
+/// We can avoid selector-matching those global rules for all elements without
+/// these pseudo-class states.
+const RARE_PSEUDO_CLASS_STATES: ElementState = ElementState::from_bits_retain(
+    ElementState::FULLSCREEN.bits() |
+        ElementState::VISITED_OR_UNVISITED.bits() |
+        ElementState::URLTARGET.bits() |
+        ElementState::INERT.bits() |
+        ElementState::FOCUS.bits() |
+        ElementState::FOCUSRING.bits() |
+        ElementState::TOPMOST_MODAL.bits(),
+);
+
 /// A simple alias for a hashmap using PrecomputedHasher.
 pub type PrecomputedHashMap<K, V> = HashMap<K, V, BuildHasherDefault<PrecomputedHasher>>;
 
@@ -94,7 +110,7 @@ pub trait SelectorMapEntry: Sized + Clone {
 /// * https://bugzilla.mozilla.org/show_bug.cgi?id=681755
 ///
 /// TODO: Tune the initial capacity of the HashMap
-#[derive(Debug, MallocSizeOf)]
+#[derive(Clone, Debug, MallocSizeOf)]
 pub struct SelectorMap<T: 'static> {
     /// Rules that have `:root` selectors.
     pub root: SmallVec<[T; 1]>,
@@ -104,10 +120,16 @@ pub struct SelectorMap<T: 'static> {
     pub class_hash: MaybeCaseInsensitiveHashMap<Atom, SmallVec<[T; 1]>>,
     /// A hash from local name to rules which contain that local name selector.
     pub local_name_hash: PrecomputedHashMap<LocalName, SmallVec<[T; 1]>>,
+    /// A hash from attributes to rules which contain that attribute selector.
+    pub attribute_hash: PrecomputedHashMap<LocalName, SmallVec<[T; 1]>>,
     /// A hash from namespace to rules which contain that namespace selector.
     pub namespace_hash: PrecomputedHashMap<Namespace, SmallVec<[T; 1]>>,
+    /// Rules for pseudo-states that are rare but have global selectors.
+    pub rare_pseudo_classes: SmallVec<[T; 1]>,
     /// All other rules.
     pub other: SmallVec<[T; 1]>,
+    /// Whether we should bucket by attribute names.
+    bucket_attributes: bool,
     /// The number of entries in this map.
     pub count: usize,
 }
@@ -119,30 +141,49 @@ impl<T: 'static> Default for SelectorMap<T> {
     }
 }
 
-// FIXME(Manishearth) the 'static bound can be removed when
-// our HashMap fork (hashglobe) is able to use NonZero,
-// or when stdlib gets fallible collections
-impl<T: 'static> SelectorMap<T> {
+impl<T> SelectorMap<T> {
     /// Trivially constructs an empty `SelectorMap`.
     pub fn new() -> Self {
         SelectorMap {
             root: SmallVec::new(),
             id_hash: MaybeCaseInsensitiveHashMap::new(),
             class_hash: MaybeCaseInsensitiveHashMap::new(),
+            attribute_hash: HashMap::default(),
             local_name_hash: HashMap::default(),
             namespace_hash: HashMap::default(),
+            rare_pseudo_classes: SmallVec::new(),
             other: SmallVec::new(),
+            bucket_attributes: static_prefs::pref!("layout.css.bucket-attribute-names.enabled"),
             count: 0,
         }
     }
 
+    /// Trivially constructs an empty `SelectorMap`, with attribute bucketing
+    /// explicitly disabled.
+    pub fn new_without_attribute_bucketing() -> Self {
+        let mut ret = Self::new();
+        ret.bucket_attributes = false;
+        ret
+    }
+
+    /// Shrink the capacity of the map if needed.
+    pub fn shrink_if_needed(&mut self) {
+        self.id_hash.shrink_if_needed();
+        self.class_hash.shrink_if_needed();
+        self.attribute_hash.shrink_if_needed();
+        self.local_name_hash.shrink_if_needed();
+        self.namespace_hash.shrink_if_needed();
+    }
+
     /// Clears the hashmap retaining storage.
     pub fn clear(&mut self) {
         self.root.clear();
         self.id_hash.clear();
         self.class_hash.clear();
+        self.attribute_hash.clear();
         self.local_name_hash.clear();
         self.namespace_hash.clear();
+        self.rare_pseudo_classes.clear();
         self.other.clear();
         self.count = 0;
     }
@@ -163,38 +204,33 @@ impl SelectorMap<Rule> {
     ///
     /// Extract matching rules as per element's ID, classes, tag name, etc..
     /// Sort the Rules at the end to maintain cascading order.
-    pub fn get_all_matching_rules<E, F>(
+    pub fn get_all_matching_rules<E>(
         &self,
         element: E,
         rule_hash_target: E,
         matching_rules_list: &mut ApplicableDeclarationList,
-        context: &mut MatchingContext<E::Impl>,
-        flags_setter: &mut F,
+        matching_context: &mut MatchingContext<E::Impl>,
         cascade_level: CascadeLevel,
-        shadow_cascade_order: ShadowCascadeOrder,
+        cascade_data: &CascadeData,
+        stylist: &Stylist,
     ) where
         E: TElement,
-        F: FnMut(&E, ElementSelectorFlags),
     {
         if self.is_empty() {
             return;
         }
 
-        let quirks_mode = context.quirks_mode();
-
-        // At the end, we're going to sort the rules that we added, so remember
-        // where we began.
-        let init_len = matching_rules_list.len();
+        let quirks_mode = matching_context.quirks_mode();
 
         if rule_hash_target.is_root() {
             SelectorMap::get_matching_rules(
                 element,
                 &self.root,
                 matching_rules_list,
-                context,
-                flags_setter,
+                matching_context,
                 cascade_level,
-                shadow_cascade_order,
+                cascade_data,
+                stylist,
             );
         }
 
@@ -204,10 +240,10 @@ impl SelectorMap<Rule> {
                     element,
                     rules,
                     matching_rules_list,
-                    context,
-                    flags_setter,
+                    matching_context,
                     cascade_level,
-                    shadow_cascade_order,
+                    cascade_data,
+                    stylist,
                 )
             }
         }
@@ -218,35 +254,66 @@ impl SelectorMap<Rule> {
                     element,
                     rules,
                     matching_rules_list,
-                    context,
-                    flags_setter,
+                    matching_context,
                     cascade_level,
-                    shadow_cascade_order,
+                    cascade_data,
+                    stylist,
                 )
             }
         });
 
+        if self.bucket_attributes {
+            rule_hash_target.each_attr_name(|name| {
+                if let Some(rules) = self.attribute_hash.get(name) {
+                    SelectorMap::get_matching_rules(
+                        element,
+                        rules,
+                        matching_rules_list,
+                        matching_context,
+                        cascade_level,
+                        cascade_data,
+                        stylist,
+                    )
+                }
+            });
+        }
+
         if let Some(rules) = self.local_name_hash.get(rule_hash_target.local_name()) {
             SelectorMap::get_matching_rules(
                 element,
                 rules,
                 matching_rules_list,
-                context,
-                flags_setter,
+                matching_context,
                 cascade_level,
-                shadow_cascade_order,
+                cascade_data,
+                stylist,
             )
         }
 
+        if rule_hash_target
+            .state()
+            .intersects(RARE_PSEUDO_CLASS_STATES)
+        {
+            SelectorMap::get_matching_rules(
+                element,
+                &self.rare_pseudo_classes,
+                matching_rules_list,
+                matching_context,
+                cascade_level,
+                cascade_data,
+                stylist,
+            );
+        }
+
         if let Some(rules) = self.namespace_hash.get(rule_hash_target.namespace()) {
             SelectorMap::get_matching_rules(
                 element,
                 rules,
                 matching_rules_list,
-                context,
-                flags_setter,
+                matching_context,
                 cascade_level,
-                shadow_cascade_order,
+                cascade_data,
+                stylist,
             )
         }
 
@@ -254,97 +321,154 @@ impl SelectorMap<Rule> {
             element,
             &self.other,
             matching_rules_list,
-            context,
-            flags_setter,
+            matching_context,
             cascade_level,
-            shadow_cascade_order,
+            cascade_data,
+            stylist,
         );
-
-        // Sort only the rules we just added.
-        matching_rules_list[init_len..]
-            .sort_unstable_by_key(|block| (block.specificity, block.source_order()));
     }
 
     /// Adds rules in `rules` that match `element` to the `matching_rules` list.
-    fn get_matching_rules<E, F>(
+    pub(crate) fn get_matching_rules<E>(
         element: E,
         rules: &[Rule],
         matching_rules: &mut ApplicableDeclarationList,
-        context: &mut MatchingContext<E::Impl>,
-        flags_setter: &mut F,
+        matching_context: &mut MatchingContext<E::Impl>,
         cascade_level: CascadeLevel,
-        shadow_cascade_order: ShadowCascadeOrder,
+        cascade_data: &CascadeData,
+        stylist: &Stylist,
     ) where
         E: TElement,
-        F: FnMut(&E, ElementSelectorFlags),
     {
         for rule in rules {
-            if matches_selector(
+            if !matches_selector(
                 &rule.selector,
                 0,
                 Some(&rule.hashes),
                 &element,
-                context,
-                flags_setter,
+                matching_context,
             ) {
-                matching_rules.push(
-                    rule.to_applicable_declaration_block(cascade_level, shadow_cascade_order),
-                );
+                continue;
             }
+
+            if rule.container_condition_id != ContainerConditionId::none() {
+                if !cascade_data.container_condition_matches(
+                    rule.container_condition_id,
+                    stylist,
+                    element,
+                    matching_context,
+                ) {
+                    continue;
+                }
+            }
+
+            matching_rules.push(rule.to_applicable_declaration_block(cascade_level, cascade_data));
         }
     }
 }
 
 impl<T: SelectorMapEntry> SelectorMap<T> {
-    /// Inserts into the correct hash, trying id, class, localname and
-    /// namespace.
-    pub fn insert(
-        &mut self,
-        entry: T,
-        quirks_mode: QuirksMode,
-    ) -> Result<(), FailedAllocationError> {
+    /// Inserts an entry into the correct bucket(s).
+    pub fn insert(&mut self, entry: T, quirks_mode: QuirksMode) -> Result<(), AllocErr> {
         self.count += 1;
 
-        let vector = match find_bucket(entry.selector()) {
-            Bucket::Root => &mut self.root,
-            Bucket::ID(id) => self
-                .id_hash
-                .try_entry(id.clone(), quirks_mode)?
-                .or_insert_with(SmallVec::new),
-            Bucket::Class(class) => self
-                .class_hash
-                .try_entry(class.clone(), quirks_mode)?
-                .or_insert_with(SmallVec::new),
-            Bucket::LocalName { name, lower_name } => {
-                // If the local name in the selector isn't lowercase, insert it
-                // into the rule hash twice. This means that, during lookup, we
-                // can always find the rules based on the local name of the
-                // element, regardless of whether it's an html element in an
-                // html document (in which case we match against lower_name) or
-                // not (in which case we match against name).
-                //
-                // In the case of a non-html-element-in-html-document with a
-                // lowercase localname and a non-lowercase selector, the
-                // rulehash lookup may produce superfluous selectors, but the
-                // subsequent selector matching work will filter them out.
-                if name != lower_name {
-                    self.local_name_hash
-                        .try_entry(lower_name.clone())?
-                        .or_insert_with(SmallVec::new)
-                        .try_push(entry.clone())?;
+        // NOTE(emilio): It'd be nice for this to be a separate function, but
+        // then the compiler can't reason about the lifetime dependency between
+        // `entry` and `bucket`, and would force us to clone the rule in the
+        // common path.
+        macro_rules! insert_into_bucket {
+            ($entry:ident, $bucket:expr) => {{
+                let vec = match $bucket {
+                    Bucket::Root => &mut self.root,
+                    Bucket::ID(id) => self
+                        .id_hash
+                        .try_entry(id.clone(), quirks_mode)?
+                        .or_default(),
+                    Bucket::Class(class) => self
+                        .class_hash
+                        .try_entry(class.clone(), quirks_mode)?
+                        .or_default(),
+                    Bucket::Attribute { name, lower_name } |
+                    Bucket::LocalName { name, lower_name } => {
+                        // If the local name in the selector isn't lowercase,
+                        // insert it into the rule hash twice. This means that,
+                        // during lookup, we can always find the rules based on
+                        // the local name of the element, regardless of whether
+                        // it's an html element in an html document (in which
+                        // case we match against lower_name) or not (in which
+                        // case we match against name).
+                        //
+                        // In the case of a non-html-element-in-html-document
+                        // with a lowercase localname and a non-lowercase
+                        // selector, the rulehash lookup may produce superfluous
+                        // selectors, but the subsequent selector matching work
+                        // will filter them out.
+                        let is_attribute = matches!($bucket, Bucket::Attribute { .. });
+                        let hash = if is_attribute {
+                            &mut self.attribute_hash
+                        } else {
+                            &mut self.local_name_hash
+                        };
+                        if name != lower_name {
+                            hash.try_reserve(1)?;
+                            let vec = hash.entry(lower_name.clone()).or_default();
+                            vec.try_reserve(1)?;
+                            vec.push($entry.clone());
+                        }
+                        hash.try_reserve(1)?;
+                        hash.entry(name.clone()).or_default()
+                    },
+                    Bucket::Namespace(url) => {
+                        self.namespace_hash.try_reserve(1)?;
+                        self.namespace_hash.entry(url.clone()).or_default()
+                    },
+                    Bucket::RarePseudoClasses => &mut self.rare_pseudo_classes,
+                    Bucket::Universal => &mut self.other,
+                };
+                vec.try_reserve(1)?;
+                vec.push($entry);
+            }};
+        }
+
+        let bucket = {
+            let mut disjoint_buckets = SmallVec::new();
+            let bucket = find_bucket(
+                entry.selector(),
+                &mut disjoint_buckets,
+                self.bucket_attributes,
+            );
+
+            // See if inserting this selector in multiple entries in the
+            // selector map would be worth it. Consider a case like:
+            //
+            //   .foo:where(div, #bar)
+            //
+            // There, `bucket` would be `Class(foo)`, and disjoint_buckets would
+            // be `[LocalName { div }, ID(bar)]`.
+            //
+            // Here we choose to insert the selector in the `.foo` bucket in
+            // such a case, as it's likely more worth it than inserting it in
+            // both `div` and `#bar`.
+            //
+            // This is specially true if there's any universal selector in the
+            // `disjoint_selectors` set, at which point we'd just be doing
+            // wasted work.
+            if !disjoint_buckets.is_empty() &&
+                disjoint_buckets
+                    .iter()
+                    .all(|b| b.more_specific_than(&bucket))
+            {
+                for bucket in &disjoint_buckets {
+                    let entry = entry.clone();
+                    insert_into_bucket!(entry, *bucket);
                 }
-                self.local_name_hash
-                    .try_entry(name.clone())?
-                    .or_insert_with(SmallVec::new)
-            },
-            Bucket::Namespace(url) => self
-                .namespace_hash
-                .try_entry(url.clone())?
-                .or_insert_with(SmallVec::new),
-            Bucket::Universal => &mut self.other,
+                return Ok(());
+            }
+            bucket
         };
 
-        vector.try_push(entry)
+        insert_into_bucket!(entry, bucket);
+        Ok(())
     }
 
     /// Looks up entries by id, class, local name, namespace, and other (in
@@ -357,8 +481,22 @@ impl<T: SelectorMapEntry> SelectorMap<T> {
     ///
     /// FIXME(bholley) This overlaps with SelectorMap<Rule>::get_all_matching_rules,
     /// but that function is extremely hot and I'd rather not rearrange it.
+    pub fn lookup<'a, E, F>(&'a self, element: E, quirks_mode: QuirksMode, f: F) -> bool
+    where
+        E: TElement,
+        F: FnMut(&'a T) -> bool,
+    {
+        self.lookup_with_state(element, element.state(), quirks_mode, f)
+    }
+
     #[inline]
-    pub fn lookup<'a, E, F>(&'a self, element: E, quirks_mode: QuirksMode, mut f: F) -> bool
+    fn lookup_with_state<'a, E, F>(
+        &'a self,
+        element: E,
+        element_state: ElementState,
+        quirks_mode: QuirksMode,
+        mut f: F,
+    ) -> bool
     where
         E: TElement,
         F: FnMut(&'a T) -> bool,
@@ -383,8 +521,29 @@ impl<T: SelectorMapEntry> SelectorMap<T> {
 
         let mut done = false;
         element.each_class(|class| {
-            if !done {
-                if let Some(v) = self.class_hash.get(class, quirks_mode) {
+            if done {
+                return;
+            }
+            if let Some(v) = self.class_hash.get(class, quirks_mode) {
+                for entry in v.iter() {
+                    if !f(&entry) {
+                        done = true;
+                        return;
+                    }
+                }
+            }
+        });
+
+        if done {
+            return false;
+        }
+
+        if self.bucket_attributes {
+            element.each_attr_name(|name| {
+                if done {
+                    return;
+                }
+                if let Some(v) = self.attribute_hash.get(name) {
                     for entry in v.iter() {
                         if !f(&entry) {
                             done = true;
@@ -392,10 +551,11 @@ impl<T: SelectorMapEntry> SelectorMap<T> {
                         }
                     }
                 }
+            });
+
+            if done {
+                return false;
             }
-        });
-        if done {
-            return false;
         }
 
         if let Some(v) = self.local_name_hash.get(element.local_name()) {
@@ -414,6 +574,14 @@ impl<T: SelectorMapEntry> SelectorMap<T> {
             }
         }
 
+        if element_state.intersects(RARE_PSEUDO_CLASS_STATES) {
+            for entry in self.rare_pseudo_classes.iter() {
+                if !f(&entry) {
+                    return false;
+                }
+            }
+        }
+
         for entry in self.other.iter() {
             if !f(&entry) {
                 return false;
@@ -437,6 +605,7 @@ impl<T: SelectorMapEntry> SelectorMap<T> {
         quirks_mode: QuirksMode,
         additional_id: Option<&WeakAtom>,
         additional_classes: &[Atom],
+        additional_states: ElementState,
         mut f: F,
     ) -> bool
     where
@@ -444,7 +613,12 @@ impl<T: SelectorMapEntry> SelectorMap<T> {
         F: FnMut(&'a T) -> bool,
     {
         // Do the normal lookup.
-        if !self.lookup(element, quirks_mode, |entry| f(entry)) {
+        if !self.lookup_with_state(
+            element,
+            element.state() | additional_states,
+            quirks_mode,
+            |entry| f(entry),
+        ) {
             return false;
         }
 
@@ -475,22 +649,77 @@ impl<T: SelectorMapEntry> SelectorMap<T> {
 }
 
 enum Bucket<'a> {
-    Root,
-    ID(&'a Atom),
-    Class(&'a Atom),
+    Universal,
+    Namespace(&'a Namespace),
+    RarePseudoClasses,
     LocalName {
         name: &'a LocalName,
         lower_name: &'a LocalName,
     },
-    Namespace(&'a Namespace),
-    Universal,
+    Attribute {
+        name: &'a LocalName,
+        lower_name: &'a LocalName,
+    },
+    Class(&'a Atom),
+    ID(&'a Atom),
+    Root,
 }
 
-fn specific_bucket_for<'a>(component: &'a Component<SelectorImpl>) -> Bucket<'a> {
+impl<'a> Bucket<'a> {
+    /// root > id > class > local name > namespace > pseudo-classes > universal.
+    #[inline]
+    fn specificity(&self) -> usize {
+        match *self {
+            Bucket::Universal => 0,
+            Bucket::Namespace(..) => 1,
+            Bucket::RarePseudoClasses => 2,
+            Bucket::LocalName { .. } => 3,
+            Bucket::Attribute { .. } => 4,
+            Bucket::Class(..) => 5,
+            Bucket::ID(..) => 6,
+            Bucket::Root => 7,
+        }
+    }
+
+    #[inline]
+    fn more_or_equally_specific_than(&self, other: &Self) -> bool {
+        self.specificity() >= other.specificity()
+    }
+
+    #[inline]
+    fn more_specific_than(&self, other: &Self) -> bool {
+        self.specificity() > other.specificity()
+    }
+}
+
+type DisjointBuckets<'a> = SmallVec<[Bucket<'a>; 5]>;
+
+fn specific_bucket_for<'a>(
+    component: &'a Component<SelectorImpl>,
+    disjoint_buckets: &mut DisjointBuckets<'a>,
+    bucket_attributes: bool,
+) -> Bucket<'a> {
     match *component {
         Component::Root => Bucket::Root,
         Component::ID(ref id) => Bucket::ID(id),
         Component::Class(ref class) => Bucket::Class(class),
+        Component::AttributeInNoNamespace { ref local_name, .. } if bucket_attributes => {
+            Bucket::Attribute {
+                name: local_name,
+                lower_name: local_name,
+            }
+        },
+        Component::AttributeInNoNamespaceExists {
+            ref local_name,
+            ref local_name_lower,
+        } if bucket_attributes => Bucket::Attribute {
+            name: local_name,
+            lower_name: local_name_lower,
+        },
+        Component::AttributeOther(ref selector) if bucket_attributes => Bucket::Attribute {
+            name: &selector.local_name,
+            lower_name: &selector.local_name_lower,
+        },
         Component::LocalName(ref selector) => Bucket::LocalName {
             name: &selector.name,
             lower_name: &selector.lower_name,
@@ -516,47 +745,54 @@ fn specific_bucket_for<'a>(component: &'a Component<SelectorImpl>) -> Bucket<'a>
         //
         // So inserting `span` in the rule hash makes sense since we want to
         // match the slotted <span>.
-        Component::Slotted(ref selector) => find_bucket(selector.iter()),
-        Component::Host(Some(ref selector)) => find_bucket(selector.iter()),
+        Component::Slotted(ref selector) => {
+            find_bucket(selector.iter(), disjoint_buckets, bucket_attributes)
+        },
+        Component::Host(Some(ref selector)) => {
+            find_bucket(selector.iter(), disjoint_buckets, bucket_attributes)
+        },
+        Component::Is(ref list) | Component::Where(ref list) => {
+            if list.len() == 1 {
+                find_bucket(list[0].iter(), disjoint_buckets, bucket_attributes)
+            } else {
+                for selector in &**list {
+                    let bucket = find_bucket(selector.iter(), disjoint_buckets, bucket_attributes);
+                    disjoint_buckets.push(bucket);
+                }
+                Bucket::Universal
+            }
+        },
+        Component::NonTSPseudoClass(ref pseudo_class)
+            if pseudo_class
+                .state_flag()
+                .intersects(RARE_PSEUDO_CLASS_STATES) =>
+        {
+            Bucket::RarePseudoClasses
+        },
         _ => Bucket::Universal,
     }
 }
 
 /// Searches a compound selector from left to right, and returns the appropriate
 /// bucket for it.
+///
+/// It also populates disjoint_buckets with dependencies from nested selectors
+/// with any semantics like :is() and :where().
 #[inline(always)]
-fn find_bucket<'a>(mut iter: SelectorIter<'a, SelectorImpl>) -> Bucket<'a> {
+fn find_bucket<'a>(
+    mut iter: SelectorIter<'a, SelectorImpl>,
+    disjoint_buckets: &mut DisjointBuckets<'a>,
+    bucket_attributes: bool,
+) -> Bucket<'a> {
     let mut current_bucket = Bucket::Universal;
 
     loop {
-        // We basically want to find the most specific bucket,
-        // where:
-        //
-        //   root > id > class > local name > namespace > universal.
-        //
         for ss in &mut iter {
-            let new_bucket = specific_bucket_for(ss);
-            match new_bucket {
-                Bucket::Root => return new_bucket,
-                Bucket::ID(..) => {
-                    current_bucket = new_bucket;
-                },
-                Bucket::Class(..) => {
-                    if !matches!(current_bucket, Bucket::ID(..)) {
-                        current_bucket = new_bucket;
-                    }
-                },
-                Bucket::LocalName { .. } => {
-                    if matches!(current_bucket, Bucket::Universal | Bucket::Namespace(..)) {
-                        current_bucket = new_bucket;
-                    }
-                },
-                Bucket::Namespace(..) => {
-                    if matches!(current_bucket, Bucket::Universal) {
-                        current_bucket = new_bucket;
-                    }
-                },
-                Bucket::Universal => {},
+            let new_bucket = specific_bucket_for(ss, disjoint_buckets, bucket_attributes);
+            // NOTE: When presented with the choice of multiple specific selectors, use the
+            // rightmost, on the assumption that that's less common, see bug 1829540.
+            if new_bucket.more_or_equally_specific_than(&current_bucket) {
+                current_bucket = new_bucket;
             }
         }
 
@@ -567,22 +803,29 @@ fn find_bucket<'a>(mut iter: SelectorIter<'a, SelectorImpl>) -> Bucket<'a> {
         }
     }
 
-    return current_bucket;
+    current_bucket
 }
 
 /// Wrapper for PrecomputedHashMap that does ASCII-case-insensitive lookup in quirks mode.
-#[derive(Debug, MallocSizeOf)]
-pub struct MaybeCaseInsensitiveHashMap<K: PrecomputedHash + Hash + Eq, V: 'static>(
-    PrecomputedHashMap<K, V>,
-);
+#[derive(Clone, Debug, MallocSizeOf)]
+pub struct MaybeCaseInsensitiveHashMap<K: PrecomputedHash + Hash + Eq, V>(PrecomputedHashMap<K, V>);
 
-// FIXME(Manishearth) the 'static bound can be removed when
-// our HashMap fork (hashglobe) is able to use NonZero,
-// or when stdlib gets fallible collections
-impl<V: 'static> MaybeCaseInsensitiveHashMap<Atom, V> {
+impl<V> Default for MaybeCaseInsensitiveHashMap<Atom, V> {
+    #[inline]
+    fn default() -> Self {
+        MaybeCaseInsensitiveHashMap(PrecomputedHashMap::default())
+    }
+}
+
+impl<V> MaybeCaseInsensitiveHashMap<Atom, V> {
     /// Empty map
     pub fn new() -> Self {
-        MaybeCaseInsensitiveHashMap(PrecomputedHashMap::default())
+        Self::default()
+    }
+
+    /// Shrink the capacity of the map if needed.
+    pub fn shrink_if_needed(&mut self) {
+        self.0.shrink_if_needed()
     }
 
     /// HashMap::try_entry
@@ -590,11 +833,18 @@ impl<V: 'static> MaybeCaseInsensitiveHashMap<Atom, V> {
         &mut self,
         mut key: Atom,
         quirks_mode: QuirksMode,
-    ) -> Result<hash_map::Entry<Atom, V>, FailedAllocationError> {
+    ) -> Result<hash_map::Entry<Atom, V>, AllocErr> {
         if quirks_mode == QuirksMode::Quirks {
             key = key.to_ascii_lowercase()
         }
-        self.0.try_entry(key)
+        self.0.try_reserve(1)?;
+        Ok(self.0.entry(key))
+    }
+
+    /// HashMap::is_empty
+    #[inline]
+    pub fn is_empty(&self) -> bool {
+        self.0.is_empty()
     }
 
     /// HashMap::iter
