@@ -6,6 +6,7 @@
 
 use crate::error_reporting::ContextualParseError;
 use crate::parser::ParserContext;
+use crate::properties::longhands::animation_composition::single_value::SpecifiedValue as SpecifiedComposition;
 use crate::properties::longhands::transition_timing_function::single_value::SpecifiedValue as SpecifiedTimingFunction;
 use crate::properties::LonghandIdSet;
 use crate::properties::{Importance, PropertyDeclaration};
@@ -17,9 +18,12 @@ use crate::str::CssStringWriter;
 use crate::stylesheets::rule_parser::VendorPrefix;
 use crate::stylesheets::{CssRuleType, StylesheetContents};
 use crate::values::{serialize_percentage, KeyframesName};
-use cssparser::{parse_one_rule, DeclarationListParser, DeclarationParser, SourceLocation, Token};
-use cssparser::{AtRuleParser, CowRcStr, Parser, ParserInput, QualifiedRuleParser, RuleListParser};
+use cssparser::{
+    parse_one_rule, AtRuleParser, CowRcStr, DeclarationParser, Parser, ParserInput, ParserState,
+    QualifiedRuleParser, RuleBodyItemParser, RuleBodyParser, SourceLocation, Token,
+};
 use servo_arc::Arc;
+use std::borrow::Cow;
 use std::fmt::{self, Write};
 use style_traits::{CssWriter, ParseError, ParsingMode, StyleParseErrorKind, ToCss};
 
@@ -149,8 +153,8 @@ impl KeyframePercentage {
 
 /// A keyframes selector is a list of percentages or from/to symbols, which are
 /// converted at parse time to percentages.
-#[css(comma)]
 #[derive(Clone, Debug, Eq, PartialEq, ToCss, ToShmem)]
+#[css(comma)]
 pub struct KeyframeSelector(#[css(iterable)] Vec<KeyframePercentage>);
 
 impl KeyframeSelector {
@@ -214,16 +218,16 @@ impl Keyframe {
             Some(CssRuleType::Keyframe),
             ParsingMode::DEFAULT,
             parent_stylesheet_contents.quirks_mode,
+            Cow::Borrowed(&*namespaces),
             None,
             None,
         );
-        context.namespaces = Some(&*namespaces);
         let mut input = ParserInput::new(css);
         let mut input = Parser::new(&mut input);
 
-        let mut declarations = SourcePropertyDeclaration::new();
+        let mut declarations = SourcePropertyDeclaration::default();
         let mut rule_parser = KeyframeListParser {
-            context: &context,
+            context: &mut context,
             shared_lock: &lock,
             declarations: &mut declarations,
         };
@@ -277,40 +281,81 @@ pub struct KeyframesStep {
     /// Declarations that will determine the final style during the step, or
     /// `ComputedValues` if this is an autogenerated step.
     pub value: KeyframesStepValue,
-    /// Wether a animation-timing-function declaration exists in the list of
+    /// Whether an animation-timing-function declaration exists in the list of
     /// declarations.
     ///
     /// This is used to know when to override the keyframe animation style.
     pub declared_timing_function: bool,
+    /// Whether an animation-composition declaration exists in the list of
+    /// declarations.
+    ///
+    /// This is used to know when to override the keyframe animation style.
+    pub declared_composition: bool,
 }
 
 impl KeyframesStep {
     #[inline]
     fn new(
-        percentage: KeyframePercentage,
+        start_percentage: KeyframePercentage,
         value: KeyframesStepValue,
         guard: &SharedRwLockReadGuard,
     ) -> Self {
-        let declared_timing_function = match value {
-            KeyframesStepValue::Declarations { ref block } => block
-                .read_with(guard)
-                .declarations()
-                .iter()
-                .any(|prop_decl| match *prop_decl {
-                    PropertyDeclaration::AnimationTimingFunction(..) => true,
-                    _ => false,
-                }),
-            _ => false,
-        };
+        let mut declared_timing_function = false;
+        let mut declared_composition = false;
+        if let KeyframesStepValue::Declarations { ref block } = value {
+            for prop_decl in block.read_with(guard).declarations().iter() {
+                match *prop_decl {
+                    PropertyDeclaration::AnimationTimingFunction(..) => {
+                        declared_timing_function = true;
+                    },
+                    PropertyDeclaration::AnimationComposition(..) => {
+                        declared_composition = true;
+                    },
+                    _ => continue,
+                }
+                // Don't need to continue the loop if both are found.
+                if declared_timing_function && declared_composition {
+                    break;
+                }
+            }
+        }
 
         KeyframesStep {
-            start_percentage: percentage,
-            value: value,
-            declared_timing_function: declared_timing_function,
+            start_percentage,
+            value,
+            declared_timing_function,
+            declared_composition,
         }
     }
 
-    /// Return specified TransitionTimingFunction if this KeyframesSteps has 'animation-timing-function'.
+    /// Return specified PropertyDeclaration.
+    #[inline]
+    fn get_declared_property<'a>(
+        &'a self,
+        guard: &'a SharedRwLockReadGuard,
+        property: LonghandId,
+    ) -> Option<&'a PropertyDeclaration> {
+        match self.value {
+            KeyframesStepValue::Declarations { ref block } => {
+                let guard = block.read_with(guard);
+                let (declaration, _) = guard
+                    .get(PropertyDeclarationId::Longhand(property))
+                    .unwrap();
+                match *declaration {
+                    PropertyDeclaration::CSSWideKeyword(..) => None,
+                    // FIXME: Bug 1710735: Support css variable in @keyframes rule.
+                    PropertyDeclaration::WithVariables(..) => None,
+                    _ => Some(declaration),
+                }
+            },
+            KeyframesStepValue::ComputedValues => {
+                panic!("Shouldn't happen to set this property in missing keyframes")
+            },
+        }
+    }
+
+    /// Return specified TransitionTimingFunction if this KeyframesSteps has
+    /// 'animation-timing-function'.
     pub fn get_animation_timing_function(
         &self,
         guard: &SharedRwLockReadGuard,
@@ -318,28 +363,38 @@ impl KeyframesStep {
         if !self.declared_timing_function {
             return None;
         }
-        match self.value {
-            KeyframesStepValue::Declarations { ref block } => {
-                let guard = block.read_with(guard);
-                let (declaration, _) = guard
-                    .get(PropertyDeclarationId::Longhand(
-                        LonghandId::AnimationTimingFunction,
-                    ))
-                    .unwrap();
-                match *declaration {
+
+        self.get_declared_property(guard, LonghandId::AnimationTimingFunction)
+            .map(|decl| {
+                match *decl {
                     PropertyDeclaration::AnimationTimingFunction(ref value) => {
-                        // Use the first value.
-                        Some(value.0[0])
+                        // Use the first value
+                        value.0[0].clone()
                     },
-                    PropertyDeclaration::CSSWideKeyword(..) => None,
-                    PropertyDeclaration::WithVariables(..) => None,
-                    _ => panic!(),
+                    _ => unreachable!("Unexpected PropertyDeclaration"),
                 }
-            },
-            KeyframesStepValue::ComputedValues => {
-                panic!("Shouldn't happen to set animation-timing-function in missing keyframes")
-            },
+            })
+    }
+
+    /// Return CompositeOperation if this KeyframesSteps has 'animation-composition'.
+    pub fn get_animation_composition(
+        &self,
+        guard: &SharedRwLockReadGuard,
+    ) -> Option<SpecifiedComposition> {
+        if !self.declared_composition {
+            return None;
         }
+
+        self.get_declared_property(guard, LonghandId::AnimationComposition)
+            .map(|decl| {
+                match *decl {
+                    PropertyDeclaration::AnimationComposition(ref value) => {
+                        // Use the first value
+                        value.0[0].clone()
+                    },
+                    _ => unreachable!("Unexpected PropertyDeclaration"),
+                }
+            })
     }
 }
 
@@ -472,44 +527,41 @@ impl KeyframesAnimation {
 /// 40%, 60%, 100% {
 ///     width: 100%;
 /// }
-struct KeyframeListParser<'a> {
-    context: &'a ParserContext<'a>,
+struct KeyframeListParser<'a, 'b> {
+    context: &'a mut ParserContext<'b>,
     shared_lock: &'a SharedRwLock,
     declarations: &'a mut SourcePropertyDeclaration,
 }
 
 /// Parses a keyframe list from CSS input.
-pub fn parse_keyframe_list(
-    context: &ParserContext,
+pub fn parse_keyframe_list<'a>(
+    context: &mut ParserContext<'a>,
     input: &mut Parser,
     shared_lock: &SharedRwLock,
 ) -> Vec<Arc<Locked<Keyframe>>> {
-    debug_assert!(
-        context.namespaces.is_some(),
-        "Parsing a keyframe list from a context without namespaces?"
-    );
-
-    let mut declarations = SourcePropertyDeclaration::new();
-    RuleListParser::new_for_nested_rule(
-        input,
-        KeyframeListParser {
-            context: context,
-            shared_lock: shared_lock,
-            declarations: &mut declarations,
-        },
-    )
-    .filter_map(Result::ok)
-    .collect()
+    let mut declarations = SourcePropertyDeclaration::default();
+    let mut parser = KeyframeListParser {
+        context,
+        shared_lock,
+        declarations: &mut declarations,
+    };
+    RuleBodyParser::new(input, &mut parser)
+        .filter_map(Result::ok)
+        .collect()
 }
 
-impl<'a, 'i> AtRuleParser<'i> for KeyframeListParser<'a> {
-    type PreludeNoBlock = ();
-    type PreludeBlock = ();
+impl<'a, 'b, 'i> AtRuleParser<'i> for KeyframeListParser<'a, 'b> {
+    type Prelude = ();
     type AtRule = Arc<Locked<Keyframe>>;
     type Error = StyleParseErrorKind<'i>;
 }
 
-impl<'a, 'i> QualifiedRuleParser<'i> for KeyframeListParser<'a> {
+impl<'a, 'b, 'i> DeclarationParser<'i> for KeyframeListParser<'a, 'b> {
+    type Declaration = Arc<Locked<Keyframe>>;
+    type Error = StyleParseErrorKind<'i>;
+}
+
+impl<'a, 'b, 'i> QualifiedRuleParser<'i> for KeyframeListParser<'a, 'b> {
     type Prelude = KeyframeSelector;
     type QualifiedRule = Arc<Locked<Keyframe>>;
     type Error = StyleParseErrorKind<'i>;
@@ -533,44 +585,55 @@ impl<'a, 'i> QualifiedRuleParser<'i> for KeyframeListParser<'a> {
     fn parse_block<'t>(
         &mut self,
         selector: Self::Prelude,
-        source_location: SourceLocation,
+        start: &ParserState,
         input: &mut Parser<'i, 't>,
     ) -> Result<Self::QualifiedRule, ParseError<'i>> {
-        let context = ParserContext::new_with_rule_type(
-            self.context,
-            CssRuleType::Keyframe,
-            self.context.namespaces.unwrap(),
-        );
-
-        let parser = KeyframeDeclarationParser {
-            context: &context,
-            declarations: self.declarations,
-        };
-        let mut iter = DeclarationListParser::new(input, parser);
         let mut block = PropertyDeclarationBlock::new();
-        while let Some(declaration) = iter.next() {
-            match declaration {
-                Ok(()) => {
-                    block.extend(iter.parser.declarations.drain(), Importance::Normal);
-                },
-                Err((error, slice)) => {
-                    iter.parser.declarations.clear();
-                    let location = error.location;
-                    let error =
-                        ContextualParseError::UnsupportedKeyframePropertyDeclaration(slice, error);
-                    context.log_css_error(location, error);
-                },
-            }
-            // `parse_important` is not called here, `!important` is not allowed in keyframe blocks.
-        }
+        let declarations = &mut self.declarations;
+        self.context
+            .nest_for_rule(CssRuleType::Keyframe, |context| {
+                let mut parser = KeyframeDeclarationParser {
+                    context: &context,
+                    declarations,
+                };
+                let mut iter = RuleBodyParser::new(input, &mut parser);
+                while let Some(declaration) = iter.next() {
+                    match declaration {
+                        Ok(()) => {
+                            block.extend(iter.parser.declarations.drain(), Importance::Normal);
+                        },
+                        Err((error, slice)) => {
+                            iter.parser.declarations.clear();
+                            let location = error.location;
+                            let error =
+                                ContextualParseError::UnsupportedKeyframePropertyDeclaration(
+                                    slice, error,
+                                );
+                            context.log_css_error(location, error);
+                        },
+                    }
+                    // `parse_important` is not called here, `!important` is not allowed in keyframe blocks.
+                }
+            });
         Ok(Arc::new(self.shared_lock.wrap(Keyframe {
             selector,
             block: Arc::new(self.shared_lock.wrap(block)),
-            source_location,
+            source_location: start.source_location(),
         })))
     }
 }
 
+impl<'a, 'b, 'i> RuleBodyItemParser<'i, Arc<Locked<Keyframe>>, StyleParseErrorKind<'i>>
+    for KeyframeListParser<'a, 'b>
+{
+    fn parse_qualified(&self) -> bool {
+        true
+    }
+    fn parse_declarations(&self) -> bool {
+        false
+    }
+}
+
 struct KeyframeDeclarationParser<'a, 'b: 'a> {
     context: &'a ParserContext<'b>,
     declarations: &'a mut SourcePropertyDeclaration,
@@ -578,12 +641,17 @@ struct KeyframeDeclarationParser<'a, 'b: 'a> {
 
 /// Default methods reject all at rules.
 impl<'a, 'b, 'i> AtRuleParser<'i> for KeyframeDeclarationParser<'a, 'b> {
-    type PreludeNoBlock = ();
-    type PreludeBlock = ();
+    type Prelude = ();
     type AtRule = ();
     type Error = StyleParseErrorKind<'i>;
 }
 
+impl<'a, 'b, 'i> QualifiedRuleParser<'i> for KeyframeDeclarationParser<'a, 'b> {
+    type Prelude = ();
+    type QualifiedRule = ();
+    type Error = StyleParseErrorKind<'i>;
+}
+
 impl<'a, 'b, 'i> DeclarationParser<'i> for KeyframeDeclarationParser<'a, 'b> {
     type Declaration = ();
     type Error = StyleParseErrorKind<'i>;
@@ -610,3 +678,14 @@ impl<'a, 'b, 'i> DeclarationParser<'i> for KeyframeDeclarationParser<'a, 'b> {
         Ok(())
     }
 }
+
+impl<'a, 'b, 'i> RuleBodyItemParser<'i, (), StyleParseErrorKind<'i>>
+    for KeyframeDeclarationParser<'a, 'b>
+{
+    fn parse_qualified(&self) -> bool {
+        false
+    }
+    fn parse_declarations(&self) -> bool {
+        true
+    }
+}
