@@ -30,7 +30,7 @@ use crate::values::distance::{ComputeSquaredDistance, SquaredDistance};
 use crate::values::generics::{calc, NonNegative};
 use crate::values::specified::length::FontBaseSize;
 use crate::values::{specified, CSSFloat};
-use crate::Zero;
+use crate::{Zero, ZeroNoPercent};
 use app_units::Au;
 use malloc_size_of::{MallocSizeOf, MallocSizeOfOps};
 use serde::{Deserialize, Serialize};
@@ -241,7 +241,7 @@ impl LengthPercentage {
         // TODO: This could in theory take ownership of the calc node in `v` if
         // possible instead of cloning.
         let mut node = v.to_calc_node().into_owned();
-        node.negate();
+        node.map(std::ops::Neg::neg);
 
         let new_node = CalcNode::Sum(
             vec![
@@ -263,11 +263,11 @@ impl LengthPercentage {
             CalcNode::Leaf(l) => {
                 return match l {
                     CalcLengthPercentageLeaf::Length(l) => {
-                        Self::new_length(Length::new(clamping_mode.clamp(l.px())))
-                    },
-                    CalcLengthPercentageLeaf::Percentage(p) => {
-                        Self::new_percent(Percentage(clamping_mode.clamp(p.0)))
+                        Self::new_length(Length::new(clamping_mode.clamp(l.px())).normalized())
                     },
+                    CalcLengthPercentageLeaf::Percentage(p) => Self::new_percent(Percentage(
+                        clamping_mode.clamp(crate::values::normalize(p.0)),
+                    )),
                 }
             },
             _ => Self::new_calc_unchecked(Box::new(CalcLengthPercentage {
@@ -378,7 +378,7 @@ impl LengthPercentage {
     pub fn resolve(&self, basis: Length) -> Length {
         match self.unpack() {
             Unpacked::Length(l) => l,
-            Unpacked::Percentage(p) => Length::new(basis.px() * p.0),
+            Unpacked::Percentage(p) => (basis * p.0).normalized(),
             Unpacked::Calc(ref c) => c.resolve(basis),
         }
     }
@@ -556,6 +556,13 @@ impl Zero for LengthPercentage {
     }
 }
 
+impl ZeroNoPercent for LengthPercentage {
+    #[inline]
+    fn is_zero_no_percent(&self) -> bool {
+        self.is_definitely_zero() && !self.has_percentage()
+    }
+}
+
 impl Serialize for LengthPercentage {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
@@ -607,31 +614,20 @@ impl CalcLengthPercentageLeaf {
 impl PartialOrd for CalcLengthPercentageLeaf {
     fn partial_cmp(&self, other: &Self) -> Option<std::cmp::Ordering> {
         use self::CalcLengthPercentageLeaf::*;
-
-        if std::mem::discriminant(self) != std::mem::discriminant(other) {
-            return None;
-        }
-
+        // NOTE: Percentages can't be compared reasonably here because the
+        // percentage basis might be negative, see bug 1709018.
         match (self, other) {
             (&Length(ref one), &Length(ref other)) => one.partial_cmp(other),
-            (&Percentage(ref one), &Percentage(ref other)) => one.partial_cmp(other),
-            _ => {
-                match *self {
-                    Length(..) | Percentage(..) => {},
-                }
-                unsafe {
-                    debug_unreachable!("Forgot a branch?");
-                }
-            },
+            _ => None,
         }
     }
 }
 
 impl calc::CalcNodeLeaf for CalcLengthPercentageLeaf {
-    fn is_negative(&self) -> bool {
+    fn unitless_value(&self) -> f32 {
         match *self {
-            Self::Length(ref l) => l.px() < 0.,
-            Self::Percentage(ref p) => p.0 < 0.,
+            Self::Length(ref l) => l.px(),
+            Self::Percentage(ref p) => p.0,
         }
     }
 
@@ -661,10 +657,36 @@ impl calc::CalcNodeLeaf for CalcLengthPercentageLeaf {
         Ok(())
     }
 
-    fn mul_by(&mut self, scalar: f32) {
-        match *self {
-            Self::Length(ref mut l) => *l = *l * scalar,
-            Self::Percentage(ref mut p) => p.0 *= scalar,
+    fn try_op<O>(&self, other: &Self, op: O) -> Result<Self, ()>
+    where
+        O: Fn(f32, f32) -> f32,
+    {
+        match (self, other) {
+            (
+                &CalcLengthPercentageLeaf::Length(ref one),
+                &CalcLengthPercentageLeaf::Length(ref other),
+            ) => Ok(CalcLengthPercentageLeaf::Length(Length::new(op(
+                one.px(),
+                other.px(),
+            )))),
+            (
+                &CalcLengthPercentageLeaf::Percentage(one),
+                &CalcLengthPercentageLeaf::Percentage(other),
+            ) => Ok(CalcLengthPercentageLeaf::Percentage(Percentage(op(
+                one.0, other.0,
+            )))),
+            _ => Err(()),
+        }
+    }
+
+    fn map(&mut self, mut op: impl FnMut(f32) -> f32) {
+        match self {
+            CalcLengthPercentageLeaf::Length(value) => {
+                *value = Length::new(op(value.px()));
+            },
+            CalcLengthPercentageLeaf::Percentage(value) => {
+                *value = Percentage(op(value.0));
+            },
         }
     }
 
@@ -707,7 +729,7 @@ impl CalcLengthPercentage {
                 })
             })
             .unwrap();
-        Length::new(self.clamping_mode.clamp(px))
+        Length::new(self.clamping_mode.clamp(px)).normalized()
     }
 }
 
@@ -741,16 +763,18 @@ impl specified::CalcLengthPercentage {
         F: Fn(Length) -> Length,
     {
         use crate::values::specified::calc::Leaf;
-        use crate::values::specified::length::NoCalcLength;
 
         let node = self.node.map_leaves(|leaf| match *leaf {
             Leaf::Percentage(p) => CalcLengthPercentageLeaf::Percentage(Percentage(p)),
-            Leaf::Length(l) => CalcLengthPercentageLeaf::Length(match l {
-                NoCalcLength::Absolute(ref abs) => zoom_fn(abs.to_computed_value(context)),
-                NoCalcLength::FontRelative(ref fr) => fr.to_computed_value(context, base_size),
-                other => other.to_computed_value(context),
+            Leaf::Length(l) => CalcLengthPercentageLeaf::Length({
+                let result = l.to_computed_value_with_base_size(context, base_size);
+                if l.should_zoom_text() {
+                    zoom_fn(result)
+                } else {
+                    result
+                }
             }),
-            Leaf::Number(..) | Leaf::Angle(..) | Leaf::Time(..) => {
+            Leaf::Number(..) | Leaf::Angle(..) | Leaf::Time(..) | Leaf::Resolution(..) => {
                 unreachable!("Shouldn't have parsed")
             },
         });
@@ -764,11 +788,7 @@ impl specified::CalcLengthPercentage {
         context: &Context,
         base_size: FontBaseSize,
     ) -> LengthPercentage {
-        self.to_computed_value_with_zoom(
-            context,
-            |abs| context.maybe_zoom_text(abs.into()),
-            base_size,
-        )
+        self.to_computed_value_with_zoom(context, |abs| context.maybe_zoom_text(abs), base_size)
     }
 
     /// Compute the value into pixel length as CSSFloat without context,
