@@ -5,7 +5,7 @@
 //! CSS handling for the [`basic-shape`](https://drafts.csswg.org/css-shapes/#typedef-basic-shape)
 //! types that are generic over their `ToCss` implementations.
 
-use crate::values::animated::{Animate, Procedure, ToAnimatedZero};
+use crate::values::animated::{lists, Animate, Procedure, ToAnimatedZero};
 use crate::values::distance::{ComputeSquaredDistance, SquaredDistance};
 use crate::values::generics::border::GenericBorderRadius;
 use crate::values::generics::position::GenericPosition;
@@ -91,7 +91,6 @@ impl Default for ShapeBox {
 
 /// A value for the `clip-path` property.
 #[allow(missing_docs)]
-#[animation(no_bound(U))]
 #[derive(
     Animate,
     Clone,
@@ -106,6 +105,7 @@ impl Default for ShapeBox {
     ToResolvedValue,
     ToShmem,
 )]
+#[animation(no_bound(U))]
 #[repr(u8)]
 pub enum GenericClipPath<BasicShape, U> {
     #[animation(error)]
@@ -126,7 +126,6 @@ pub use self::GenericClipPath as ClipPath;
 
 /// A value for the `shape-outside` property.
 #[allow(missing_docs)]
-#[animation(no_bound(I))]
 #[derive(
     Animate,
     Clone,
@@ -141,6 +140,7 @@ pub use self::GenericClipPath as ClipPath;
     ToResolvedValue,
     ToShmem,
 )]
+#[animation(no_bound(I))]
 #[repr(u8)]
 pub enum GenericShapeOutside<BasicShape, I> {
     #[animation(error)]
@@ -193,7 +193,6 @@ pub use self::GenericBasicShape as BasicShape;
 
 /// <https://drafts.csswg.org/css-shapes/#funcdef-inset>
 #[allow(missing_docs)]
-#[css(function = "inset")]
 #[derive(
     Animate,
     Clone,
@@ -207,6 +206,7 @@ pub use self::GenericBasicShape as BasicShape;
     ToResolvedValue,
     ToShmem,
 )]
+#[css(function = "inset")]
 #[repr(C)]
 pub struct InsetRect<LengthPercentage, NonNegativeLengthPercentage> {
     pub rect: Rect<LengthPercentage>,
@@ -216,7 +216,6 @@ pub struct InsetRect<LengthPercentage, NonNegativeLengthPercentage> {
 
 /// <https://drafts.csswg.org/css-shapes/#funcdef-circle>
 #[allow(missing_docs)]
-#[css(function)]
 #[derive(
     Animate,
     Clone,
@@ -231,6 +230,7 @@ pub struct InsetRect<LengthPercentage, NonNegativeLengthPercentage> {
     ToResolvedValue,
     ToShmem,
 )]
+#[css(function)]
 #[repr(C)]
 pub struct Circle<H, V, NonNegativeLengthPercentage> {
     pub position: GenericPosition<H, V>,
@@ -239,7 +239,6 @@ pub struct Circle<H, V, NonNegativeLengthPercentage> {
 
 /// <https://drafts.csswg.org/css-shapes/#funcdef-ellipse>
 #[allow(missing_docs)]
-#[css(function)]
 #[derive(
     Animate,
     Clone,
@@ -254,6 +253,7 @@ pub struct Circle<H, V, NonNegativeLengthPercentage> {
     ToResolvedValue,
     ToShmem,
 )]
+#[css(function)]
 #[repr(C)]
 pub struct Ellipse<H, V, NonNegativeLengthPercentage> {
     pub position: GenericPosition<H, V>,
@@ -293,7 +293,6 @@ pub use self::GenericShapeRadius as ShapeRadius;
 /// A generic type for representing the `polygon()` function
 ///
 /// <https://drafts.csswg.org/css-shapes/#funcdef-polygon>
-#[css(comma, function = "polygon")]
 #[derive(
     Clone,
     Debug,
@@ -306,6 +305,7 @@ pub use self::GenericShapeRadius as ShapeRadius;
     ToResolvedValue,
     ToShmem,
 )]
+#[css(comma, function = "polygon")]
 #[repr(C)]
 pub struct GenericPolygon<LengthPercentage> {
     /// The filling rule for a polygon.
@@ -320,7 +320,9 @@ pub use self::GenericPolygon as Polygon;
 
 /// Coordinates for Polygon.
 #[derive(
+    Animate,
     Clone,
+    ComputeSquaredDistance,
     Debug,
     MallocSizeOf,
     PartialEq,
@@ -364,7 +366,6 @@ pub enum FillRule {
 /// The path function defined in css-shape-2.
 ///
 /// https://drafts.csswg.org/css-shapes-2/#funcdef-path
-#[css(comma)]
 #[derive(
     Animate,
     Clone,
@@ -379,6 +380,7 @@ pub enum FillRule {
     ToResolvedValue,
     ToShmem,
 )]
+#[css(comma)]
 #[repr(C)]
 pub struct Path {
     /// The filling rule for the svg path.
@@ -416,7 +418,7 @@ where
             dest.write_str(" round ")?;
             self.round.to_css(dest)?;
         }
-        dest.write_str(")")
+        dest.write_char(')')
     }
 }
 
@@ -432,11 +434,11 @@ where
         dest.write_str("circle(")?;
         if self.radius != Default::default() {
             self.radius.to_css(dest)?;
-            dest.write_str(" ")?;
+            dest.write_char(' ')?;
         }
         dest.write_str("at ")?;
         self.position.to_css(dest)?;
-        dest.write_str(")")
+        dest.write_char(')')
     }
 }
 
@@ -452,13 +454,13 @@ where
         dest.write_str("ellipse(")?;
         if self.semiaxis_x != Default::default() || self.semiaxis_y != Default::default() {
             self.semiaxis_x.to_css(dest)?;
-            dest.write_str(" ")?;
+            dest.write_char(' ')?;
             self.semiaxis_y.to_css(dest)?;
-            dest.write_str(" ")?;
+            dest.write_char(' ')?;
         }
         dest.write_str("at ")?;
         self.position.to_css(dest)?;
-        dest.write_str(")")
+        dest.write_char(')')
     }
 }
 
@@ -477,21 +479,8 @@ where
         if self.fill != other.fill {
             return Err(());
         }
-        if self.coordinates.len() != other.coordinates.len() {
-            return Err(());
-        }
-        let coordinates = self
-            .coordinates
-            .iter()
-            .zip(other.coordinates.iter())
-            .map(|(this, other)| {
-                Ok(PolygonCoord(
-                    this.0.animate(&other.0, procedure)?,
-                    this.1.animate(&other.1, procedure)?,
-                ))
-            })
-            .collect::<Result<Vec<_>, _>>()?
-            .into();
+        let coordinates =
+            lists::by_computed_value::animate(&self.coordinates, &other.coordinates, procedure)?;
         Ok(Polygon {
             fill: self.fill,
             coordinates,
@@ -507,18 +496,7 @@ where
         if self.fill != other.fill {
             return Err(());
         }
-        if self.coordinates.len() != other.coordinates.len() {
-            return Err(());
-        }
-        self.coordinates
-            .iter()
-            .zip(other.coordinates.iter())
-            .map(|(this, other)| {
-                let d1 = this.0.compute_squared_distance(&other.0)?;
-                let d2 = this.1.compute_squared_distance(&other.1)?;
-                Ok(d1 + d2)
-            })
-            .sum()
+        lists::by_computed_value::squared_distance(&self.coordinates, &other.coordinates)
     }
 }
 
