@@ -6,12 +6,13 @@
 
 use super::{Context, ToResolvedValue};
 
-use crate::values::computed;
+use crate::color::AbsoluteColor;
+use crate::values::computed::color as computed;
 use crate::values::generics::color as generics;
 
 impl ToResolvedValue for computed::Color {
     // A resolved color value is an rgba color, with currentcolor resolved.
-    type ResolvedValue = cssparser::RGBA;
+    type ResolvedValue = AbsoluteColor;
 
     #[inline]
     fn to_resolved_value(self, context: &Context) -> Self::ResolvedValue {
@@ -20,26 +21,28 @@ impl ToResolvedValue for computed::Color {
 
     #[inline]
     fn from_resolved_value(resolved: Self::ResolvedValue) -> Self {
-        generics::Color::Numeric(resolved)
+        generics::Color::Absolute(resolved)
     }
 }
 
-impl ToResolvedValue for computed::ColorOrAuto {
+impl ToResolvedValue for computed::CaretColor {
     // A resolved caret-color value is an rgba color, with auto resolving to
     // currentcolor.
-    type ResolvedValue = cssparser::RGBA;
+    type ResolvedValue = AbsoluteColor;
 
     #[inline]
     fn to_resolved_value(self, context: &Context) -> Self::ResolvedValue {
-        let color = match self {
+        let color = match self.0 {
             generics::ColorOrAuto::Color(color) => color,
-            generics::ColorOrAuto::Auto => generics::Color::CurrentColor,
+            generics::ColorOrAuto::Auto => generics::Color::currentcolor(),
         };
         color.to_resolved_value(context)
     }
 
     #[inline]
     fn from_resolved_value(resolved: Self::ResolvedValue) -> Self {
-        generics::ColorOrAuto::Color(computed::Color::from_resolved_value(resolved))
+        generics::CaretColor(generics::ColorOrAuto::Color(
+            computed::Color::from_resolved_value(resolved),
+        ))
     }
 }
