@@ -13,6 +13,3 @@ pub use crate::servo::url::{SpecifiedImageUrl, SpecifiedUrl};
 
 /// Specified <url> | <none>
 pub type UrlOrNone = GenericUrlOrNone<SpecifiedUrl>;
-
-/// Specified image <url> | <none>
-pub type ImageUrlOrNone = GenericUrlOrNone<SpecifiedImageUrl>;
