@@ -13,14 +13,13 @@ use style::gecko_bindings::structs::{Loader, LoaderReusableStyleSheets};
 use style::gecko_bindings::structs::{
     SheetLoadData, SheetLoadDataHolder, StyleSheet as DomStyleSheet,
 };
-use style::gecko_bindings::sugar::ownership::{FFIArcHelpers, HasBoxFFI, OwnedOrNull};
 use style::gecko_bindings::sugar::refptr::RefPtr;
 use style::global_style_data::GLOBAL_STYLE_DATA;
 use style::media_queries::MediaList;
 use style::parser::ParserContext;
 use style::shared_lock::{Locked, SharedRwLock};
+use style::stylesheets::import_rule::{ImportLayer, ImportSheet, ImportSupportsCondition};
 use style::stylesheets::AllowImportRules;
-use style::stylesheets::import_rule::ImportSheet;
 use style::stylesheets::{ImportRule, Origin, StylesheetLoader as StyleStylesheetLoader};
 use style::stylesheets::{StylesheetContents, UrlExtraData};
 use style::use_counters::UseCounters;
@@ -52,22 +51,27 @@ impl StyleStylesheetLoader for StylesheetLoader {
         _context: &ParserContext,
         lock: &SharedRwLock,
         media: Arc<Locked<MediaList>>,
+        supports: Option<ImportSupportsCondition>,
+        layer: ImportLayer,
     ) -> Arc<Locked<ImportRule>> {
+        // Ensure the supports conditions for this @import are true, if not, refuse to load
+        if !supports.as_ref().map_or(true, |s| s.enabled) {
+            return Arc::new(lock.wrap(ImportRule {
+                url,
+                stylesheet: ImportSheet::new_refused(),
+                supports,
+                layer,
+                source_location,
+            }));
+        }
+
         // After we get this raw pointer ImportRule will be moved into a lock and Arc
         // and so the Arc<Url> pointer inside will also move,
         // but the Url it points to or the allocating backing the String inside that Url won’t,
         // so this raw pointer will still be valid.
 
-        let child_sheet = unsafe {
-            Gecko_LoadStyleSheet(
-                self.0,
-                self.1,
-                self.2,
-                self.3,
-                &url,
-                media.into_strong(),
-            )
-        };
+        let child_sheet =
+            unsafe { Gecko_LoadStyleSheet(self.0, self.1, self.2, self.3, &url, media.into()) };
 
         debug_assert!(
             !child_sheet.is_null(),
@@ -77,8 +81,10 @@ impl StyleStylesheetLoader for StylesheetLoader {
         let stylesheet = ImportSheet::new(sheet);
         Arc::new(lock.wrap(ImportRule {
             url,
-            source_location,
             stylesheet,
+            supports,
+            layer,
+            source_location,
         }))
     }
 }
@@ -129,7 +135,7 @@ impl AsyncStylesheetParser {
 
         // Note: Parallel CSS parsing doesn't report CSS errors. When errors are
         // being logged, Gecko prevents the parallel parsing path from running.
-        let sheet = Arc::new(StylesheetContents::from_str(
+        let sheet = StylesheetContents::from_str(
             input,
             self.extra_data.clone(),
             self.origin,
@@ -138,21 +144,16 @@ impl AsyncStylesheetParser {
             None,
             self.quirks_mode.into(),
             self.line_number_offset,
-            use_counters.as_ref().map(|c| &**c),
+            use_counters.as_deref(),
             self.allow_import_rules,
             /* sanitized_output = */ None,
-        ));
-
-        let use_counters = match use_counters {
-            Some(c) => c.into_ffi().maybe(),
-            None => OwnedOrNull::null(),
-        };
+        );
 
         unsafe {
             bindings::Gecko_StyleSheet_FinishAsyncParse(
                 self.load_data.get(),
-                sheet.into_strong(),
-                use_counters,
+                sheet.into(),
+                use_counters.map_or(std::ptr::null_mut(), Box::into_raw),
             );
         }
     }
@@ -166,20 +167,35 @@ impl StyleStylesheetLoader for AsyncStylesheetParser {
         _context: &ParserContext,
         lock: &SharedRwLock,
         media: Arc<Locked<MediaList>>,
+        supports: Option<ImportSupportsCondition>,
+        layer: ImportLayer,
     ) -> Arc<Locked<ImportRule>> {
-        let stylesheet = ImportSheet::new_pending(self.origin, self.quirks_mode);
+        // Ensure the supports conditions for this @import are true, if not, refuse to load
+        if !supports.as_ref().map_or(true, |s| s.enabled) {
+            return Arc::new(lock.wrap(ImportRule {
+                url: url.clone(),
+                stylesheet: ImportSheet::new_refused(),
+                supports,
+                layer,
+                source_location,
+            }));
+        }
+
+        let stylesheet = ImportSheet::new_pending();
         let rule = Arc::new(lock.wrap(ImportRule {
             url: url.clone(),
-            source_location,
             stylesheet,
+            supports,
+            layer,
+            source_location,
         }));
 
         unsafe {
             bindings::Gecko_LoadStyleSheetAsync(
                 self.load_data.get(),
                 &url,
-                media.into_strong(),
-                rule.clone().into_strong(),
+                media.into(),
+                rule.clone().into(),
             );
         }
 
