@@ -24,8 +24,6 @@ mkdir -p $CROSSTOOLS_BUILD_DIR $LIBTAPI_BUILD_DIR
 
 cd $GECKO_PATH
 
-export PATH="$MOZ_FETCHES_DIR/binutils/bin:$PATH"
-
 # Common setup for libtapi and cctools
 export CC=$CLANG_DIR/bin/clang
 export CXX=$CLANG_DIR/bin/clang++
@@ -48,19 +46,25 @@ cmake $LIBTAPI_SOURCE_DIR/src/llvm \
       -DLLVM_INCLUDE_TESTS=OFF \
       -DCMAKE_BUILD_TYPE=RELEASE \
       -DCMAKE_INSTALL_PREFIX=$CROSSTOOLS_BUILD_DIR \
+      -DCMAKE_SYSROOT=$MOZ_FETCHES_DIR/sysroot \
+      -DCMAKE_EXE_LINKER_FLAGS=-fuse-ld=lld \
+      -DCMAKE_SHARED_LINKER_FLAGS=-fuse-ld=lld \
       -DLLVM_TARGETS_TO_BUILD="X86;ARM;AArch64" \
       -DTAPI_REPOSITORY_STRING=$TAPI_REPOSITORY \
       -DTAPI_FULL_VERSION=$TAPI_VERSION
 
-ninja clangBasic
-ninja libtapi install-libtapi install-tapi-headers
+ninja clangBasic -v
+ninja libtapi install-libtapi install-tapi-headers -v
 
 # Setup LDFLAGS late so run-at-build-time tools in the basic clang build don't
 # pick up the possibly-incompatible libstdc++ from clang.
 # Also set it up such that loading libtapi doesn't require a LD_LIBRARY_PATH.
 # (this requires two dollars and extra backslashing because it's used verbatim
 # via a Makefile)
-export LDFLAGS="-lpthread -Wl,-rpath-link,$CLANG_DIR/lib -Wl,-rpath,\\\$\$ORIGIN/../lib,-rpath,\\\$\$ORIGIN/../../clang/lib"
+export LDFLAGS="-fuse-ld=lld -lpthread -Wl,-rpath-link,$MOZ_FETCHES_DIR/sysroot/lib/x86_64-linux-gnu -Wl,-rpath-link,$MOZ_FETCHES_DIR/sysroot/usr/lib/x86_64-linux-gnu -Wl,-rpath,\\\$\$ORIGIN/../lib,-rpath,\\\$\$ORIGIN/../../clang/lib"
+
+export CC="$CC --sysroot=$MOZ_FETCHES_DIR/sysroot"
+export CXX="$CXX --sysroot=$MOZ_FETCHES_DIR/sysroot"
 
 # Configure crosstools-port
 cd $CROSSTOOLS_CCTOOLS_DIR
@@ -80,11 +84,10 @@ make -j `nproc --all` install
 
 # Build ldid
 cd $LDID_SOURCE_DIR
-make -j `nproc --all` install INSTALLPREFIX=$CROSSTOOLS_BUILD_DIR LDFLAGS="-Wl,-Bstatic -lcrypto -Wl,-Bdynamic -ldl -pthread"
+# The crypto library in the sysroot cannot be linked in a PIE executable so we use -no-pie
+make -j `nproc --all` install INSTALLPREFIX=$CROSSTOOLS_BUILD_DIR LDFLAGS="-no-pie -Wl,-Bstatic -lcrypto -Wl,-Bdynamic -ldl -pthread"
 
 strip $CROSSTOOLS_BUILD_DIR/bin/*
-# cctools-port doesn't include dsymutil but clang will need to find it.
-cp $CLANG_DIR/bin/dsymutil $CROSSTOOLS_BUILD_DIR/bin/x86_64-apple-darwin-dsymutil
 # various build scripts based on cmake want to find `lipo` without a prefix
 cp $CROSSTOOLS_BUILD_DIR/bin/x86_64-apple-darwin-lipo $CROSSTOOLS_BUILD_DIR/bin/lipo
 
@@ -94,4 +97,4 @@ done)
 
 # Put a tarball in the artifacts dir
 mkdir -p $UPLOAD_DIR
-tar cJf $UPLOAD_DIR/cctools.tar.xz -C $CROSSTOOLS_BUILD_DIR/.. `basename $CROSSTOOLS_BUILD_DIR`
+tar caf $UPLOAD_DIR/cctools.tar.zst -C $CROSSTOOLS_BUILD_DIR/.. `basename $CROSSTOOLS_BUILD_DIR`
