@@ -2,8 +2,6 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
-from __future__ import absolute_import
-
 import json
 import socket
 import sys
@@ -223,7 +221,8 @@ class TcpTransport(object):
 
                 try:
                     chunk = sock.recv(recv_bytes)
-                except OSError:
+                except socket.timeout:
+                    # Lets handle it with our own timeout check
                     continue
 
                 if not chunk:
