@@ -7,7 +7,6 @@
  */
 
 // This file expects the following files to be loaded.
-/* import-globals-from ../../../modules/StructuredLog.jsm */
 /* import-globals-from LogController.js */
 /* import-globals-from MemoryStats.js */
 /* import-globals-from MozillaLogger.js */
@@ -16,6 +15,11 @@
 
 "use strict";
 
+const { StructuredLogger, StructuredFormatter } =
+  SpecialPowers.ChromeUtils.importESModule(
+    "resource://testing-common/StructuredLog.sys.mjs"
+  );
+
 function getElement(id) {
   return typeof id == "string" ? document.getElementById(id) : id;
 }
@@ -75,6 +79,21 @@ function flattenArguments(lst /* ...*/) {
   return res;
 }
 
+function testInXOriginFrame() {
+  // Check if the test running in an iframe is a cross origin test.
+  try {
+    $("testframe").contentWindow.origin;
+    return false;
+  } catch (e) {
+    return true;
+  }
+}
+
+function testInDifferentProcess() {
+  // Check if the test running in an iframe that is loaded in a different process.
+  return SpecialPowers.Cu.isRemoteProxy($("testframe").contentWindow);
+}
+
 /**
  * TestRunner: A test runner for SimpleTest
  * TODO:
@@ -105,6 +124,9 @@ TestRunner.slowestTestTime = 0;
 TestRunner.slowestTestURL = "";
 TestRunner.interactiveDebugger = false;
 TestRunner.cleanupCrashes = false;
+TestRunner.timeoutAsPass = false;
+TestRunner.conditionedProfile = false;
+TestRunner.comparePrefs = false;
 
 TestRunner._expectingProcessCrash = false;
 TestRunner._structuredFormatter = new StructuredFormatter();
@@ -122,17 +144,58 @@ TestRunner._timeoutFactor = 1;
 TestRunner.jscovDirPrefix = "";
 var coverageCollector = {};
 
-TestRunner._checkForHangs = function() {
+function record(succeeded, expectedFail, msg) {
+  let successInfo;
+  let failureInfo;
+  if (expectedFail) {
+    successInfo = {
+      status: "PASS",
+      expected: "FAIL",
+      message: "TEST-UNEXPECTED-PASS",
+    };
+    failureInfo = {
+      status: "FAIL",
+      expected: "FAIL",
+      message: "TEST-KNOWN-FAIL",
+    };
+  } else {
+    successInfo = {
+      status: "PASS",
+      expected: "PASS",
+      message: "TEST-PASS",
+    };
+    failureInfo = {
+      status: "FAIL",
+      expected: "PASS",
+      message: "TEST-UNEXPECTED-FAIL",
+    };
+  }
+
+  let result = succeeded ? successInfo : failureInfo;
+
+  TestRunner.structuredLogger.testStatus(
+    TestRunner.currentTestURL,
+    msg,
+    result.status,
+    result.expected,
+    "",
+    ""
+  );
+}
+
+TestRunner._checkForHangs = function () {
   function reportError(win, msg) {
-    if ("SimpleTest" in win) {
-      win.SimpleTest.ok(false, msg);
+    if (testInXOriginFrame() || "SimpleTest" in win) {
+      record(false, TestRunner.timeoutAsPass, msg);
     } else if ("W3CTest" in win) {
       win.W3CTest.logFailure(msg);
     }
   }
 
   async function killTest(win) {
-    if ("SimpleTest" in win) {
+    if (testInXOriginFrame()) {
+      win.postMessage("SimpleTest:timeout", "*");
+    } else if ("SimpleTest" in win) {
       await win.SimpleTest.timeout();
       win.SimpleTest.finish();
     } else if ("W3CTest" in win) {
@@ -143,11 +206,12 @@ TestRunner._checkForHangs = function() {
   if (TestRunner._currentTest < TestRunner._urls.length) {
     var runtime = new Date().valueOf() - TestRunner._currentTestStartTime;
     if (runtime >= TestRunner.timeout * TestRunner._timeoutFactor) {
+      let testIframe = $("testframe");
       var frameWindow =
-        $("testframe").contentWindow.wrappedJSObject ||
-        $("testframe").contentWindow;
-      // TODO : Do this in a way that reports that the test ended with a status "TIMEOUT"
+        (!testInXOriginFrame() && testIframe.contentWindow.wrappedJSObject) ||
+        testIframe.contentWindow;
       reportError(frameWindow, "Test timed out.");
+      TestRunner.updateUI([{ result: false }]);
 
       // If we have too many timeouts, give up. We don't want to wait hours
       // for results if some bug causes lots of tests to time out.
@@ -173,6 +237,7 @@ TestRunner._checkForHangs = function() {
           await killTest(frameWindow);
         } catch (e) {
           reportError(frameWindow, "Test error: " + e);
+          TestRunner.updateUI([{ result: false }]);
         }
       }, 1000);
 
@@ -185,7 +250,7 @@ TestRunner._checkForHangs = function() {
   }
 };
 
-TestRunner.requestLongerTimeout = function(factor) {
+TestRunner.requestLongerTimeout = function (factor) {
   TestRunner._timeoutFactor = factor;
 };
 
@@ -195,7 +260,7 @@ TestRunner.requestLongerTimeout = function(factor) {
 TestRunner.repeat = 0;
 TestRunner._currentLoop = 1;
 
-TestRunner.expectAssertions = function(min, max) {
+TestRunner.expectAssertions = function (min, max) {
   if (typeof max == "undefined") {
     max = min;
   }
@@ -222,17 +287,17 @@ TestRunner.onComplete = null;
 TestRunner._failedTests = {};
 TestRunner._failureFile = "";
 
-TestRunner.addFailedTest = function(testName) {
+TestRunner.addFailedTest = function (testName) {
   if (TestRunner._failedTests[testName] == undefined) {
     TestRunner._failedTests[testName] = "";
   }
 };
 
-TestRunner.setFailureFile = function(fileName) {
+TestRunner.setFailureFile = function (fileName) {
   TestRunner._failureFile = fileName;
 };
 
-TestRunner.generateFailureList = function() {
+TestRunner.generateFailureList = function () {
   if (TestRunner._failureFile) {
     var failures = new MozillaFileLogger(TestRunner._failureFile);
     failures.log(JSON.stringify(TestRunner._failedTests));
@@ -247,8 +312,8 @@ TestRunner.generateFailureList = function() {
 // This delimiter is used to avoid interleaving Mochitest/Gecko logs.
 var LOG_DELIMITER = "\ue175\uee31\u2c32\uacbf";
 
-// A log callback for StructuredLog.jsm
-TestRunner._dumpMessage = function(message) {
+// A log callback for StructuredLog.sys.mjs
+TestRunner._dumpMessage = function (message) {
   var str;
 
   // This is a directive to python to format these messages
@@ -275,19 +340,21 @@ TestRunner._dumpMessage = function(message) {
   }
 };
 
-// From https://dxr.mozilla.org/mozilla-central/source/testing/modules/StructuredLog.jsm
+// From https://searchfox.org/mozilla-central/source/testing/modules/StructuredLog.sys.mjs
 TestRunner.structuredLogger = new StructuredLogger(
   "mochitest",
-  TestRunner._dumpMessage
+  TestRunner._dumpMessage,
+  [],
+  TestRunner
 );
-TestRunner.structuredLogger.deactivateBuffering = function() {
-  TestRunner.structuredLogger._logData("buffering_off");
+TestRunner.structuredLogger.deactivateBuffering = function () {
+  TestRunner.structuredLogger.logData("buffering_off");
 };
-TestRunner.structuredLogger.activateBuffering = function() {
-  TestRunner.structuredLogger._logData("buffering_on");
+TestRunner.structuredLogger.activateBuffering = function () {
+  TestRunner.structuredLogger.logData("buffering_on");
 };
 
-TestRunner.log = function(msg) {
+TestRunner.log = function (msg) {
   if (TestRunner.logEnabled) {
     TestRunner.structuredLogger.info(msg);
   } else {
@@ -295,7 +362,7 @@ TestRunner.log = function(msg) {
   }
 };
 
-TestRunner.error = function(msg) {
+TestRunner.error = function (msg) {
   if (TestRunner.logEnabled) {
     TestRunner.structuredLogger.error(msg);
   } else {
@@ -304,7 +371,7 @@ TestRunner.error = function(msg) {
   }
 };
 
-TestRunner.failureHandler = function() {
+TestRunner.failureHandler = function () {
   if (TestRunner.runUntilFailure) {
     TestRunner._haltTests = true;
   }
@@ -320,7 +387,7 @@ TestRunner.failureHandler = function() {
 /**
  * Toggle element visibility
  **/
-TestRunner._toggle = function(el) {
+TestRunner._toggle = function (el) {
   if (el.className == "noshow") {
     el.className = "";
     el.style.cssText = "";
@@ -333,7 +400,7 @@ TestRunner._toggle = function(el) {
 /**
  * Creates the iframe that contains a test
  **/
-TestRunner._makeIframe = function(url, retry) {
+TestRunner._makeIframe = function (url, retry) {
   var iframe = $("testframe");
   if (
     url != "about:blank" &&
@@ -345,7 +412,7 @@ TestRunner._makeIframe = function(url, retry) {
     SpecialPowers.focus();
     iframe.focus();
     if (retry < 3) {
-      window.setTimeout(function() {
+      window.setTimeout(function () {
         TestRunner._makeIframe(url, retry + 1);
       }, 1000);
       return;
@@ -356,7 +423,30 @@ TestRunner._makeIframe = function(url, retry) {
     );
   }
   window.scrollTo(0, $("indicator").offsetTop);
-  iframe.src = url;
+  try {
+    let urlObj = new URL(url);
+    if (TestRunner.xOriginTests) {
+      // The test will run in a xorigin iframe, so we pass in additional test params in the
+      // URL since the content process won't be able to access them from the parentRunner
+      // directly.
+      let params = TestRunner.getParameterInfo();
+      urlObj.searchParams.append(
+        "currentTestURL",
+        urlObj.pathname.replace("/tests/", "")
+      );
+      urlObj.searchParams.append("closeWhenDone", params.closeWhenDone);
+      urlObj.searchParams.append("showTestReport", TestRunner.showTestReport);
+      urlObj.searchParams.append("expected", TestRunner.expected);
+      iframe.src = urlObj.href;
+    } else {
+      iframe.src = url;
+    }
+  } catch {
+    // If the provided `url` is not a valid URL (i.e. doesn't include a protocol)
+    // then the new URL() constructor will raise a TypeError. This is expected in the
+    // usual case (i.e. non-xorigin iFrame tests) so set the URL in the usual way.
+    iframe.src = url;
+  }
   iframe.name = url;
   iframe.width = "500";
 };
@@ -366,41 +456,60 @@ TestRunner._makeIframe = function(url, retry) {
  * We use this to tell whether the test has navigated to another test without
  * being finished first.
  */
-TestRunner.getLoadedTestURL = function() {
-  var prefix = "";
-  // handle mochitest-chrome URIs
-  if ($("testframe").contentWindow.location.protocol == "chrome:") {
-    prefix = "chrome://mochitests";
+TestRunner.getLoadedTestURL = function () {
+  if (!testInXOriginFrame()) {
+    var prefix = "";
+    // handle mochitest-chrome URIs
+    if ($("testframe").contentWindow.location.protocol == "chrome:") {
+      prefix = "chrome://mochitests";
+    }
+    return prefix + $("testframe").contentWindow.location.pathname;
   }
-  return prefix + $("testframe").contentWindow.location.pathname;
+  return TestRunner.currentTestURL;
 };
 
-TestRunner.setParameterInfo = function(params) {
+TestRunner.setParameterInfo = function (params) {
   this._params = params;
 };
 
-TestRunner.getParameterInfo = function() {
+TestRunner.getParameterInfo = function () {
   return this._params;
 };
 
+/**
+ * Print information about which prefs are set.
+ * This is used to help validate that the tests are actually
+ * running in the expected context.
+ */
+TestRunner.dumpPrefContext = function () {
+  let prefs = ["fission.autostart"];
+
+  let message = ["Dumping test context:"];
+  prefs.forEach(function formatPref(pref) {
+    let val = SpecialPowers.getBoolPref(pref);
+    message.push(pref + "=" + val);
+  });
+  TestRunner.structuredLogger.info(message.join("\n  "));
+};
+
 /**
  * TestRunner entry point.
  *
  * The arguments are the URLs of the test to be ran.
  *
  **/
-TestRunner.runTests = function(/*url...*/) {
+TestRunner.runTests = function (/*url...*/) {
   TestRunner.structuredLogger.info("SimpleTest START");
+  TestRunner.dumpPrefContext();
   TestRunner.originalTestURL = $("current-test").innerHTML;
 
   SpecialPowers.registerProcessCrashObservers();
 
   // Initialize code coverage
   if (TestRunner.jscovDirPrefix != "") {
-    var CoverageCollector = SpecialPowers.Cu.import(
-      "resource://testing-common/CoverageUtils.jsm",
-      {}
-    ).CoverageCollector;
+    var { CoverageCollector } = SpecialPowers.ChromeUtils.importESModule(
+      "resource://testing-common/CoverageUtils.sys.mjs"
+    );
     coverageCollector = new CoverageCollector(TestRunner.jscovDirPrefix);
   }
 
@@ -426,7 +535,7 @@ TestRunner.runTests = function(/*url...*/) {
  * Used for running a set of tests in a loop for debugging purposes
  * Takes an array of URLs
  **/
-TestRunner.resetTests = function(listURLs) {
+TestRunner.resetTests = function (listURLs) {
   TestRunner._currentTest = 0;
   // Reset our "Current-test" line - functionality depends on it
   $("current-test").innerHTML = TestRunner.originalTestURL;
@@ -442,7 +551,7 @@ TestRunner.resetTests = function(listURLs) {
   TestRunner.runNextTest();
 };
 
-TestRunner.getNextUrl = function() {
+TestRunner.getNextUrl = function () {
   var url = "";
   // sometimes we have a subtest/harness which doesn't use a manifest
   if (
@@ -463,7 +572,7 @@ TestRunner.getNextUrl = function() {
  * Run the next test. If no test remains, calls onComplete().
  **/
 TestRunner._haltTests = false;
-TestRunner.runNextTest = function() {
+async function _runNextTest() {
   if (
     TestRunner._currentTest < TestRunner._urls.length &&
     !TestRunner._haltTests
@@ -481,6 +590,12 @@ TestRunner.runNextTest = function() {
 
     TestRunner.structuredLogger.testStart(url);
 
+    if (TestRunner._urls[TestRunner._currentTest].test.allow_xul_xbl) {
+      await SpecialPowers.pushPermissions([
+        { type: "allowXULXBL", allow: true, context: "http://mochi.test:8888" },
+        { type: "allowXULXBL", allow: true, context: "http://example.org" },
+      ]);
+    }
     TestRunner._makeIframe(url, 0);
   } else {
     $("current-test").innerHTML = "<b>Finished</b>";
@@ -497,7 +612,7 @@ TestRunner.runNextTest = function() {
       // No |$('testframe').contentWindow|, so manually update: ...
       // ... the log,
       TestRunner.structuredLogger.error(
-        "SimpleTest/TestRunner.js | No checks actually run"
+        "TEST-UNEXPECTED-FAIL | SimpleTest/TestRunner.js | No checks actually run"
       );
       // ... the count,
       $("fail-count").innerHTML = 1;
@@ -558,16 +673,17 @@ TestRunner.runNextTest = function() {
       coverageCollector.finalize();
     }
   }
-};
+}
+TestRunner.runNextTest = _runNextTest;
 
-TestRunner.expectChildProcessCrash = function() {
+TestRunner.expectChildProcessCrash = function () {
   TestRunner._expectingProcessCrash = true;
 };
 
 /**
  * This stub is called by SimpleTest when a test is finished.
  **/
-TestRunner.testFinished = function(tests) {
+TestRunner.testFinished = function (tests) {
   // Need to track subtests recorded here separately or else they'll
   // trigger the `result after SimpleTest.finish()` error.
   var extraTests = [];
@@ -625,7 +741,8 @@ TestRunner.testFinished = function(tests) {
         result = "ERROR";
       }
 
-      var unexpectedCrashDumpFiles = await SpecialPowers.findUnexpectedCrashDumpFiles();
+      var unexpectedCrashDumpFiles =
+        await SpecialPowers.findUnexpectedCrashDumpFiles();
       TestRunner._expectingProcessCrash = false;
       if (unexpectedCrashDumpFiles.length) {
         let subtest = "unexpected-crash-dump-found";
@@ -641,7 +758,7 @@ TestRunner.testFinished = function(tests) {
         );
         extraTests.push({ name: subtest, result: false });
         result = "CRASH";
-        unexpectedCrashDumpFiles.sort().forEach(function(aFilename) {
+        unexpectedCrashDumpFiles.sort().forEach(function (aFilename) {
           TestRunner.structuredLogger.info(
             "Found unexpected crash dump file " + aFilename + "."
           );
@@ -674,7 +791,7 @@ TestRunner.testFinished = function(tests) {
 
       SpecialPowers.addProfilerMarker(
         "TestRunner",
-        TestRunner._currentTestStartTimestamp,
+        { category: "Test", startTime: TestRunner._currentTestStartTimestamp },
         TestRunner.currentTestURL
       );
       var runtime = new Date().valueOf() - TestRunner._currentTestStartTime;
@@ -704,38 +821,44 @@ TestRunner.testFinished = function(tests) {
       }
 
       var interstitialURL;
-      if ($("testframe").contentWindow.location.protocol == "chrome:") {
+      if (
+        !testInXOriginFrame() &&
+        $("testframe").contentWindow.location.protocol == "chrome:"
+      ) {
         interstitialURL = "tests/SimpleTest/iframe-between-tests.html";
       } else {
         interstitialURL = "/tests/SimpleTest/iframe-between-tests.html";
       }
       // check if there were test run after SimpleTest.finish, which should never happen
-      $("testframe").contentWindow.addEventListener("unload", function() {
-        var testwin = $("testframe").contentWindow;
-        if (
-          testwin.SimpleTest &&
-          testwin.SimpleTest._tests.length != testwin.SimpleTest.testsLength
-        ) {
-          var wrongtestlength =
-            testwin.SimpleTest._tests.length - testwin.SimpleTest.testsLength;
-          var wrongtestname = "";
-          for (var i = 0; i < wrongtestlength; i++) {
-            wrongtestname =
-              testwin.SimpleTest._tests[testwin.SimpleTest.testsLength + i]
-                .name;
-            TestRunner.structuredLogger.error(
-              TestRunner.currentTestURL +
-                " logged result after SimpleTest.finish(): " +
-                wrongtestname
-            );
+      if (!testInXOriginFrame()) {
+        $("testframe").contentWindow.addEventListener("unload", function () {
+          var testwin = $("testframe").contentWindow;
+          if (
+            testwin.SimpleTest &&
+            testwin.SimpleTest._tests.length != testwin.SimpleTest.testsLength
+          ) {
+            var wrongtestlength =
+              testwin.SimpleTest._tests.length - testwin.SimpleTest.testsLength;
+            var wrongtestname = "";
+            for (var i = 0; i < wrongtestlength; i++) {
+              wrongtestname =
+                testwin.SimpleTest._tests[testwin.SimpleTest.testsLength + i]
+                  .name;
+              TestRunner.structuredLogger.error(
+                "TEST-UNEXPECTED-FAIL | " +
+                  TestRunner.currentTestURL +
+                  " logged result after SimpleTest.finish(): " +
+                  wrongtestname
+              );
+            }
+            TestRunner.updateUI([{ result: false }]);
           }
-          TestRunner.updateUI([{ result: false }]);
-        }
-      });
+        });
+      }
       TestRunner._makeIframe(interstitialURL, 0);
     }
 
-    SpecialPowers.executeAfterFlushingMessageQueue(async function() {
+    SpecialPowers.executeAfterFlushingMessageQueue(async function () {
       await SpecialPowers.waitForCrashes(TestRunner._expectingProcessCrash);
       await cleanUpCrashDumpFiles();
       await SpecialPowers.flushPermissions();
@@ -745,12 +868,30 @@ TestRunner.testFinished = function(tests) {
   });
 };
 
-TestRunner.testUnloaded = function() {
+/**
+ * This stub is called by XOrigin Tests to report assertion count.
+ **/
+TestRunner._xoriginAssertionCount = 0;
+TestRunner.addAssertionCount = function (count) {
+  if (!testInXOriginFrame()) {
+    TestRunner.error(
+      `addAssertionCount should only be called by a cross origin test`
+    );
+    return;
+  }
+
+  if (testInDifferentProcess()) {
+    TestRunner._xoriginAssertionCount += count;
+  }
+};
+
+TestRunner.testUnloaded = function () {
   // If we're in a debug build, check assertion counts.  This code is
   // similar to the code in Tester_nextTest in browser-test.js used
   // for browser-chrome mochitests.
   if (SpecialPowers.isDebugBuild) {
-    var newAssertionCount = SpecialPowers.assertionCount();
+    var newAssertionCount =
+      SpecialPowers.assertionCount() + TestRunner._xoriginAssertionCount;
     var numAsserts = newAssertionCount - TestRunner._lastAssertionCount;
     TestRunner._lastAssertionCount = newAssertionCount;
 
@@ -775,6 +916,31 @@ TestRunner.testUnloaded = function() {
     );
   }
 
+  // Always do this, so we can "reset" preferences between tests
+  SpecialPowers.comparePrefsToBaseline(
+    TestRunner.ignorePrefs,
+    TestRunner.verifyPrefsNextTest
+  );
+};
+
+TestRunner.verifyPrefsNextTest = function (p) {
+  if (TestRunner.comparePrefs) {
+    let prefs = Array.from(SpecialPowers.Cu.waiveXrays(p), x =>
+      SpecialPowers.unwrapIfWrapped(SpecialPowers.Cu.unwaiveXrays(x))
+    );
+    prefs.forEach(pr =>
+      TestRunner.structuredLogger.error(
+        "TEST-UNEXPECTED-FAIL | " +
+          TestRunner.currentTestURL +
+          " | changed preference: " +
+          pr
+      )
+    );
+  }
+  TestRunner.doNextTest();
+};
+
+TestRunner.doNextTest = function () {
   TestRunner._currentTest++;
   if (TestRunner.runSlower) {
     setTimeout(TestRunner.runNextTest, 1000);
@@ -786,7 +952,7 @@ TestRunner.testUnloaded = function() {
 /**
  * Get the results.
  */
-TestRunner.countResults = function(tests) {
+TestRunner.countResults = function (tests) {
   var nOK = 0;
   var nNotOK = 0;
   var nTodo = 0;
@@ -806,11 +972,11 @@ TestRunner.countResults = function(tests) {
 /**
  * Print out table of any error messages found during looped run
  */
-TestRunner.displayLoopErrors = function(tableName, tests) {
+TestRunner.displayLoopErrors = function (tableName, tests) {
   if (TestRunner.countResults(tests).notOK > 0) {
     var table = $(tableName);
     var curtest;
-    if (table.rows.length == 0) {
+    if (!table.rows.length) {
       //if table headers are not yet generated, make them
       var row = table.insertRow(table.rows.length);
       var cell = row.insertCell(0);
@@ -849,7 +1015,7 @@ TestRunner.displayLoopErrors = function(tableName, tests) {
   }
 };
 
-TestRunner.updateUI = function(tests) {
+TestRunner.updateUI = function (tests) {
   var results = TestRunner.countResults(tests);
   var passCount = parseInt($("pass-count").innerHTML) + results.OK;
   var failCount = parseInt($("fail-count").innerHTML) + results.notOK;
@@ -891,3 +1057,47 @@ TestRunner.updateUI = function(tests) {
     TestRunner.displayLoopErrors("fail-table", tests);
   }
 };
+
+// XOrigin Tests
+// If "--enable-xorigin-tests" is set, mochitests are run in a cross origin iframe.
+// The parent process will run at http://mochi.xorigin-test:8888", and individual
+// mochitests will be launched in a cross-origin iframe at http://mochi.test:8888.
+
+var xOriginDispatchMap = {
+  runner: TestRunner,
+  logger: TestRunner.structuredLogger,
+  addFailedTest: TestRunner.addFailedTest,
+  expectAssertions: TestRunner.expectAssertions,
+  expectChildProcessCrash: TestRunner.expectChildProcessCrash,
+  requestLongerTimeout: TestRunner.requestLongerTimeout,
+  "structuredLogger.deactivateBuffering":
+    TestRunner.structuredLogger.deactivateBuffering,
+  "structuredLogger.activateBuffering":
+    TestRunner.structuredLogger.activateBuffering,
+  "structuredLogger.testStatus": TestRunner.structuredLogger.testStatus,
+  "structuredLogger.info": TestRunner.structuredLogger.info,
+  "structuredLogger.warning": TestRunner.structuredLogger.warning,
+  "structuredLogger.error": TestRunner.structuredLogger.error,
+  testFinished: TestRunner.testFinished,
+  addAssertionCount: TestRunner.addAssertionCount,
+};
+
+function xOriginTestRunnerHandler(event) {
+  if (event.data.harnessType != "SimpleTest") {
+    return;
+  }
+  // Handles messages from xOriginRunner in SimpleTest.js.
+  if (event.data.command in xOriginDispatchMap) {
+    xOriginDispatchMap[event.data.command].apply(
+      xOriginDispatchMap[event.data.applyOn],
+      event.data.params
+    );
+  } else {
+    TestRunner.error(`Command ${event.data.command} not found
+      in xOriginDispatchMap`);
+  }
+}
+
+TestRunner.setXOriginEventHandler = function () {
+  window.addEventListener("message", xOriginTestRunnerHandler);
+};
