@@ -4,10 +4,9 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
-from __future__ import absolute_import
-
 import json
 import os
+import pathlib
 import re
 import shutil
 import subprocess
@@ -18,12 +17,12 @@ from abc import ABCMeta, abstractmethod
 
 import mozinfo
 import mozprocess
-import mozversion
 import mozproxy.utils as mpu
+import mozversion
+import six
 from mozprofile import create_profile
 from mozproxy import get_playback
 
-
 # need this so raptor imports work both from /raptor and via mach
 here = os.path.abspath(os.path.dirname(__file__))
 paths = [here]
@@ -36,18 +35,17 @@ for path in paths:
         raise IOError("%s does not exist. " % path)
     sys.path.insert(0, path)
 
-
 from cmdline import FIREFOX_ANDROID_APPS
-from condprof.client import get_profile, ProfileNotFoundError
+from condprof.client import ProfileNotFoundError, get_profile
 from condprof.util import get_current_platform
-from logger.logger import RaptorLogger
 from gecko_profile import GeckoProfile
+from logger.logger import RaptorLogger
 from results import RaptorResultsHandler
 
 LOG = RaptorLogger(component="raptor-perftest")
 
 # - mozproxy.utils LOG displayed INFO messages even when LOG.error() was used in mitm.py
-mpu.LOG = RaptorLogger(component='raptor-mitmproxy')
+mpu.LOG = RaptorLogger(component="raptor-mitmproxy")
 
 try:
     from mozbuild.base import MozbuildObject
@@ -61,11 +59,10 @@ POST_DELAY_DEBUG = 3000
 POST_DELAY_DEFAULT = 30000
 
 
+@six.add_metaclass(ABCMeta)
 class Perftest(object):
     """Abstract base class for perftests that execute via a subharness,
-either Raptor or browsertime."""
-
-    __metaclass__ = ABCMeta
+    either Raptor or browsertime."""
 
     def __init__(
         self,
@@ -79,6 +76,10 @@ either Raptor or browsertime."""
         gecko_profile=False,
         gecko_profile_interval=None,
         gecko_profile_entries=None,
+        gecko_profile_extra_threads=None,
+        gecko_profile_threads=None,
+        gecko_profile_features=None,
+        extra_profiler_run=False,
         symbols_path=None,
         host=None,
         power_test=False,
@@ -91,19 +92,28 @@ either Raptor or browsertime."""
         post_startup_delay=POST_DELAY_DEFAULT,
         interrupt_handler=None,
         e10s=True,
-        enable_webrender=False,
         results_handler_class=RaptorResultsHandler,
-        no_conditioned_profile=False,
         device_name=None,
         disable_perf_tuning=False,
-        conditioned_profile_scenario='settled',
+        conditioned_profile=None,
+        test_bytecode_cache=False,
+        chimera=False,
         extra_prefs={},
+        environment={},
         project="mozilla-central",
         verbose=False,
+        python=None,
+        fission=True,
+        extra_summary_methods=[],
+        benchmark_repository=None,
+        benchmark_revision=None,
+        benchmark_branch=None,
         **kwargs
     ):
+        self._remote_test_root = None
         self._dirs_to_remove = []
         self.verbose = verbose
+        self.page_count = []
 
         # Override the magic --host HOST_IP with the value of the environment variable.
         if host == "HOST_IP":
@@ -119,6 +129,10 @@ either Raptor or browsertime."""
             "gecko_profile": gecko_profile,
             "gecko_profile_interval": gecko_profile_interval,
             "gecko_profile_entries": gecko_profile_entries,
+            "gecko_profile_extra_threads": gecko_profile_extra_threads,
+            "gecko_profile_threads": gecko_profile_threads,
+            "gecko_profile_features": gecko_profile_features,
+            "extra_profiler_run": extra_profiler_run,
             "symbols_path": symbols_path,
             "host": host,
             "power_test": power_test,
@@ -129,37 +143,42 @@ either Raptor or browsertime."""
             "is_release_build": is_release_build,
             "enable_control_server_wait": memory_test or cpu_test,
             "e10s": e10s,
-            "enable_webrender": enable_webrender,
-            "no_conditioned_profile": no_conditioned_profile,
             "device_name": device_name,
-            "enable_fission": extra_prefs.get("fission.autostart", False),
+            "fission": fission,
             "disable_perf_tuning": disable_perf_tuning,
-            "conditioned_profile_scenario": conditioned_profile_scenario,
+            "conditioned_profile": conditioned_profile,
+            "test_bytecode_cache": test_bytecode_cache,
+            "chimera": chimera,
             "extra_prefs": extra_prefs,
+            "environment": environment,
             "project": project,
-            "verbose": verbose
+            "verbose": verbose,
+            "extra_summary_methods": extra_summary_methods,
+            "benchmark_repository": benchmark_repository,
+            "benchmark_revision": benchmark_revision,
+            "benchmark_branch": benchmark_branch,
         }
 
         self.firefox_android_apps = FIREFOX_ANDROID_APPS
+
         # We are deactivating the conditioned profiles for:
         # - win10-aarch64 : no support for geckodriver see 1582757
-        # - fennec_aurora: no conditioned profiles created see 1606199
         # - reference browser: no conditioned profiles created see 1606767
-        self.using_condprof = not (
-             (self.config["platform"] == "win" and self.config["processor"] == "aarch64")
-             or self.config["binary"] == "org.mozilla.fennec_aurora"
-             or self.config["binary"] == "org.mozilla.reference.browser.raptor"
-             or self.config["no_conditioned_profile"]
-        )
-        if self.using_condprof:
+        if (
+            self.config["platform"] == "win" and self.config["processor"] == "aarch64"
+        ) or self.config["binary"] == "org.mozilla.reference.browser.raptor":
+            self.config["conditioned_profile"] = None
+
+        if self.config["conditioned_profile"]:
             LOG.info("Using a conditioned profile.")
         else:
             LOG.info("Using an empty profile.")
-        self.config["using_condprof"] = self.using_condprof
 
-        # We can never use e10s on fennec
-        if self.config["app"] == "fennec":
-            self.config["e10s"] = False
+        # To differentiate between chrome/firefox failures, we
+        # set an app variable in the logger which prefixes messages
+        # with the app name
+        if self.config["app"] in ("chrome", "chrome-m", "chromium", "custom-car"):
+            LOG.set_app(self.config["app"])
 
         self.browser_name = None
         self.browser_version = None
@@ -172,6 +191,9 @@ either Raptor or browsertime."""
         self.device = None
         self.runtime_error = None
         self.profile_class = profile_class or app
+        # Use the `chromium` profile class for custom-car
+        if app in ["custom-car"]:
+            self.profile_class = "chromium"
         self.conditioned_profile_dir = None
         self.interrupt_handler = interrupt_handler
         self.results_handler = results_handler_class(**self.config)
@@ -182,23 +204,25 @@ either Raptor or browsertime."""
         self.results_handler.add_browser_meta(self.config["app"], browser_version)
 
         # debug mode is currently only supported when running locally
-        self.run_local = self.config['run_local']
+        self.run_local = self.config["run_local"]
         self.debug_mode = debug_mode if self.run_local else False
 
         # For the post startup delay, we want to max it to 1s when using the
         # conditioned profiles.
-        if self.using_condprof and not self.run_local:
+        if self.config.get("conditioned_profile"):
             self.post_startup_delay = min(post_startup_delay, POST_DELAY_CONDPROF)
         else:
             # if running debug-mode reduce the pause after browser startup
             if self.debug_mode:
-                self.post_startup_delay = min(post_startup_delay,
-                                              POST_DELAY_DEBUG)
+                self.post_startup_delay = min(post_startup_delay, POST_DELAY_DEBUG)
             else:
                 self.post_startup_delay = post_startup_delay
 
         LOG.info("Post startup delay set to %d ms" % self.post_startup_delay)
         LOG.info("main raptor init, config is: %s" % str(self.config))
+
+        # TODO: Move this outside of the perftest initialization, it contains
+        # platform-specific code
         self.build_browser_profile()
 
         # Crashes counter
@@ -213,15 +237,79 @@ either Raptor or browsertime."""
     def is_localhost(self):
         return self.config.get("host") in ("localhost", "127.0.0.1")
 
+    @property
+    def android_external_storage(self):
+        return "/sdcard/test_root/"
+
     @property
     def conditioned_profile_copy(self):
         """Returns a copy of the original conditioned profile that was created."""
         condprof_copy = os.path.join(self._get_temp_dir(), "profile")
-        shutil.copytree(self.conditioned_profile_dir, condprof_copy)
+        shutil.copytree(
+            self.conditioned_profile_dir,
+            condprof_copy,
+            ignore=shutil.ignore_patterns("lock"),
+        )
         LOG.info("Created a conditioned-profile copy: %s" % condprof_copy)
         return condprof_copy
 
-    def get_conditioned_profile(self):
+    def build_conditioned_profile(self):
+        # Late import so python-test doesn't import it
+        import asyncio
+
+        # The following import patchs an issue with invalid
+        # content-type, see bug 1655869
+        from condprof import patch  # noqa
+        from condprof.runner import Runner
+
+        if not getattr(self, "browsertime"):
+            raise Exception(
+                "Building conditioned profiles within a test is only supported "
+                "when using Browsertime."
+            )
+
+        geckodriver = getattr(self, "browsertime_geckodriver", None)
+        if not geckodriver:
+            geckodriver = (
+                sys.platform.startswith("win") and "geckodriver.exe" or "geckodriver"
+            )
+
+        scenario = self.config.get("conditioned_profile")
+        runner = Runner(
+            profile=None,
+            firefox=self.config.get("binary"),
+            geckodriver=geckodriver,
+            archive=None,
+            device_name=self.config.get("device_name"),
+            strict=True,
+            visible=True,
+            force_new=True,
+            skip_logs=True,
+            remote_test_root=self.android_external_storage,
+        )
+
+        if self.config.get("is_release_build", False):
+            # Enable non-local connections for building the conditioned profile
+            self.enable_non_local_connections()
+
+        if scenario == "settled-youtube":
+            runner.prepare(scenario, "youtube")
+
+            loop = asyncio.get_event_loop()
+            loop.run_until_complete(runner.one_run(scenario, "youtube"))
+        else:
+            runner.prepare(scenario, "default")
+
+            loop = asyncio.get_event_loop()
+            loop.run_until_complete(runner.one_run(scenario, "default"))
+
+        if self.config.get("is_release_build", False):
+            self.disable_non_local_connections()
+
+        self.conditioned_profile_dir = runner.env.profile
+        return self.conditioned_profile_copy
+
+    def get_conditioned_profile(self, binary=None):
         """Downloads a platform-specific conditioned profile, using the
         condprofile client API; returns a self.conditioned_profile_dir"""
         if self.conditioned_profile_dir:
@@ -229,6 +317,10 @@ either Raptor or browsertime."""
             # will get deleted after it's done with
             return self.conditioned_profile_copy
 
+        # Build the conditioned profile before the test
+        if not self.config.get("conditioned_profile").startswith("artifact:"):
+            return self.build_conditioned_profile()
+
         # create a temp file to help ensure uniqueness
         temp_download_dir = self._get_temp_dir()
         LOG.info(
@@ -261,20 +353,16 @@ either Raptor or browsertime."""
         alternate_repo = "mozilla-central" if repo != "mozilla-central" else "try"
         LOG.info("Getting profile from project %s" % repo)
 
-        profile_scenario = self.config.get("conditioned_profile_scenario", "settled")
+        profile_scenario = self.config.get("conditioned_profile").replace(
+            "artifact:", ""
+        )
         try:
             cond_prof_target_dir = get_profile(
-                temp_download_dir,
-                platform,
-                profile_scenario,
-                repo=repo
+                temp_download_dir, platform, profile_scenario, repo=repo
             )
         except ProfileNotFoundError:
             cond_prof_target_dir = get_profile(
-                temp_download_dir,
-                platform,
-                profile_scenario,
-                repo=alternate_repo
+                temp_download_dir, platform, profile_scenario, repo=alternate_repo
             )
         except Exception:
             # any other error is a showstopper
@@ -290,10 +378,7 @@ either Raptor or browsertime."""
             LOG.critical(
                 "Can't find target_dir {}, from get_profile()"
                 "temp_download_dir {}, platform {}, scenario {}".format(
-                    cond_prof_target_dir,
-                    temp_download_dir,
-                    platform,
-                    profile_scenario
+                    cond_prof_target_dir, temp_download_dir, platform, profile_scenario
                 )
             )
             raise OSError
@@ -306,7 +391,13 @@ either Raptor or browsertime."""
         return self.conditioned_profile_copy
 
     def build_browser_profile(self):
-        if not self.using_condprof or self.config['app'] in ['chrome', 'chromium', 'chrome-m']:
+        if self.config["app"] in ["safari"]:
+            self.profile = None
+            return
+        elif (
+            self.config["app"] in ["chrome", "chromium", "chrome-m", "custom-car"]
+            or self.config.get("conditioned_profile") is None
+        ):
             self.profile = create_profile(self.profile_class)
         else:
             # use mozprofile to create a profile for us, from our conditioned profile's path
@@ -322,9 +413,7 @@ either Raptor or browsertime."""
             LOG.info("Merging profile: {}".format(path))
             self.profile.merge(path)
 
-        if self.config["extra_prefs"].get("fission.autostart", False):
-            LOG.info("Enabling fission via browser preferences")
-            LOG.info("Browser preferences: {}".format(self.config["extra_prefs"]))
+        LOG.info("Browser preferences: {}".format(self.config["extra_prefs"]))
         self.profile.set_preferences(self.config["extra_prefs"])
 
         # share the profile dir with the config and the control server
@@ -368,7 +457,7 @@ either Raptor or browsertime."""
         if test.get("playback") is not None and self.playback is None:
             self.start_playback(test)
 
-        if test.get("preferences") is not None:
+        if test.get("preferences") is not None and self.config["app"] not in "safari":
             self.set_browser_test_prefs(test["preferences"])
 
     @abstractmethod
@@ -380,8 +469,11 @@ either Raptor or browsertime."""
         pass
 
     def run_tests(self, tests, test_names):
+        tests_to_run = tests
+        if self.results_handler.existing_results:
+            tests_to_run = []
         try:
-            for test in tests:
+            for test in tests_to_run:
                 try:
                     self.run_test(test, timeout=int(test.get("page_timeout")))
                 except RuntimeError as e:
@@ -404,13 +496,6 @@ either Raptor or browsertime."""
     def run_test_teardown(self, test):
         self.check_for_crashes()
 
-        # gecko profiling symbolication
-        if self.config["gecko_profile"]:
-            self.gecko_profiler.symbolicate()
-            # clean up the temp gecko profiling folders
-            LOG.info("cleaning up after gecko profiling")
-            self.gecko_profiler.clean()
-
     def process_results(self, tests, test_names):
         # when running locally output results in build/raptor.json; when running
         # in production output to a local.json to be turned into tc job artifact
@@ -420,7 +505,20 @@ either Raptor or browsertime."""
 
         self.config["raptor_json_path"] = raptor_json_path
         self.config["artifact_dir"] = self.artifact_dir
-        return self.results_handler.summarize_and_output(self.config, tests, test_names)
+        self.config["page_count"] = self.page_count
+        res = self.results_handler.summarize_and_output(self.config, tests, test_names)
+
+        # Gecko profiling symbolication
+        # We enable the gecko profiler either when the profiler is enabled with
+        # gecko_profile flag form the command line or when an extra profiler-enabled
+        # run is added with extra_profiler_run flag.
+        if self.config["gecko_profile"] or self.config.get("extra_profiler_run"):
+            self.gecko_profiler.symbolicate()
+            # clean up the temp gecko profiling folders
+            LOG.info("cleaning up after gecko profiling")
+            self.gecko_profiler.clean()
+
+        return res
 
     @abstractmethod
     def set_browser_test_prefs(self):
@@ -431,6 +529,7 @@ either Raptor or browsertime."""
         pass
 
     def clean_up(self):
+        # Cleanup all of our temporary directories
         for dir_to_rm in self._dirs_to_remove:
             if not os.path.exists(dir_to_rm):
                 continue
@@ -438,71 +537,25 @@ either Raptor or browsertime."""
             shutil.rmtree(dir_to_rm, ignore_errors=True)
         self._dirs_to_remove = []
 
+        # Go through the artifact directory and ensure we
+        # don't have too many JPG/PNG files from a task failure
+        if (
+            not self.run_local
+            and self.results_handler
+            and self.results_handler.result_dir()
+        ):
+            artifact_dir = pathlib.Path(self.artifact_dir)
+            for filetype in ("*.png", "*.jpg"):
+                # Limit the number of images uploaded to the last (newest) 5
+                for file in sorted(artifact_dir.rglob(filetype))[:-5]:
+                    try:
+                        file.unlink()
+                    except FileNotFoundError:
+                        pass
+
     def get_page_timeout_list(self):
         return self.results_handler.page_timeout_list
 
-    def get_recording_paths(self, test):
-        recordings = test.get("playback_recordings")
-
-        if recordings:
-            recording_paths = []
-            proxy_dir = self.playback.mozproxy_dir
-
-            for recording in recordings.split():
-                if not recording:
-                    continue
-                recording_paths.append(os.path.join(proxy_dir, recording))
-
-            return recording_paths
-
-    def log_recording_dates(self, test):
-        _recording_paths = self.get_recording_paths(test)
-        if _recording_paths is None:
-            LOG.info(
-                "No playback recordings specified in the test; so not getting recording info"
-            )
-            return
-
-        for r in _recording_paths:
-            json_path = "{}.json".format(r.split(".")[0])
-
-            if os.path.exists(json_path):
-                with open(json_path) as f:
-                    recording_date = json.loads(f.read()).get("recording_date")
-
-                    if recording_date is not None:
-                        LOG.info(
-                            "Playback recording date: {} ".format(
-                                recording_date.split(" ")[0]
-                            )
-                        )
-                    else:
-                        LOG.info("Playback recording date not available")
-            else:
-                LOG.info("Playback recording information not available")
-
-    def get_playback_config(self, test):
-        platform = self.config["platform"]
-        playback_dir = os.path.join(here, "tooltool-manifests", "playback")
-
-        self.config.update(
-            {
-                "playback_tool": test.get("playback"),
-                "playback_version": test.get("playback_version", "4.0.4"),
-                "playback_binary_zip": test.get("playback_binary_zip_%s" % platform),
-                "playback_pageset_zip": test.get("playback_pageset_zip_%s" % platform),
-                "playback_binary_manifest": test.get("playback_binary_manifest"),
-                "playback_pageset_manifest": test.get("playback_pageset_manifest"),
-            }
-        )
-
-        for key in ("playback_pageset_manifest", "playback_pageset_zip"):
-            if self.config.get(key) is None:
-                continue
-            self.config[key] = os.path.join(playback_dir, self.config[key])
-
-        LOG.info("test uses playback tool: %s " % self.config["playback_tool"])
-
     def delete_proxy_settings_from_profile(self):
         # Must delete the proxy settings from the profile if running
         # the test with a host different from localhost.
@@ -515,16 +568,26 @@ either Raptor or browsertime."""
 
     def start_playback(self, test):
         # creating the playback tool
-        self.get_playback_config(test)
-        self.playback = get_playback(self.config)
 
-        self.playback.config["playback_files"] = self.get_recording_paths(test)
+        playback_dir = os.path.join(here, "tooltool-manifests", "playback")
+
+        self.config.update(
+            {
+                "playback_tool": test.get("playback"),
+                "playback_version": test.get("playback_version", "8.1.1"),
+                "playback_files": [
+                    os.path.join(playback_dir, test.get("playback_pageset_manifest"))
+                ],
+            }
+        )
+
+        LOG.info("test uses playback tool: %s " % self.config["playback_tool"])
+
+        self.playback = get_playback(self.config)
 
         # let's start it!
         self.playback.start()
 
-        self.log_recording_dates(test)
-
     def _init_gecko_profiling(self, test):
         LOG.info("initializing gecko profiler")
         upload_dir = os.getenv("MOZ_UPLOAD_DIR")
@@ -533,6 +596,18 @@ either Raptor or browsertime."""
         else:
             self.gecko_profiler = GeckoProfile(upload_dir, self.config, test)
 
+    def disable_non_local_connections(self):
+        # For Firefox we need to set MOZ_DISABLE_NONLOCAL_CONNECTIONS=1 env var before startup
+        # when testing release builds from mozilla-beta/release. This is because of restrictions
+        # on release builds that require webextensions to be signed unless this env var is set
+        LOG.info("setting MOZ_DISABLE_NONLOCAL_CONNECTIONS=1")
+        os.environ["MOZ_DISABLE_NONLOCAL_CONNECTIONS"] = "1"
+
+    def enable_non_local_connections(self):
+        # pageload tests need to be able to access non-local connections via mitmproxy
+        LOG.info("setting MOZ_DISABLE_NONLOCAL_CONNECTIONS=0")
+        os.environ["MOZ_DISABLE_NONLOCAL_CONNECTIONS"] = "0"
+
 
 class PerftestAndroid(Perftest):
     """Mixin class for Android-specific Perftest subclasses."""
@@ -567,13 +642,18 @@ class PerftestAndroid(Perftest):
                     % (e.__class__.__name__, e)
                 )
 
-        if self.config["app"] == "chrome-m":
+        if self.config["app"] == "chrome-m" or browser_version is None:
             # We absolutely need to determine the chrome
             # version here so that we can select the correct
             # chromedriver for browsertime
-            from mozdevice import ADBDevice
-            device = ADBDevice(verbose=True)
+            from mozdevice import ADBDeviceFactory
+
+            device = ADBDeviceFactory(verbose=True)
+
+            # Chrome uses a specific binary that we don't set as a command line option
             binary = "com.android.chrome"
+            if self.config["app"] not in ("chrome-m",):
+                binary = self.config["binary"]
 
             pkg_info = device.shell_output("dumpsys package %s" % binary)
             version_matcher = re.compile(r".*versionName=([\d.]+)")
@@ -587,7 +667,9 @@ class PerftestAndroid(Perftest):
                     break
 
             if not browser_version:
-                raise Exception("Could not determine version for Google Chrome for Android")
+                raise Exception(
+                    "Could not determine version for Google Chrome for Android"
+                )
 
         if not browser_name:
             LOG.warning("Could not find a browser name")
@@ -650,12 +732,9 @@ class PerftestAndroid(Perftest):
 
         try:
             LOG.info("copying profile to device: %s" % self.remote_profile)
-            # We must use root=True since the remote profile has been
-            # modified by gecko and has content which is only
-            # accessible to the gecko user.
-            self.device.rm(self.remote_profile, force=True, recursive=True, root=True)
+            self.device.rm(self.remote_profile, force=True, recursive=True)
             self.device.push(self.profile.profile, self.remote_profile)
-            self.device.chmod(self.remote_profile, recursive=True, root=True)
+            self.device.chmod(self.remote_profile, recursive=True)
 
         except Exception:
             LOG.error("Unable to copy profile to device.")
@@ -684,6 +763,9 @@ class PerftestAndroid(Perftest):
 class PerftestDesktop(Perftest):
     """Mixin class for Desktop-specific Perftest subclasses"""
 
+    def __init__(self, *args, **kwargs):
+        super(PerftestDesktop, self).__init__(*args, **kwargs)
+
     def setup_chrome_args(self, test):
         """Sets up chrome/chromium cmd-line arguments.
 
@@ -739,10 +821,20 @@ class PerftestDesktop(Perftest):
 
             # Fall-back method to get browser version on desktop
             try:
-                if (
-                    "linux" in self.config["platform"]
-                    or "mac" in self.config["platform"]
-                ):
+                if "mac" in self.config["platform"]:
+                    import plistlib
+
+                    for plist_file in ("version.plist", "Info.plist"):
+                        try:
+                            binary_path = pathlib.Path(self.config["binary"])
+                            plist_path = binary_path.parent.parent.joinpath(plist_file)
+                            with plist_path.open("rb") as plist:
+                                plist = plistlib.load(plist)
+                        except FileNotFoundError:
+                            pass
+                    browser_name = self.config["app"]
+                    browser_version = plist.get("CFBundleShortVersionString")
+                elif "linux" in self.config["platform"]:
                     command = [self.config["binary"], "--version"]
                     proc = mozprocess.ProcessHandler(command)
                     proc.run(timeout=10, outputTimeout=10)
@@ -751,7 +843,7 @@ class PerftestDesktop(Perftest):
                     bmeta = proc.output
                     meta_re = re.compile(r"([A-z\s]+)\s+([\w.]*)")
                     if len(bmeta) != 0:
-                        match = meta_re.match(bmeta[0])
+                        match = meta_re.match(bmeta[0].decode("utf-8"))
                         if match:
                             browser_name = self.config["app"]
                             browser_version = match.group(2)
@@ -765,7 +857,7 @@ class PerftestDesktop(Perftest):
                     bmeta = subprocess.check_output(command)
 
                     meta_re = re.compile(r"\s+([\d.a-z]+)\s+")
-                    match = meta_re.findall(bmeta)
+                    match = meta_re.findall(bmeta.decode("utf-8"))
                     if len(match) > 0:
                         browser_name = self.config["app"]
                         browser_version = match[-1]
