@@ -4,23 +4,38 @@
 
 # class to process, format, and report raptor test results
 # received from the raptor control server
-from __future__ import absolute_import
-
 import json
 import os
-
+import pathlib
+import shutil
 from abc import ABCMeta, abstractmethod
+from collections.abc import Iterable
+from io import open
+
+import six
 from logger.logger import RaptorLogger
-from output import RaptorOutput, BrowsertimeOutput
+from output import BrowsertimeOutput, RaptorOutput
+from utils import flatten
 
 LOG = RaptorLogger(component="perftest-results-handler")
+KNOWN_TEST_MODIFIERS = [
+    "condprof-settled",
+    "fission",
+    "live",
+    "gecko-profile",
+    "cold",
+    "webrender",
+    "bytecode-cached",
+]
+NON_FIREFOX_OPTS = ("webrender", "bytecode-cached", "fission")
+NON_FIREFOX_BROWSERS = ("chrome", "chromium", "custom-car", "safari")
+NON_FIREFOX_BROWSERS_MOBILE = ("chrome-m",)
 
 
+@six.add_metaclass(ABCMeta)
 class PerftestResultsHandler(object):
     """Abstract base class to handle perftest results"""
 
-    __metaclass__ = ABCMeta
-
     def __init__(
         self,
         gecko_profile=False,
@@ -29,7 +44,13 @@ class PerftestResultsHandler(object):
         memory_test=False,
         live_sites=False,
         app=None,
-        no_conditioned_profile=False,
+        conditioned_profile=None,
+        cold=False,
+        chimera=False,
+        fission=True,
+        perfstats=False,
+        test_bytecode_cache=False,
+        extra_summary_methods=[],
         **kwargs
     ):
         self.gecko_profile = gecko_profile
@@ -38,21 +59,77 @@ class PerftestResultsHandler(object):
         self.memory_test = memory_test
         self.live_sites = live_sites
         self.app = app
+        self.conditioned_profile = conditioned_profile
         self.results = []
         self.page_timeout_list = []
         self.images = []
         self.supporting_data = None
-        self.fission_enabled = kwargs.get("extra_prefs", {}).get(
-            "fission.autostart", False
-        )
+        self.fission_enabled = fission
         self.browser_version = None
         self.browser_name = None
-        self.no_conditioned_profile = no_conditioned_profile
+        self.cold = cold
+        self.chimera = chimera
+        self.perfstats = perfstats
+        self.test_bytecode_cache = test_bytecode_cache
+        self.existing_results = None
+        self.extra_summary_methods = extra_summary_methods
 
     @abstractmethod
     def add(self, new_result_json):
         raise NotImplementedError()
 
+    def result_dir(self):
+        return None
+
+    def build_extra_options(self, modifiers=None):
+        extra_options = []
+
+        # If fields is not None, then we default to
+        # checking all known fields. Otherwise, we only check
+        # the fields that were given to us.
+        if modifiers is None:
+            if self.conditioned_profile:
+                # Don't add an option/tag for the base test case
+                if self.conditioned_profile != "settled":
+                    extra_options.append(
+                        "condprof-%s"
+                        % self.conditioned_profile.replace("artifact:", "")
+                    )
+            if self.fission_enabled:
+                extra_options.append("fission")
+            if self.live_sites:
+                extra_options.append("live")
+            if self.gecko_profile:
+                extra_options.append("gecko-profile")
+            if self.cold:
+                extra_options.append("cold")
+            if self.test_bytecode_cache:
+                extra_options.append("bytecode-cached")
+            extra_options.append("webrender")
+        else:
+            for modifier, name in modifiers:
+                if not modifier:
+                    continue
+                if name in KNOWN_TEST_MODIFIERS:
+                    extra_options.append(name)
+                else:
+                    raise Exception(
+                        "Unknown test modifier %s was provided as an extra option"
+                        % name
+                    )
+
+        # Bug 1770225: Make this more dynamic, this will fail us again in the future
+        self._clean_up_browser_options(extra_options=extra_options)
+
+        return extra_options
+
+    def _clean_up_browser_options(self, extra_options):
+        """Remove certain firefox specific options from different browsers"""
+        if self.app.lower() in NON_FIREFOX_BROWSERS + NON_FIREFOX_BROWSERS_MOBILE:
+            for opts in NON_FIREFOX_OPTS:
+                if opts in extra_options:
+                    extra_options.remove(opts)
+
     def add_browser_meta(self, browser_name, browser_version):
         # sets the browser metadata for the perfherder data
         self.browser_name = browser_name
@@ -78,7 +155,7 @@ class PerftestResultsHandler(object):
         self.page_timeout_list.append(timeout_details)
 
     def add_supporting_data(self, supporting_data):
-        """ Supporting data is additional data gathered outside of the regular
+        """Supporting data is additional data gathered outside of the regular
         Raptor test run (i.e. power data). Will arrive in a dict in the format of:
 
         supporting_data = {'type': 'data-type',
@@ -107,6 +184,9 @@ class PerftestResultsHandler(object):
             self.supporting_data = []
         self.supporting_data.append(supporting_data)
 
+    def use_existing_results(self, directory):
+        self.existing_results = directory
+
     def _get_expected_perfherder(self, output):
         def is_resource_test():
             if self.power_test or self.cpu_test or self.memory_test:
@@ -132,10 +212,6 @@ class PerftestResultsHandler(object):
 
         expected_perfherder = 1
 
-        if output.mozproxy_data:
-            # Check if we have mozproxy data available.
-            expected_perfherder += 1
-
         if is_resource_test():
             # when resource tests are run, no perfherder data is output
             # for the regular raptor tests (i.e. speedometer) so we
@@ -174,7 +250,7 @@ class PerftestResultsHandler(object):
         )
         LOG.info("Validating PERFHERDER_DATA against %s" % schema_path)
         try:
-            with open(schema_path) as f:
+            with open(schema_path, encoding="utf-8") as f:
                 schema = json.load(f)
             if output.summarized_results:
                 data = output.summarized_results
@@ -196,21 +272,31 @@ class RaptorResultsHandler(PerftestResultsHandler):
     """Process Raptor results"""
 
     def add(self, new_result_json):
-        # add to results
-        if new_result_json.get("extra_options") is None:
-            new_result_json["extra_options"] = []
         LOG.info("received results in RaptorResultsHandler.add")
-        if self.no_conditioned_profile:
-            new_result_json["extra_options"].append("nocondprof")
-        if self.fission_enabled:
-            new_result_json["extra_options"].append("fission")
+        new_result_json.setdefault("extra_options", []).extend(
+            self.build_extra_options(
+                [
+                    (
+                        self.conditioned_profile,
+                        "condprof-%s" % self.conditioned_profile,
+                    ),
+                    (self.fission_enabled, "fission"),
+                ]
+            )
+        )
+        if self.live_sites:
+            new_result_json.setdefault("tags", []).append("live")
+            new_result_json["extra_options"].append("live")
         self.results.append(new_result_json)
 
     def summarize_and_output(self, test_config, tests, test_names):
         # summarize the result data, write to file and output PERFHERDER_DATA
         LOG.info("summarizing raptor test results")
         output = RaptorOutput(
-            self.results, self.supporting_data, test_config["subtest_alert_on"]
+            self.results,
+            self.supporting_data,
+            test_config.get("subtest_alert_on", []),
+            self.app,
         )
         output.set_browser_meta(self.browser_name, self.browser_version)
         output.summarize(test_names)
@@ -234,7 +320,9 @@ class RaptorResultsHandler(PerftestResultsHandler):
                 output, out_sup_perfdata + out_perfdata
             )
 
-        return sup_success and success and validate_success and not self.page_timeout_list
+        return (
+            sup_success and success and validate_success and not self.page_timeout_list
+        )
 
 
 class BrowsertimeResultsHandler(PerftestResultsHandler):
@@ -243,19 +331,64 @@ class BrowsertimeResultsHandler(PerftestResultsHandler):
     def __init__(self, config, root_results_dir=None):
         super(BrowsertimeResultsHandler, self).__init__(**config)
         self._root_results_dir = root_results_dir
+        self.browsertime_visualmetrics = False
+        self.failed_vismets = []
+        if not os.path.exists(self._root_results_dir):
+            os.mkdir(self._root_results_dir)
 
     def result_dir(self):
         return self._root_results_dir
 
     def result_dir_for_test(self, test):
-        return os.path.join(self._root_results_dir, test["name"])
+        if self.existing_results is None:
+            results_root = self._root_results_dir
+        else:
+            results_root = self.existing_results
+        return os.path.join(results_root, test["name"])
+
+    def remove_result_dir_for_test(self, test):
+        test_result_dir = self.result_dir_for_test(test)
+        if os.path.exists(test_result_dir):
+            shutil.rmtree(test_result_dir)
+        return test_result_dir
+
+    def result_dir_for_test_profiling(self, test):
+        profiling_dir = os.path.join(self.result_dir_for_test(test), "profiling")
+        if not os.path.exists(profiling_dir):
+            os.mkdir(profiling_dir)
+        return profiling_dir
 
     def add(self, new_result_json):
         # not using control server with bt
         pass
 
+    def build_tags(self, test={}):
+        """Use to build the tags option for our perfherder data.
+
+        This should only contain items that will only be shown within
+        the tags section and excluded from the extra options.
+        """
+        tags = []
+        LOG.info(test)
+        if test.get("interactive", False):
+            tags.append("interactive")
+        return tags
+
     def parse_browsertime_json(
-        self, raw_btresults, page_cycles, cold, browser_cycles, measure
+        self,
+        raw_btresults,
+        page_cycles,
+        cold,
+        browser_cycles,
+        measure,
+        page_count,
+        test_name,
+        accept_zero_vismet,
+        load_existing,
+        test_summary,
+        subtest_name_filters,
+        handle_custom_data,
+        **kwargs
     ):
         """
         Receive a json blob that contains the results direct from the browsertime tool. Parse
@@ -339,7 +472,17 @@ class BrowsertimeResultsHandler(PerftestResultsHandler):
                   "max": 864
                 },
               }
-            }
+            },
+            "geckoPerfStats": [
+              {
+                "Compositing": 71,
+                "MajorGC": 144
+              },
+              {
+                "Compositing": 13,
+                "MajorGC": 126
+              }
+            ]
           }
         ]
 
@@ -367,19 +510,14 @@ class BrowsertimeResultsHandler(PerftestResultsHandler):
            }
         """
         LOG.info("parsing results from browsertime json")
-
         # bt to raptor names
         conversion = (
             ("fnbpaint", "firstPaint"),
-            ("fcp", "timeToContentfulPaint"),
+            ("fcp", ["paintTiming", "first-contentful-paint"]),
             ("dcf", "timeToDomContentFlushed"),
             ("loadtime", "loadEventEnd"),
         )
 
-        chrome_raptor_conversion = {
-            "timeToContentfulPaint": ["paintTiming", "first-contentful-paint"]
-        }
-
         def _get_raptor_val(mdict, mname, retval=False):
             # gets the measurement requested, returns the value
             # if one was found, or retval if it couldn't be found
@@ -417,6 +555,8 @@ class BrowsertimeResultsHandler(PerftestResultsHandler):
                 raise MissingResultsError(
                     "Missing results for all cold browser cycles."
                 )
+        elif load_existing:
+            pass  # Use whatever is there.
         else:
             if len(raw_btresults) != int(page_cycles):
                 raise MissingResultsError(
@@ -424,6 +564,7 @@ class BrowsertimeResultsHandler(PerftestResultsHandler):
                 )
 
         # now parse out the values
+        page_counter = 0
         for raw_result in raw_btresults:
             if not raw_result["browserScripts"]:
                 raise MissingResultsError("Browsertime cycle produced no measurements.")
@@ -434,43 +575,119 @@ class BrowsertimeResultsHandler(PerftestResultsHandler):
             # Desktop chrome doesn't have `browser` scripts data available for now
             bt_browser = raw_result["browserScripts"][0].get("browser", None)
             bt_ver = raw_result["info"]["browsertime"]["version"]
-            bt_url = (raw_result["info"]["url"],)
+
+            # when doing actions, we append a .X for each additional pageload in a scenario
+            extra = ""
+            if len(page_count) > 0:
+                extra = ".%s" % page_count[page_counter % len(page_count)]
+            url_parts = raw_result["info"]["url"].split("/")
+            page_counter += 1
+
+            bt_url = "%s%s/%s," % ("/".join(url_parts[:-1]), extra, url_parts[-1])
             bt_result = {
                 "bt_ver": bt_ver,
                 "browser": bt_browser,
-                "url": bt_url,
+                "url": (bt_url,),
+                "name": "%s%s" % (test_name, extra),
                 "measurements": {},
                 "statistics": {},
             }
 
-            custom_types = raw_result["browserScripts"][0].get("custom")
+            def _extract_cpu_vals():
+                # Bug 1806402 - Handle chrome cpu data properly
+                cpu_vals = raw_result.get("cpu", None)
+                if (
+                    cpu_vals
+                    and self.app
+                    not in NON_FIREFOX_BROWSERS + NON_FIREFOX_BROWSERS_MOBILE
+                ):
+                    bt_result["measurements"].setdefault("cpuTime", []).extend(cpu_vals)
+
+            if self.power_test:
+                power_result = {
+                    "bt_ver": bt_ver,
+                    "browser": bt_browser,
+                    "url": bt_url,
+                    "measurements": {},
+                    "statistics": {},
+                    "power_data": True,
+                }
+
+                for cycle in raw_result["android"]["power"]:
+                    for metric in cycle:
+                        if "total" in metric:
+                            continue
+                        power_result["measurements"].setdefault(metric, []).append(
+                            cycle[metric]
+                        )
+                power_result["statistics"] = raw_result["statistics"]["android"][
+                    "power"
+                ]
+                results.append(power_result)
+
+            custom_types = raw_result["extras"][0]
             if custom_types:
                 for custom_type in custom_types:
-                    bt_result["measurements"].update(
-                        {k: [v] for k, v in custom_types[custom_type].items()}
-                    )
+                    data = custom_types[custom_type]
+                    if handle_custom_data:
+                        if test_summary in ("flatten",):
+                            data = flatten(data, ())
+                        for k, v in data.items():
+
+                            def _ignore_metric(*args):
+                                if any(type(arg) not in (int, float) for arg in args):
+                                    return True
+                                return False
+
+                            # Ignore any non-numerical results
+                            if _ignore_metric(v) and _ignore_metric(*v):
+                                continue
+
+                            # Clean up the name if requested
+                            for name_filter in subtest_name_filters.split(","):
+                                k = k.replace(name_filter, "")
+
+                            if isinstance(v, Iterable):
+                                bt_result["measurements"].setdefault(k, []).extend(v)
+                            else:
+                                bt_result["measurements"].setdefault(k, []).append(v)
+                        bt_result["custom_data"] = True
+                    else:
+                        for k, v in data.items():
+                            bt_result["measurements"].setdefault(k, []).append(v)
+                if self.perfstats:
+                    for cycle in raw_result["geckoPerfStats"]:
+                        for metric in cycle:
+                            bt_result["measurements"].setdefault(
+                                "perfstat-" + metric, []
+                            ).append(cycle[metric])
+                if kwargs.get("gather_cpuTime", None):
+                    _extract_cpu_vals()
             else:
                 # extracting values from browserScripts and statistics
                 for bt, raptor in conversion:
                     if measure is not None and bt not in measure:
                         continue
-                    # chrome we just measure fcp and loadtime; skip fnbpaint and dcf
+                    # chrome and safari we just measure fcp and loadtime; skip fnbpaint and dcf
                     if (
                         self.app
-                        and "chrome" in self.app.lower()
-                        and bt in ("fnbpaint", "dcf")
+                        and self.app.lower()
+                        in NON_FIREFOX_BROWSERS + NON_FIREFOX_BROWSERS_MOBILE
+                        and bt
+                        in (
+                            "fnbpaint",
+                            "dcf",
+                        )
                     ):
                         continue
-                    # fennec doesn't support 'fcp'
-                    if self.app and "fennec" in self.app.lower() and bt == "fcp":
-                        continue
 
-                    # chrome currently uses different names (and locations) for some metrics
-                    if raptor in chrome_raptor_conversion and _get_raptor_val(
+                    # FCP uses a different path to get the timing, so we need to do
+                    # some checks here
+                    if bt == "fcp" and not _get_raptor_val(
                         raw_result["browserScripts"][0]["timings"],
-                        chrome_raptor_conversion[raptor],
+                        raptor,
                     ):
-                        raptor = chrome_raptor_conversion[raptor]
+                        continue
 
                     # XXX looping several times in the list, could do better
                     for cycle in raw_result["browserScripts"]:
@@ -491,11 +708,49 @@ class BrowsertimeResultsHandler(PerftestResultsHandler):
                         raw_result["statistics"]["timings"], raptor, retval={}
                     )
 
+                _extract_cpu_vals()
+
+                if self.perfstats:
+                    for cycle in raw_result["geckoPerfStats"]:
+                        for metric in cycle:
+                            bt_result["measurements"].setdefault(
+                                "perfstat-" + metric, []
+                            ).append(cycle[metric])
+
+                if self.browsertime_visualmetrics:
+                    for cycle in raw_result["visualMetrics"]:
+                        for metric in cycle:
+                            if "progress" in metric.lower():
+                                # Bug 1665750 - Determine if we should display progress
+                                continue
+
+                            if metric not in measure:
+                                continue
+
+                            val = cycle[metric]
+                            if not accept_zero_vismet:
+                                if val == 0:
+                                    self.failed_vismets.append(metric)
+                                    continue
+
+                            bt_result["measurements"].setdefault(metric, []).append(val)
+                            bt_result["statistics"][metric] = raw_result["statistics"][
+                                "visualMetrics"
+                            ][metric]
+
             results.append(bt_result)
 
         return results
 
-    def _extract_vmetrics(self, test_name, browsertime_json):
+    def _extract_vmetrics(
+        self,
+        test_name,
+        browsertime_json,
+        json_name="browsertime.json",
+        tags=[],
+        extra_options=[],
+        accept_zero_vismet=False,
+    ):
         # The visual metrics task expects posix paths.
         def _normalized_join(*args):
             path = os.path.join(*args)
@@ -505,10 +760,31 @@ class BrowsertimeResultsHandler(PerftestResultsHandler):
         reldir = _normalized_join("browsertime-results", name)
 
         return {
-            "browsertime_json_path": _normalized_join(reldir, "browsertime.json"),
+            "browsertime_json_path": _normalized_join(reldir, json_name),
             "test_name": test_name,
+            "tags": tags,
+            "extra_options": extra_options,
+            "accept_zero_vismet": accept_zero_vismet,
         }
 
+    def _label_video_folder(self, result_data, base_dir, kind="warm"):
+        for filetype in result_data["files"]:
+            for idx, data in enumerate(result_data["files"][filetype]):
+                parts = list(pathlib.Path(data).parts)
+                lable_idx = parts.index("data")
+                if "query-" in parts[lable_idx - 1]:
+                    lable_idx -= 1
+
+                src_dir = pathlib.Path(base_dir).joinpath(*parts[: lable_idx + 1])
+                parts.insert(lable_idx, kind)
+                dst_dir = pathlib.Path(base_dir).joinpath(*parts[: lable_idx + 1])
+
+                if src_dir.exists() and not dst_dir.exists():
+                    pathlib.Path(dst_dir).mkdir(parents=True, exist_ok=True)
+                    shutil.move(str(src_dir), str(dst_dir))
+
+                result_data["files"][filetype][idx] = str(pathlib.Path(*parts[:]))
+
     def summarize_and_output(self, test_config, tests, test_names):
         """
         Retrieve, process, and output the browsertime test results. Currently supports page-load
@@ -542,6 +818,8 @@ class BrowsertimeResultsHandler(PerftestResultsHandler):
 
         for test in tests:
             test_name = test["name"]
+            accept_zero_vismet = test.get("accept_zero_vismet", False)
+
             bt_res_json = os.path.join(
                 self.result_dir_for_test(test), "browsertime.json"
             )
@@ -552,7 +830,7 @@ class BrowsertimeResultsHandler(PerftestResultsHandler):
                 return False
 
             try:
-                with open(bt_res_json, "r") as f:
+                with open(bt_res_json, "r", encoding="utf8") as f:
                     raw_btresults = json.load(f)
             except Exception as e:
                 LOG.error("Exception reading %s" % bt_res_json)
@@ -560,8 +838,109 @@ class BrowsertimeResultsHandler(PerftestResultsHandler):
                 LOG.error("Exception: %s %s" % (type(e).__name__, str(e)))
                 raise
 
+            # Split the chimera videos here for local testing
+            def split_browsertime_results(result_json_path, raw_btresults):
+                # First result is cold, second is warm
+                cold_data = raw_btresults[0]
+                warm_data = raw_btresults[1]
+
+                dirpath = os.path.dirname(os.path.abspath(result_json_path))
+                _cold_path = os.path.join(dirpath, "cold-browsertime.json")
+                _warm_path = os.path.join(dirpath, "warm-browsertime.json")
+
+                self._label_video_folder(cold_data, dirpath, "cold")
+                self._label_video_folder(warm_data, dirpath, "warm")
+
+                with open(_cold_path, "w") as f:
+                    json.dump([cold_data], f)
+                with open(_warm_path, "w") as f:
+                    json.dump([warm_data], f)
+
+                raw_btresults[0] = cold_data
+                raw_btresults[1] = warm_data
+
+                return _cold_path, _warm_path
+
+            cold_path = None
+            warm_path = None
+            if self.chimera:
+                cold_path, warm_path = split_browsertime_results(
+                    bt_res_json, raw_btresults
+                )
+
+                # Overwrite the contents of the browsertime.json file
+                # to update it with the new file paths
+                try:
+                    with open(bt_res_json, "w", encoding="utf8") as f:
+                        json.dump(raw_btresults, f)
+                except Exception as e:
+                    LOG.error("Exception reading %s" % bt_res_json)
+                    # XXX this should be replaced by a traceback call
+                    LOG.error("Exception: %s %s" % (type(e).__name__, str(e)))
+                    raise
+
+                # If extra profiler run is enabled, split its browsertime.json
+                # file to cold and warm json files as well.
+                bt_profiling_res_json = os.path.join(
+                    self.result_dir_for_test_profiling(test), "browsertime.json"
+                )
+                has_extra_profiler_run = test_config.get(
+                    "extra_profiler_run", False
+                ) and os.path.exists(bt_profiling_res_json)
+                if has_extra_profiler_run:
+                    try:
+                        with open(bt_profiling_res_json, "r", encoding="utf8") as f:
+                            raw_profiling_btresults = json.load(f)
+                            split_browsertime_results(
+                                bt_profiling_res_json, raw_profiling_btresults
+                            )
+                    except Exception as e:
+                        LOG.info(
+                            "Exception reading and writing %s" % bt_profiling_res_json
+                        )
+                        LOG.info("Exception: %s %s" % (type(e).__name__, str(e)))
+
             if not run_local:
-                video_jobs.append(self._extract_vmetrics(test_name, bt_res_json))
+                extra_options = self.build_extra_options()
+                tags = self.build_tags(test=test)
+
+                if self.chimera:
+                    if cold_path is None or warm_path is None:
+                        raise Exception("Cold and warm paths were not created")
+
+                    video_jobs.append(
+                        self._extract_vmetrics(
+                            test_name,
+                            cold_path,
+                            json_name="cold-browsertime.json",
+                            tags=list(tags),
+                            extra_options=list(extra_options),
+                            accept_zero_vismet=accept_zero_vismet,
+                        )
+                    )
+
+                    extra_options.remove("cold")
+                    extra_options.append("warm")
+                    video_jobs.append(
+                        self._extract_vmetrics(
+                            test_name,
+                            warm_path,
+                            json_name="warm-browsertime.json",
+                            tags=list(tags),
+                            extra_options=list(extra_options),
+                            accept_zero_vismet=accept_zero_vismet,
+                        )
+                    )
+                else:
+                    video_jobs.append(
+                        self._extract_vmetrics(
+                            test_name,
+                            bt_res_json,
+                            tags=list(tags),
+                            extra_options=list(extra_options),
+                            accept_zero_vismet=accept_zero_vismet,
+                        )
+                    )
 
             for new_result in self.parse_browsertime_json(
                 raw_btresults,
@@ -569,9 +948,17 @@ class BrowsertimeResultsHandler(PerftestResultsHandler):
                 test["cold"],
                 test["browser_cycles"],
                 test.get("measure"),
+                test_config.get("page_count", []),
+                test["name"],
+                accept_zero_vismet,
+                self.existing_results is not None,
+                test.get("test_summary", "pageload"),
+                test.get("subtest_name_filters", ""),
+                test.get("custom_data", False) == "true",
+                gather_cpuTime=test.get("gather_cpuTime", None),
             ):
 
-                def _new_pageload_result(new_result):
+                def _new_standard_result(new_result, subtest_unit="ms"):
                     # add additional info not from the browsertime json
                     for field in (
                         "name",
@@ -580,67 +967,88 @@ class BrowsertimeResultsHandler(PerftestResultsHandler):
                         "alert_threshold",
                         "cold",
                     ):
+                        if field in new_result:
+                            continue
                         new_result[field] = test[field]
 
-                    # Differentiate Raptor `pageload` tests from `browsertime-pageload`
-                    # tests while we compare and contrast.
-                    new_result["type"] = "browsertime-pageload"
+                    new_result["min_back_window"] = test.get("min_back_window", None)
+                    new_result["max_back_window"] = test.get("max_back_window", None)
+                    new_result["fore_window"] = test.get("fore_window", None)
 
                     # All Browsertime measurements are elapsed times in milliseconds.
-                    new_result["subtest_lower_is_better"] = True
-                    new_result["subtest_unit"] = "ms"
-                    LOG.info("parsed new result: %s" % str(new_result))
+                    new_result["subtest_lower_is_better"] = test.get(
+                        "subtest_lower_is_better", True
+                    )
+                    new_result["subtest_unit"] = subtest_unit
 
-                    new_result["extra_options"] = []
-                    if self.no_conditioned_profile:
-                        new_result["extra_options"].append("nocondprof")
-                    if self.fission_enabled:
-                        new_result["extra_options"].append("fission")
-                    if self.live_sites:
-                        new_result["extra_options"].append("live")
-                    if self.gecko_profile:
-                        new_result["extra_options"].append("gecko_profile")
+                    new_result["extra_options"] = self.build_extra_options()
+                    new_result.setdefault("tags", []).extend(self.build_tags(test=test))
 
+                    # Split the chimera
+                    if self.chimera and "run=2" in new_result["url"][0]:
+                        new_result["extra_options"].remove("cold")
+                        new_result["extra_options"].append("warm")
+
+                    return new_result
+
+                def _new_powertest_result(new_result):
+                    new_result["type"] = "power"
+                    new_result["unit"] = "mAh"
+                    new_result["lower_is_better"] = True
+
+                    new_result = _new_standard_result(new_result, subtest_unit="mAh")
+                    new_result["extra_options"].append("power")
+
+                    LOG.info("parsed new power result: %s" % str(new_result))
+                    return new_result
+
+                def _new_custom_result(new_result):
+                    new_result["type"] = "pageload"
+                    new_result = _new_standard_result(
+                        new_result, subtest_unit=test.get("subtest_unit", "ms")
+                    )
+
+                    LOG.info("parsed new custom result: %s" % str(new_result))
+                    return new_result
+
+                def _new_pageload_result(new_result):
+                    new_result["type"] = "pageload"
+                    new_result = _new_standard_result(new_result)
+
+                    LOG.info("parsed new pageload result: %s" % str(new_result))
                     return new_result
 
                 def _new_benchmark_result(new_result):
-                    # add additional info not from the browsertime json
-                    for field in (
-                        "name",
-                        "unit",
-                        "lower_is_better",
-                        "alert_threshold",
-                        "cold",
-                    ):
-                        new_result[field] = test[field]
-
-                    # Differentiate Raptor `pageload` tests from other `browsertime`
-                    # tests while we compare and contrast.
-                    new_result["type"] = "browsertime-%s" % test["type"]
-
-                    # Try to get subtest values or use the defaults
-                    # If values not available use the defaults
-                    subtest_lower_is_better = test.get("subtest_lower_is_better", True)
-                    new_result["subtest_lower_is_better"] = subtest_lower_is_better
-
-                    new_result["subtest_unit"] = test.get("subtest_unit", "ms")
-                    LOG.info("parsed new result: %s" % str(new_result))
-
-                    new_result["extra_options"] = []
-                    if self.app != "firefox":
-                        new_result["extra_options"].append(self.app)
-                    if self.gecko_profile:
-                        new_result["extra_options"].append("gecko_profile")
+                    new_result["type"] = "benchmark"
 
+                    new_result = _new_standard_result(
+                        new_result, subtest_unit=test.get("subtest_unit", "ms")
+                    )
+                    new_result["gather_cpuTime"] = test.get("gather_cpuTime", None)
+                    LOG.info("parsed new benchmark result: %s" % str(new_result))
                     return new_result
 
-                if test["type"] == "pageload":
-                    self.results.append(_new_pageload_result(new_result))
+                def _is_supporting_data(res):
+                    if res.get("power_data", False):
+                        return True
+                    return False
+
+                if new_result.get("power_data", False):
+                    self.results.append(_new_powertest_result(new_result))
+                elif test["type"] == "pageload":
+                    if test.get("custom_data", False) == "true":
+                        self.results.append(_new_custom_result(new_result))
+                    else:
+                        self.results.append(_new_pageload_result(new_result))
                 elif test["type"] == "benchmark":
                     for i, item in enumerate(self.results):
-                        if item["name"] == test["name"]:
+                        if item["name"] == test["name"] and not _is_supporting_data(
+                            item
+                        ):
                             # add page cycle custom measurements to the existing results
-                            for measurement in new_result["measurements"].iteritems():
+                            for measurement in six.iteritems(
+                                new_result["measurements"]
+                            ):
                                 self.results[i]["measurements"][measurement[0]].extend(
                                     measurement[1]
                                 )
@@ -650,12 +1058,22 @@ class BrowsertimeResultsHandler(PerftestResultsHandler):
 
         # now have all results gathered from all browsertime test URLs; format them for output
         output = BrowsertimeOutput(
-            self.results, self.supporting_data, test_config["subtest_alert_on"]
+            self.results,
+            self.supporting_data,
+            test_config.get("subtest_alert_on", []),
+            self.app,
+            self.extra_summary_methods,
         )
         output.set_browser_meta(self.browser_name, self.browser_version)
         output.summarize(test_names)
         success, out_perfdata = output.output(test_names)
 
+        if len(self.failed_vismets) > 0:
+            LOG.critical(
+                "TEST-UNEXPECTED-FAIL | Some visual metrics have an erroneous value of 0."
+            )
+            LOG.info("Visual metric tests failed: %s" % str(self.failed_vismets))
+
         validate_success = True
         if not self.gecko_profile:
             validate_success = self._validate_treeherder_data(output, out_perfdata)
@@ -666,7 +1084,7 @@ class BrowsertimeResultsHandler(PerftestResultsHandler):
             jobs_json = {
                 "jobs": video_jobs,
                 "application": {"name": self.browser_name},
-                "extra_options": output.summarized_results["suites"][0]["extraOptions"]
+                "extra_options": output.summarized_results["suites"][0]["extraOptions"],
             }
 
             if self.browser_version is not None:
@@ -681,7 +1099,7 @@ class BrowsertimeResultsHandler(PerftestResultsHandler):
             with open(jobs_file, "w") as f:
                 f.write(json.dumps(jobs_json))
 
-        return success and validate_success
+        return (success and validate_success) and len(self.failed_vismets) == 0
 
 
 class MissingResultsError(Exception):
